﻿;;;-*- Mode: Lisp; Package: "CSCWD-MEMBERS" -*-
;;;===============================================================================
;;;14/09/16
;;;             AGENT CSCWD-MEMBERS: ONTOLOGY (file CSCWD-MEMBERS-ontology.lisp)
;;;             Copyright Barthes@UTC, 2014
;;;
;;;=============================================================================== 

#|
Contains the list of members of the Technical Committee

2014
 0916 creation
|#

;;; useful if ontology is loaded directly in the MOSS package (without OMAS and agents)
(defpackage :CSCWD-MEMBERS (:use :CL :moss :omas))

(in-package :CSCWD-MEMBERS)

(defontology 
  (:title "CSCWD-MEMBERS ontology")
  (:version "1.0")
  (:language :all)
  )

;;;-------------------------------------------------------------------------------
;;;                               MEMBER
;;;-------------------------------------------------------------------------------	

(defconcept ((:en "Member") (:fr "Membre"))
  (:doc :en "A MEMBER here is a member of the CSCWD Tehnical Committee."
         :fr "Un MEMBRE est ici un membre du Comité Technique CSCWD.")
  (:is-a "Person")
  (:att ((:en "login")) (:unique)(:entry)) ; login name, e.g. "barthes" for "Barthès"
  (:att ((:en "password") (:fr "mot de passe")))
  )
  
;;;-------------------------------------------------------------------------------
;;;                             ORGANIZATION
;;;-------------------------------------------------------------------------------	

(defconcept (:en "organization" :fr "organisation")
  (:att ((:en "name") (:fr "nom"))(:min 1)(:entry))
  (:att ((:en "acronym") (:fr "sigle")) (:entry))
  (:att ((:en "first-name") (:fr "prénom"))(:entry))
  (:att ((:en "country") (:fr "pays")))
  (:doc (:en "An ORGANIZATION is a set of persons who work for a common goal.")
        (:fr "Une ORGANISATION est un ensemble de personnes aynt in but commun."))
  )
  
;;;-------------------------------------------------------------------------------
;;;                              PERSON
;;;-------------------------------------------------------------------------------	

(defconcept ((:en "person") (:fr "personne"))
  (:att ((:en "identifier") (:fr "identifiant"))(:entry))
  (:att ((:en "name") (:fr "nom"))(:min 1) (:entry))
  (:att ((:en "first-name") (:fr "prénom"))(:min 1)(:entry))
  (:att ((:en "access code") (:fr "code d'accès")))
  (:att ((:en "email") (:fr "e-mail" "mel" "courriel"))(:min 1))
  (:rel ((:en "organization") (:fr "organisation"))  (:to "organization"))
  (:rel ((:en "topic of interest") (:fr "sujet d'intérêt"))(:to "topic"))
  (:att ((:en "personal assistant") (:fr "assistant personnel")))
  (:doc (:en "A PERSON is here a member of the HEUDIASYC laboratory.")
        (:fr "Une PERSONNE est ici un membre du laboratoire HEUDIASYC."))
  )

;;;-------------------------------------------------------------------------------
;;;                              TOPIC
;;;-------------------------------------------------------------------------------	

(defconcept ((:en "topic") (:fr "sujet"))
  (:att ((:en "label") (:fr "libellé"))(:entry))
  (:doc (:en "A TOPIC is a domain of interest.")
        (:fr "Un SUJET est un domaine d'intérêt."))
  )

;;;===============================================================================
;;;
;;;                               METHODS
;;;
;;;===============================================================================

;;;------------------------------------------------------ (PASSWORD) =format-value

(defownmethod =format-value 
    ("password" "moss property name" "moss attribute" :class-ref "member")
  (value)
  (declare (ignore value))
  "********")

;;;--------------------------------------------------------- (PASSWORD) =if-needed

(defownmethod =if-needed
    ("password" "moss property name" "moss attribute" :class-ref "member")
  (obj-id)
  (send obj-id '=get "login"))

;;;---------------------------------------------- (PERSON) =make-local-description
;;; ad hoc method for returning name and assistant 

(definstmethod =make-local-description PERSON (language assistant-name)
  (let ((name (send *self* '=get "name"))
        (first-name (send *self* '=get "first name"))
		(email (send *self* '=get "email"))
		(assistant (or assistant-name (send *self* '=get "personal assistant"))))
   (case language
     (:fr
	   `("personne" ("nom" ,@name)("prénom" ,@first-name)("email" ,email)
	                ("assistant personnel" ,assistant)))
	 (:en
	   `("person" ("name" ,@name)("first name" ,@first-name)("email" ,email)
	              ("personal assistant" , assistant))))))
      
#|
(send '$e-member.3 '=make-local-description :fr "Albert")
("personne" ("nom" "Bouabdallah") ("prénom" "Abdelmadjid") 
 ("assistant personnel" "Albert"))
(send '$e-member.3 '=make-local-description :en "Albert")
("person" ("name" "Bouabdallah") ("first name" "Abdelmadjid")
 ("personal assistant" "Albert"))
|#
;;;--------------------------------------------------------- (PERSON) =make-string

(definstmethod =make-string PERSON ()
   (list (format nil "~{~A~^ ~}: ~{~A~^ ~}"
              (HAS-NAME)(or (HAS-FIRST-NAME) "?"))))
#|
(send '$E-member.1 '=make-string)
("Abel: Marie-Hélène")
|#			  
;;;------------------------------------------------------- (ORGANIZATION) =summary

(definstmethod =summary ORGANIZATION ()
  (list (format nil "~{~A~^, ~}" 
          (or (HAS-ACRONYM) (HAS-NAME) '("?")) )))

#|
(send '$E-ORGANIZATION.3 '=summary)
("Coventry University")
|#

;;;------------------------------------------------------------- (PERSON) =summary

(definstmethod =summary PERSON ()
  (list (format nil "~{~A~^, ~} ~{~A~^, ~}" 
          (or (HAS-NAME) '("?"))(or (HAS-FIRST-NAME) '("?")) )))

#|
(send '$E-PERSON.3 '=summary)
("Anane Rachid")
|#
;;;-------------------------------------------------------------- (TOPIC) =summary

(definstmethod =summary TOPIC ()
  (list (format nil "~{~A~^, ~}" 
          (or (HAS-LABEL) '("?")) )))

#|
(send '$E-TOPIC.3 '=summary)

|#

;;;===============================================================================
;;;===============================================================================
;;;
;;;                                INDIVIDUALS
;;;
;;;===============================================================================
;;;===============================================================================

;;;================================================================== ORGANIZATION

(defchapter 
   (:name :en "Organisation"))
   
(defmacro deforg (name &optional acronym country)
  `(defindividual "organization"
       ("name" ,name)
     ,@(if acronym `(("acronym" ,acronym)))
     ,@(if country `(("country" ,country)))
     ))

(deforg "Université de Technologie de Compiègne" "UTC" "France")
(deforg "Instituto de Tecnologia do Paraná" "TECPAR" "Brésil")
(deforg "Chiba Institute of Technology" "CIT" "Japon")
(deforg "Centro de Investigación y de Estudios Avanzados del Instituto Politécnico Nacional" "CINVESTAV" "Mexique")
(deforg "Coventry University" "CU" "UK")

;;;======================================================================== PEOPLE 

(defchapter
  (:name :en "Persons"))

(defMacro defmember (login name &optional first-name organization email)
  `(defindividual "Member"
     ("login" ,login)
     ,@(if name (if (listp name) `(("name" ,@name)) `(("name" ,name))))
     ,@(if first-name (if (listp first-name) `(("first name" ,@first-name))
                           `(("first name" ,first-name))))
     ,@(if organization `(("organization" ,organization)))
     ,@(if email `(("email" ,email)))
	 ))

(defmember "abel" "Abel" "Marie-Hélène" "UTC" "marie-helene.abel@utc.fr")
(defmember "barthes" "Barthès" "Jean-Paul" "UTC" "barthes@utc.fr")
(defmember "mramos" "Ramos" "Milton" "TECPAR")
(defmember "framos" "Ramos" "Felix" "CINVESTAV" "framos@gdl.cinvestav.mx")
(defmember "sugawara" "Sugawara" "Kenji" "CIT" "suga@net.it-chiba.ac.jp")
(defmember "thompson" "Thompson" "Phil" "csx245@coventry.ac.uk")

;;;=============================================================================== 

:EOF