﻿;;;-*- Mode: Lisp; Package: "CSCWD-ONTO" -*-
;;;==============================================================================
;;;14/11/10
;;;                           AGENT CSCWD-ONTO
;;;                           Copyright Barthès 2014
;;;
;;;==============================================================================

;;; Skills (+ -> not yet implemented), D from Dialog, W from Web
;;;   - ADD-CONCEPT                 add a new synonym or create a new category
;;;   - ADD-ROLE                    add a new synonym or create a new index
;;;   - ADD-PERSON


#|
2014
 0920 Creation
 0930 new skills get-all-concepts-json,get-concept-json and 
      utility functions loading (mfk)
 1110 correcting create-cscwd-concept
|#

(defpackage :CSCWD-ONTO (:use :cl :moss :omas)(:nicknames :onto))
(in-package :CSCWD-ONTO)

;(setq moss::*debug* t)
;(trace moss::%string-norm)

(format t "~A" 
  (catch :error
         (omas::defagent :CSCWD-ONTO :language :en :persistency nil)))

;;; ask OMAS to send messages only once
(setf (omas::max-repeat-count *omas*) 0)

;;; ===============================================================================
;;;
;;;            Temporary loading some utility functions for rendering (mfk)
;;;
;;; ===============================================================================
(omas::compile-and/or-load 
  (merge-pathnames "utils.lisp" 
    (string+ (namestring omas::*omas-directory-string*) 
             "applications\\nb-ontocodesign\\")))

(defun zap ()
  (setf (get :cscwd-onto :cscwd-concept) nil)
  (setf (get :cscwd-onto :annotation) nil)
  (setf (get :cscwd-onto :role-type) nil)
  (setf (get :cscwd-onto :role) nil)
  )

;;;=============================================================================
;;;============================ EDIT FORMATS ===================================
;;;=============================================================================

;;; the following parameter describes the various layouts for editing objects
;;; :row means that we have a single line
;;; :area means that we have several lines
;;; :header specifies the header of the property in the page
;;; :parm specifies the name of the parameter in the http line
;;; :edit if t means that we can change the value without unlinking the successor
;;; :path gives the path to access the displayed value or the successor
;;; :rows indicate the height of the area
;;; :cols gives the length of the area
;;; :max specifyies the max number of values (1 is used to tell the system that we do
;;;   not want to split the text into several values)
;;; :separator indicates the separator in case of multiple values (comma is default)
;;; :type type of the value
;;; we assume that the property "name" is the name of the attribute, however for
;;; a relation, the name of the relation is the first element of the path

(defeditlayout "ANNOTATION" :CSCWD-ONTO
  ((:area :header "Author (read only)" 
          :name "author" :rows 1 :cols "150%" :read-only t
          :path ("author" "Person") :initfcn create-author)
   (:row :header "Creation Date (read only)" :name "Creation date" :cols "60%"
         :read-only t :initfcn create-date)
   (:row :header "Current language" :selfcn select-language :name "language"
         :selected (symbol-name *language*) :actfcn set-foreign-language)
   (:area :header "Text" :name "text" :rows 6 :cols "150%")
   ))
  
;;;********** missing an area for annotations
(defeditlayout "CSCWD-CONCEPT" :CSCWD-ONTO
  ((:row :header "CSCWD English Concept Name (required)" :name "label"
         :property "label"
         :cols "60%" 
         :format-value select-english-tag)
   (:area :header "English Definition" :name "definition" :rows 4 :cols "60%"
          :format-value select-english-tag)
   (:row :header "Subsuming Concept" :name "subsuming concept" 
         :path ("subsuming concept" "Concept")
         :if-does-not-exist :ignore :selfcn select-concept :selprop "label")
   ;; :if-does-not-exist :ignore means that if we do not find the concept, we do 
   ;; not create a new one (shold not hapen since we select from existing values)
   (:area :header "Author (read only)" 
          :name "author" :rows 1 :cols "60%" :read-only t
          :path ("author" "Person") :initfcn create-author)
   ;; source is an MLN with a pointer for each language. The link option created a
   ;; pointer in the first second column of the table (should use javascript)
   (:row :header "Source (URL)" :name "source" :format-value select-english-source
         :link t)
   (:row :header "Creation Date (read only)" :name "Creation date" :cols "60%"
         :read-only t :initfcn create-date)
   (:row :header "Modification Date (read only)" :name "Modification date" 
         :cols "60%" :read-only t)
   ;;---
   ;; specifying a language different from the PA language sets the *foreign-language*
   ;; variable, used to post data into the following fields. However, refreshing 
   ;; should be done with javascript (?). Possibly it is enough to blank the 
   ;; various fields and issue a message.
   ;; One could do that with passing object-to-edit, or label and definitions MLNs
   ;; in a hidden field!
   (:js :code "function clearFields() {
      //alert(\"coucou\");
      document.getElementById(\"flabel\").value = \"\";
      document.getElementById(\"fdefinition\").value = \"\";
      document.getElementById(\"fsource\").value = \"\";
      //location.reload(); // change not taken into account
	     }")
   ;; using "language" as :name allows changing the global value of the *language*
   ;; variable, since in each of the edit functions, language is reset from the
   ;; values associated with "language" in the user entry
   (:row :header "2nd language (if changed, please refresh page)" 
         :selfcn select-language :name "language"
         :selected (symbol-name *language*) :actfcn set-foreign-language
         :on-change "clearFields();")
   (:row :header "CSCWD Concept Name if not English" :name "foreign-label"
         :property "label" :id "flabel"
         :cols "60%" 
         :format-value select-current-tag)
   (:area :header "Definition if not English" :name "foreign-definition" :rows 4 
          :cols "60%" :property "definition" :format-value select-current-tag
          :id "fdefinition")
   ;; we must not include :type (crashes on nil values)
   (:row :header "Source if not English (URL)" :name "foreign-source" 
         :property "source" :id "fsource"
         :format-value select-current-source :link t)
   ;;---
   (:user-action :header "View annotations..." 
                 :name "show/create annotations" :actfcn show-annotations)
   )
  )

;;;(:row :header "Catégorie (max 1)" :name "catégorie" 
;;;         :path ("catégorie" "Catégorie")
;;;         :if-does-not-exist :ignore :selfcn sel-category :selprop "libellé")

;;;(:row :header "Mots-clés" :name "mots clés" :path ("mots clés" "Mot Clé")
;;;         :if-does-not-exist :create 
;;;         :list-fcn ("keywords?var=mots%20clés&language=FR" "label"))

(defeditlayout "ROLE" :CSCWD-ONTO
  ((:row :header "Type of ROLE" :name "role type" :class-ref "Role Type"
         :path ("MOSS-IS-A" "Role Type" "label")
         :selfcn select-role-type :sel-prop "label"
         :format-value select-english-tag)
   (:area :header "English Definition of Role" :name "definition" :rows 4
          :path ("MOSS-IS-A" "Role Type" "definition")
          :cols "60%" :format-value select-english-tag)
   (:row :header "From (Initial Concept)" :name "from" :class-ref "CSCWD Concept"
         :path ("from" "Concept") 
         :if-does-not-exist :ignore :selfcn select-concept :selprop "label")
   (:row :header "To (Target Concept)" :name "target" :id "target"
         :path ("target" "CSCWD Concept") :if-does-not-exist :create
         :list-fcn ("popupconcepts?var=target&language=EN" "label"))
   (:area :header "Author (read only)" 
          :name "author" :rows 1 :cols "60%" :read-only t
          :path ("author" "Person") :initfcn create-author)
   (:row :header "Creation Date (read only)" :name "creation date" :cols "60%"
         :read-only t :initfcn create-date)
   (:row :header "Modification Date (read only)" :name "modification date" 
         :cols "60%" :read-only t)
   )
  )
  
(defeditlayout "ROLE-TYPE" :CSCWD-ONTO
  ((:row :header "CSCWD English Role Name (required)" :name "LABEL"
         :property "LABEL"
         :format-value select-english-tag)
   (:area :header "English Definition of Role" :name "DEFINITION" :rows 4 
          :cols "60%" :format-value select-english-tag)
   (:area :header "Author (read only)" 
          :name "author" :rows 1 :cols "60%" :read-only t
          :path ("author" "Person") :initfcn create-author)
   (:row :header "Creation Date (read only)" :name "Creation date" :cols "60%"
         :read-only t :initfcn create-date)
   (:row :header "Modification Date (read only)" :name "Modification date" 
         :cols "60%" :read-only t)
;;;   (:row :header "Current Language (read only)" :name "current-language" 
;;;         :read-only t
;;;         :property "label" 
;;;         :format-value select-current-language
;;;         :initfcn show-current-language)
   (:js :code "function clearFields() {
      //alert(\"coucou\");
      document.getElementById(\"flabel\").value = \"\";
      document.getElementById(\"fdefinition\").value = \"\";
      //location.reload(); // change not taken into account
	     }")
   (:row :header "2nd language (if changed, please refresh page)" 
         :selfcn select-language :name "language"
         :selected (symbol-name *language*) :actfcn set-foreign-language
         :on-change "clearFields();")
   (:row :header "CSCWD Concept Name if not English" :name "foreign-label"
         :property "label" :id "flabel"
         :cols "60%" 
         :format-value select-current-tag)
   (:area :header "Definition if not English" :name "foreign-definition" :rows 4 
          :cols "60%" :property "DEFINITION" :format-value select-current-tag
          :id "fdefinition")
   )
  )

   
;;;=============================================================================
;;;================================ GLOBALS ====================================
;;;=============================================================================


;;;=============================================================================
;;;=========================== SERVICE FUNCTIONS ===============================
;;;=============================================================================

;;;------------------------------------------------------ ADD-ENGLISH-ANNOTATION

(defun add-english-annotation (ip)
  "must call the edit page for annotation while specifying the language and ~
   id of the object to which annotation objects are attached. I.e. must produce:
Arguments:
   ip: index into the user-list acting as a cookie
Return:
  e.g. \"editentry?create=t&ip=<ip>&class-ref=<class-ref>&ref=<ref>&language=EN\"
 "
  (let* ()
    ;; test
    "editentry?create=t&class-ref=annotation&language=EN&ref=$e-cswd-concept.2"
    ))

;;;------------------------------------------------------ ADD-FOREIGN-ANNOTATION

(defun add-foreign-annotation (ip)
  "add-foreign-annotation")

;;;----------------------------------------------------------- CREATE-ANNOTATION

(defun create-annotation (agent args)
  "creates a new annotation and link it to tis owner.
Arguments:
   agent: CSCWD-ONTO
   args: (:data <class-ref> <parms>)
Return:
   the id-ref and a list of messages."
  (let* ((data (omas.web::remove-empty-parameters (cdr (getvl args :data))))
         id author owner
         )
    ;; get the author name by parsing the data
    (setq author (send '$E-person '=parse-summary (get-field "author" data)))
    ;; should isify it!
    (setq author (cons "person" (isify author)))
    
    ;; get the owner, quit if not there
    (setq owner (get-field "owner" data))
    (if owner 
        (setq owner (read-from-string owner nil nil))
      (return-from create-annotation
        (values nil
                (list "Can't create annotation because I don't find its owner."))))
    
    ;; create now new object and link it to its owner
    (with-transaction agent
      (setq id
            (apply #'make-individual
                   `(,(car (getvl args :data))
                       ("author" ,author)
                       ("creation date" ,(get-field "creation date" data))
                       ("text" ,(get-field "text" data))
                       ("language" ,(get-field "language" data))
                       )))
      (send owner '=add-related-objects "annotation" (list id))
      )
    
    (values (format nil "~A" id)
            (list "The new annotation has been created."))
    ))

;;;--------------------------------------------------------------- CREATE-AUTHOR
;;; we get the user name by accessing the omas::users entry by testing the
;;; "pa-key" field
;;; used by the page layout

(defun create-author ()
  "get the user name from the users entry."
  (let ((user-list (omas::users *omas*)))
    ;; check each entry until the right one is found
    (dolist (item user-list)
      (when (equal+ (get-field "agent-ref" (cdr item)) "CSCWD-ONTO")
        ;; note that user-name is different from user-login
        (return-from create-author
          (list (format nil "~A: ~A" 
                  (get-field "user-name" (cdr item)) 
                  (or (get-field "user-first-name" (cdr item)) "?"))))
        )
      )
    (list "CSCWD-ONTO")))

#|
(create-author)
("Barthès: Jean-Paul")
|#
;;;-------------------------------------------------------- CREATE-CSCWD-CONCEPT
;;; args looks like:
;;;((:DATA "CSCWD-CONCEPT" ("LABEL" . "MAS")
;;;  ("DEFINITION" . "Multi") ("author" . "Barthès: Jean-Paul")
;;;  ("Creation date" . "22/09/2014") ("language" . ":FR")
;;;  ("FOREIGN-LABEL" . "SMA")
;;;  ("FOREIGN-DEFINITION" . "Agents"))
;;; (:LANGUAGE . :FR) (:TIMEOUT . 20))

(defun create-cscwd-concept (agent args)
  "create a CSCWD concept from the message arguments.
Arguments:
   agent: CSCWD-ONTO
   args: (:data <class-ref> <parms>)
Return:
   the id-ref and a list of messages."
  (let* ((data (omas.web::remove-empty-parameters (cdr (getvl args :data))))
         (language (get-field "language" data))
         (foreign-language (omas::set-language language :only-return t))
         (english-label (get-field "label" data))
         (english-definition (get-field "definition" data))
         (foreign-label (get-field "foreign-label" data))
         (foreign-definition (get-field "foreign-definition" data))
         (source (get-field "source" data))
         (foreign-source (get-field "foreign-source" data))
         (subsuming-concept (get-field "subsuming concept" data))
         label-mln definition-mln id author subsumer
         )
    
    ;; if foreign language is not recognized complain (should not happen since we
    ;; select the language from the official list!). Note that language is taken
    ;; from parms, i.e. the value of the web page language field, not from the 
    ;; :LANGUAGE option of the content language
    (unless foreign-language
      (return-from create-cscwd-concept
        (values nil (list (format nil "Language ~S is illegal!" language)))))
    
    ;; Note that the global variable *language* is not modified here. Changes are
    ;; done directly using the foreign-language variable
    
    ;; if English part is missing complain
    (unless (and english-label english-definition)
      (return-from create-cscwd-concept
        (values nil (list "English data are missing!"))))
    
    ;; we should here check for the existence of the concept. If there we should 
    ;; not create it again, since it will zap the definitions in other languages!
    ;; the label can have multiple values
    (dolist (item (omas::split-val english-label #\,))
      (when (access `("Cscwd-concept" ("label" :is ,item)))
        (return-from create-cscwd-concept
          (values nil (list "This concept already exists!")))))
    
    (cond
     ;; if language is English then only consider English input
     ((eql foreign-language :en)
      (setq label-mln (mln::make-mln `(:en ,english-label) :separator #\,))
      (setq definition-mln (mln::make-mln `(:en ,english-definition)))
      )
     ;; if language is not English but foreign inputs are missing, then complain
     ((null (and foreign-label foreign-definition))
      (return-from create-cscwd-concept
        (values nil (list "Foreign data are missing!"))))
     ;; otherwise build the ad hoc MLNs (labels can have synonyms)
     (t
      (setq label-mln (mln::make-mln `(:en ,english-label
                                           ,foreign-language ,foreign-label)
                                     :separator #\,))
      (setq definition-mln 
            (mln::make-mln `(:en ,english-definition
                                 ,foreign-language ,foreign-definition)))
      )
     )
    ;; get the author name by parsing the data
    (setq author (send '$E-person '=parse-summary (get-field "author" data)))
    ;; should isify it!
    (setq author (cons "person" (isify author)))
    
    ;; build source value
    (when (or source foreign-source)
      (setq source 
            (mln::make-mln
             `(,@(if source `(:en ,source))
                  ,@(if foreign-source `(,foreign-language ,foreign-source))))))
    
    (when subsuming-concept
      (setq subsumer (substitute #\space #\_ subsuming-concept)))
    
    ;; create now new object
    (with-transaction agent
      (setq id
            (apply #'make-individual
                   `(,(car (getvl args :data))
                       ("label" ,label-mln)
                       ("definition" ,definition-mln)
                       ("author" ,author)
                       ,@(if source `(("source" ,source)))
                       ("creation date" ,(get-field "creation date" data))
                       ,@(if subsuming-concept `(("subsuming concept" ,subsumer)))
                       ))))
    
    (values (format nil "~A" id)
            (list (format nil "The new concept ~A has been created." english-label)))
    ))

#|
(setq args
      '((:DATA "CSCWD-CONCEPT" ("LABEL" . "MAS")
               ("DEFINITION" . "Multi") ("author" . "Barthès: Jean-Paul")
               ("Creation date" . "22/09/2014") ("language" . "FR")
               ("FOREIGN-LABEL" . "SMA")
               ("source" "http://en.wikipedia.org/wiki/Engineering")
               ("FOREIGN-DEFINITION" . "Agents"))
        (:LANGUAGE . :FR) (:TIMEOUT . 20)))

(create-cscwd-concept sa_cscwd-onto args)
"$E-CSCWD-CONCEPT.72"
("The new concept MAS has been created.")

$E-CSCWD-CONCEPT.72
((MOSS::$TYPE (0 $E-CSCWD-CONCEPT)) (MOSS::$ID (0 $E-CSCWD-CONCEPT.72))
 ($T-CSCWD-CONCEPT-LABEL (0 ((:EN "MAS") (:FR "SMA"))))
 ($T-CSCWD-CONCEPT-DEFINITION (0 ((:EN "Multi") (:FR "Agents"))))
 ($S-CSCWD-CONCEPT-AUTHOR (0 $E-PERSON.1))
 ($T-CSCWD-CONCEPT-SOURCE (0 ((:EN "http://en.wikipedia.org/wiki/Engineering"))))
 ($T-CSCWD-CONCEPT-CREATION-DATE (0 "22/09/2014")))
|#
;;;----------------------------------------------------------------- CREATE-DATE
;;; used by the page layout

(defun create-date ()
  "creates a date as jj/mm/yyyy"
  (list (moss::get-current-date :time (get-universal-time))))

#|
(create-date)
("28/06/2014")
|#
;;;----------------------------------------------------------------- CREATE-ROLE
;;;(:DATA "ROLE" ("LABEL" . "use")
;;;       ("DEFINITION"
;;;        . "using means that the first thing denated by its concept uses the second one in its daily life")
;;;       ("from" . "Knowledge_Intensive_Approach") ("target" . "Knowledge")
;;;       ("author" . "Barthès: ?") ("Creation date" . "02/10/2014")
;;;       ("Modification date" . "") ("current-language" . "")
;;;       ("FOREIGN-LABEL" . "") ("FOREIGN-DEFINITION" . ""))

(defun create-role (agent args)
  "create a role from the message arguments.
Arguments:
   agent: CSCWD-ONTO
   args: (:data <class-ref> <parms>)
Return:
   the id-ref and a list of messages."
  (declare (special *language*))
  (let* ((data (cdr (getvl args :data)))
         (parms (omas.web::remove-empty-parameters data))
         (from-label (substitute #\space #\_ (get-field "from" parms)))
         (target-label (get-field "target" parms)) 
         id author role-kind role-prototype
         )
    
    (format t "~2%;---------- CSCWD-ONTO: Entering create-role")
    (format t "~%;--- *language*: ~S" *language*)
    (format t "~%;--- data: ~S" data)
    
    ;; role type is set in English because of the pull down menus
    (let ((*language* :EN))
      (setq role-kind (substitute #\space #\_ (get-field "role type" parms)))
      (setq role-prototype (car (access `("Role Type" ("label" :is ,role-kind)))))
      )
    
    (format t "~%;--- role-kind: ~S" role-kind)
    (format t "~%;--- access: ~S" (access `("Role Type" ("label" :is ,role-kind))))
    (format t "~%;--- role-prototype: ~S" role-prototype)
    ;; if English part is missing complain
    (unless role-prototype
      (return-from create-role
        (values nil (list "Illegal or missing role-type!"))))
    
    (unless from-label
      (return-from create-role
        (values nil (list "Missing From concept!"))))
    
    (unless target-label
      (return-from create-role
        (values nil (list "Missing Target concept(s)!"))))
    
    ;; get the author name by parsing the data
    (setq author (send '$E-person '=parse-summary (get-field "author" parms)))
    ;; should isify it!
    (setq author (cons "person" (isify author)))
    
    ;; create now new object
    (with-transaction agent
      (setq id
            (apply #'make-individual
                   "Role"
                   `("author" ,author)
                   `("creation-date" ,(get-field "creation date" parms))
                   `("from" ,from-label)
                   `("target" ,@(omas::split-val target-label #\,))
                   nil))
      ;; does not seem to be done in any other way
      (send id '=add-sp-id 'moss::$is-a role-prototype)
      )
    (values (format nil "~A" id)
            (list (format nil "The new ~{~A~} role has been created."
                    (send id '=summary)))
            )))

;;;------------------------------------------------------------ CREATE-ROLE-TYPE

(defun create-role-type (agent args)
  "create a new type of role from the message arguments.
Arguments:
   agent: CSCWD-ONTO
   args: (:data <class-ref> <parms>)
Return:
   the id-ref and a list of messages."
  (let* ((data (cdr (getvl args :data)))
         (data (omas.web::remove-empty-parameters data))
         (language (getvl args :language))
         (english-label (get-field "label" data))
         (english-definition (get-field "definition" data))
         (foreign-label (get-field "foreign-label" data))
         (foreign-definition (get-field "foreign-definition" data))         
         label-mln definition-mln id author
         )
    ;; if English part is missing complain
    (unless (and english-label english-definition)
      (return-from create-role-type
        (values nil (list "English data are missing!"))))
    
    (cond
     ((eql language :en)
      (setq label-mln (mln::make-mln `(:en ,english-label)))
      (setq definition-mln (mln::make-mln `(:en ,english-definition)))
      )
     ((null (and foreign-label foreign-definition))
      (return-from create-role-type
        (values nil (list "Foreign data are missing!"))))
     (t
      (setq label-mln (mln::make-mln `(:en ,english-label
                                           ,language ,foreign-label)))
      (setq definition-mln (mln::make-mln `(:en ,english-definition
                                                ,language ,foreign-definition)))
      )
     )
    ;; get the author name by parsing the data
    (setq author (send '$E-person '=parse-summary (get-field "author" data)))
    ;; should isify it!
    (setq author (cons "person" (isify author)))
        
    ;; create now new object
    (with-transaction agent
      (setq id
            (apply #'make-individual
                   (car (getvl args :data))
                   `("label" ,label-mln)
                   `("definition" ,definition-mln)
                   `("author" ,author)
                   `("creation-date" ,(get-field "creation-date" data))
                   nil)))
    ;; here we must remove the page description of a role from the p-list of
    ;; :CSCWD-ONTO where it is cached
    (setf (get :CSCWD-ONTO :ROLE) nil)
    (values (format nil "~A" id)
            (list (format nil "The new role type ~A has been created." 
                    english-label)))
    ))

;;;-------------------------------------------------------------------- FIND-VAL

(defun find-val (class-ref prop-ref val-ref)
  "gets class prop and value(s) and should return a pair. Wont work when val-ref ~
   is a summary value.
Argument:
   class-ref: class of successors
   prop-ref: relation
   val-ref: blank string or summary-string value(s)
Return
   2 values: NIL or a list (prop-ref ids) and a message list."
  (format t "~2%;---------- CSCWD-ONTO: Entering find-val")
  (let* (class-id id-list message-list suc-list)
    ;; if parameter is empty return no pair no message
    (if (equal+ (moss::%string-norm val-ref) "")
        (return-from find-val (values (list prop-ref) nil)))
    
    ;; get class id and complain if it cannot be found
    (setq class-id (moss::%%get-id class-ref :class))
    ;(format t "~%;--- class-id: ~S" class-id)
    (unless class-id
      (setq message-list 
            (list (format nil "Can't find class ~S" class-ref)))
      (return-from find-val (values (list prop-ref) message-list)))
    
    ;; for each entry get corresponding objects 
    (dolist (val (omas::split-val val-ref))
      (format t "~%;--- val: ~S" val)
      ;; do we have an object with this value
      (setq id-list (access val))
      ;(format t "~%;--- id-list: ~S" id-list)
      (setq id-list 
            (remove nil
                    (remove-if-not #'(lambda (xx) (moss::%type? xx class-id))
                                   id-list)))
      ;(format t "~%;--- id-list after test: ~S" id-list)
      (if id-list
          (setq suc-list (append suc-list id-list))
        (push (format nil "~A ~S is unknown." class-ref val) message-list)
        )
      )
    
    ;; remove possible duplicates
    (setq suc-list (delete-duplicates suc-list))
    ;; return result
    (values (cons prop-ref suc-list) message-list)
    ))

#|
(find-val "cscwd concept" "subsuming concept" " ")
("subsuming concept")
NIL

(find-val "cscwd conct" "subsuming concept" "CSCWD")
("subsuming concept")
("Can't find class \"cscwd conct\"")

(find-val "cscwd concept" "subsuming concept" "CSCW")
("subsuming concept" $E-CSCWD-CONCEPT.1)
NIL

(find-val "cscwd concept" "subsuming concept" "Workflow")
("subsuming concept")
("cscwd concept \"Workflow\" is unknown.")
|#    
;;;----------------------------------------------------------------------- ISIFY

(defun isify (alist)
  "takes a list of sublists (<prop> <value>*) and produces a list of sublists ~
   (<prop> :is <value>*)"
  (mapcar #'(lambda (xx) `(,(car xx) :is ,@(cdr xx))) alist))

#|
(isify nil)
NIL
(isify '(("a" "aa")("b" "bb")))
(("a" :IS "aa") ("b" :IS "bb"))
|#
;;;---------------------------------------------------------MODIFY-CSCWD-CONCEPT
;;; Use the parms values to build a new object, then erase the old one, replacing
;;; it with the new one. The function is tightly linked to the form being posted
;;; Indeed, all the properties may not be posted for modification. If a property is
;;; not posted, then the value associated with this property will not be replaced.
;;; Therefore: we build pairs (<prop> <value>*) using the parms, and replace new
;;; values with the result.
;;; args:
;;;((:DATA 
;;;  "CSCWD-CONCEPT"
;;;
;;;  (("id" "$E-CSCWD-CONCEPT.2")
;;;   ("LABEL"
;;;    ((:EN "Computer-Supported Cooperative Work in Design"
;;;          "CSCWD")
;;;     (:FR
;;;      "Travail coopératif assisté par ordinateur pour le desing"
;;;      "TCAOD")))
;;;   ("DEFINITION"
;;;    ((:EN
;;;      "This designate a specialization of CSCWD applied to Design.")
;;;     (:FR
;;;      "Ceci designe une spécialisation du TCAO à la conception.")))
;;;   ("subsuming concept"
;;;    ("Computer-Supported Cooperative Work"
;;;     "subsuming concept" "$E-CSCWD-CONCEPT.1"))
;;;   ("author" ("Barthès : Jean-Paul" "author" "$E-PERSON.1"))
;;;   ("Creation date" "21/09/2014"))
;;;
;;;  (("LABEL"
;;;    . "Computer-Supported Cooperative Work in Design, CSCWD")
;;;   ("DEFINITION"
;;;    . "This designates a specialization of CSCWD applied to Design.")
;;;   ("subsuming concept" . "")
;;;   ("author" . "Barthès : Jean-Paul")
;;;   ("Creation date" . "21/09/2014") ("language" . ":FR")
;;;   ("FOREIGN-LABEL"
;;;    . "Travail coopératif assisté par ordinateur pour le desing, TCAOD")
;;;   ("FOREIGN-DEFINITION"
;;;    . "Ceci designe une spécialisation du TCAO à la conception.")))
;;; (:LANGUAGE . :FR) (:TIMEOUT . 20))

(defun modify-cscwd-concept (agent args)
  "args contains original object and data to modify it.
Arguments:
   agent: CSCWD-ONTO
   args: (<class-ref><object-to-modify><parms>)
Return:
   2 values: id-ref and list of messages."
  (let* ((data (getvl args :data))
         (obj-l (cadr data))
         (parms (omas.web::remove-empty-parameters (caddr data)))
         (foreign-language (or (get-field "language" parms)
                               (getvl args :language)))
         (subsuming-concept (get-field "subsuming concept" parms))
         (id-ref (car (get-field "id" obj-l)))
         (id (intern (moss::%string-norm id-ref)))
         changes msg-list object-list message-list
         old-suc-list)
    
    (setq foreign-language (omas::set-language foreign-language :only-return t))
    
    (unless foreign-language
      (error "modify-cscwd-concept: foreign language could not be retrieved ~
	          either from the user entry or from the message :LANGUAGE option."))
    
    (format t "~2%;---------- CSCWD-ONTO: Entering modify-cscwd-concept")
    ;(format t "~%;--- obj-l: ~S" obj-l)
    (format t "~%;--- foreign-language: ~S" foreign-language)
    
    ;; first process label and description
    (multiple-value-setq (changes message-list)
      (modify-label-and-definition obj-l parms foreign-language))
    (format t "~%;--- changes: ~S" changes)
    (format t "~%;--- second values argument: ~S" message-list)
    
    (if (eql changes :error)
        ;; if answer is error return object reference and message list
        (return-from modify-cscwd-concept (values id-ref message-list)))
    
    ;; if changes is empty, next instruction is a NOOP
    (setq object-list (append changes object-list))
    
    ;; modification date must be set
    (push (list "modification date" (moss::get-current-date)) object-list)
    
    ;; author are read-only
    ;; roles are not displayed in the concept form
    
    ;; subsuming concept
    (format t "~4%;--- subsuming-concept: ~S" subsuming-concept)
    (when subsuming-concept
      (multiple-value-setq (changes msg-list)
        (find-val "CSCWD Concept" "subsuming concept" subsuming-concept)))
    
    (when changes
      ;; check that successors are different from those we already have
      (setq old-suc-list (mapcar #'caddr (getvl obj-l "subsuming concept")))
      (format t "~%;--- old-suc-list: ~S" old-suc-list)
      ;; compute ids
      (setq old-suc-list (mapcar #'intern old-suc-list))
      (format t "~%;--- old-suc-list: ~S" old-suc-list)
      ;; if the two list of symbols are different must update
      (unless (equal+ old-suc-list (cdr changes))
        (push changes object-list)))
    (when msg-list
      (setq message-list (append message-list msg-list)))
    
    (format t "~%;--- object-list (values to update): ~% ~S" object-list)
    
    ;; update-list contains at least modification date
    (dolist (item object-list)
      (with-transaction agent
        (send id '=replace (car item) (cdr item))))
    
    ;(format t "~%;--- properties to update:~%  ~S" object-list)
    ;; return standard list
    (values id-ref 
            (list (format nil "~{~A~^, ~}" 
                    (cons "Object has been updated" message-list))))
    ))

#| tests
(modify-cscwd-concept 
 sa_cscwd-onto
 '((:DATA 
    "CSCWD-CONCEPT"
    (("id" "$E-CSCWD-CONCEPT.1")
     ("LABEL"
      ((:EN "Computer-Supported Cooperative Work" "CSCW")
       (:FR "Travail coopératif assisté par ordinateur"
            "TCAO")))
     ("DEFINITION"
      ((:EN
        "CSCW is a generic term, which combines the understanding of the way people work in groups with the enabling technologies of computer networking, and associated hardware, software, services and techniques.")
       (:FR
        "Le TRAVAIL CCOPERATIF ASSISTE PAR ORDINATEUR (TCAO) est une méthode de travail permettant à des utilisateurs de travailler en groupe et en réseau, associant des matériels, logiciels, techniques et méthodes spécifiques.")))
     ("subsuming concept")
     ("author" ("Barthès : Jean-Paul" "author" "$E-PERSON.1"))
     ("Creation date" "21/09/2014"))
    (("LABEL" . "Computer-Supported Cooperative Work, CSCW")
     ("DEFINITION"
      . "CSCW is a generic term, which combines the understanding of the way people work in groups with the enabling technologies of computer networking, and associated hardware, software, services and techniques.")
     ("subsuming concept" . "")
     ("author" . "Barthès : Jean-Paul")
     ("Creation date" . "21/09/2014") ("language" . ":FR")
     ("FOREIGN-LABEL"
      . "Travail coopératif assisté par ordinateur, TCAO")
     ("FOREIGN-DEFINITION"
      . "Le TRAVAIL CCOPERATIF ASSISTE PAR ORDINATEUR (TCAO) est une méthode de travail permettant à des utilisateurs de travailler en groupe et en réseau, associant des matériels, logiciels, techniques et méthodes spécifiques.")))
   (:LANGUAGE . :FR) (:TIMEOUT . 20)))
"$E-CSCWD-CONCEPT.1"
("Object has been updated")

(modify-cscwd-concept 
 sa_cscwd-onto
 '((:DATA 
    "CSCWD-CONCEPT"
    (("id" "$E-CSCWD-CONCEPT.1")
     ("LABEL"
      ((:EN "Computer-Supported Cooperative Work" "CSCW")
       (:FR "Travail coopératif assisté par ordinateur"
            "TCAO")))
     ("DEFINITION"
      ((:EN
        "CSCW is a generic term, which combines the understanding of the way people work in groups with the enabling technologies of computer networking, and associated hardware, software, services and techniques.")
       (:FR
        "Le TRAVAIL CCOPERATIF ASSISTE PAR ORDINATEUR (TCAO) est une méthode de travail permettant à des utilisateurs de travailler en groupe et en réseau, associant des matériels, logiciels, techniques et méthodes spécifiques.")))
     ("subsuming concept")
     ("author" ("Barthès : Jean-Paul" "author" "$E-PERSON.1"))
     ("Creation date" "21/09/2014"))
    (("LABEL" . "Computer-Supported Cooperative Work, CSCW")
     ("DEFINITION"
      . "CSCW is a generic term, which combines the understanding of the way people work in groups with the enabling technologies of computer networking, and associated hardware, software, services and techniques.")
     ("subsuming concept" . "")
     ("author" . "Barthès : Jean-Paul")
     ("Creation date" . "21/09/2014") ("language" . ":FR")
     ("FOREIGN-DEFINITION"
      . "Le TRAVAIL CCOPERATIF ASSISTE PAR ORDINATEUR (TCAO) est une méthode de travail permettant à des utilisateurs de travailler en groupe et en réseau, associant des matériels, logiciels, techniques et méthodes spécifiques.")))
   (:LANGUAGE . :FR) (:TIMEOUT . 20)))
"$E-CSCWD-CONCEPT.1"
("Missing foreign (non English) definitions.")
|#

;;;------------------------------------------------- MODIFY-LABEL-AND-DEFINITION
;;; We compare the values of obj-l with the values posted in the web page. If we
;;; do not have English labels, then error
;;; if language is not English and we do not have foreign values, then error
;;; if English values are different, then new values replace the old oned
;;; if foreign values are different, then new values replace the old ones
(defun modify-label-and-definition (obj-l parms language)
  
  "compares the values of old and new label and definition and returns modified ~
   mlns if different or nil if same.
Arguments:
   obj-l: object as it is now
   parms: changes from the edit page fields
   language: language of the edit page
Return:
   2 values: :ERROR <message-list>
             NIL NIL if no changes 
             <list of changes> NIL"
  (format t "~2%;----------- CSCWD-ONTO: Entering modify-label-and-definition")
  (let* ((new-label (get-field "label" parms))
         (new-definition (get-field "definition" parms))
         new-label-foreign new-definition-foreign
         label-mln old-label-values new-label-values label-flag
         definition-mln old-definition definition-flag
         source-mln old-source new-source source-flag)
    ;; if we miss English label and definition complain
    (unless (and new-label new-definition)
      (return-from modify-label-and-definition 
        (values :error '("Missing English definitions."))))
    
    ;; if language is not English and we are missing foreign data, complain
    (unless (eql language :en)
      ;; get data
      (setq new-label-foreign (get-field "foreign-label" parms)
          new-definition-foreign (get-field "foreign-definition" parms))
      (unless (and new-label-foreign new-definition-foreign)
        (return-from modify-label-and-definition 
          (values :error '("Missing foreign (non English) definitions.")))
        )
      )
    
    ;;=== now compare English labels
    (setq label-mln (car (get-field "label" obj-l)))
    (setq old-label-values 
          (when label-mln (mln::extract label-mln :language :en)))
    (format t "~%;--- old-label-values: ~S" old-label-values)
    (setq new-label-values (omas::split-val new-label #\,))
    (format t "~%;--- new-label-values: ~S" new-label-values)
    (format t "~%;--- set-difference new old: ~S"
      (omas::set-equal new-label-values old-label-values))
    ;; using set-equal means that the values should be the same. This allows to
    ;; remove synonyms for example
    (unless (omas::set-equal new-label-values old-label-values)
      (setq label-mln 
            (if label-mln
                (mln::set-values label-mln :en new-label-values)
              (mln::make-mln `(:en ,new-label) :separator #\,)))
      ;; update values
      (setq label-flag t)
      )
    ;; compare English definitions
    (setq definition-mln (car (get-field "definition" obj-l)))
    
    (setq old-definition 
          (when definition-mln 
            (mln::extract-to-string definition-mln :language :en)))
    (format t "~%;--- old-definition: ~S" old-definition)
    (format t "~%;--- new-definition: ~S" new-definition)
    (unless (equal+ old-definition new-definition)
      (setq definition-mln 
            (if definition-mln
                (mln::set-values definition-mln :en (list new-definition))
              (mln::make-mln `(:en ,new-definition))))
      (setq definition-flag t)
      )
    ;; compare English source
    (setq source-mln (car (get-field "source" obj-l)))
    (setq old-source 
          (or
           (when source-mln (mln::extract-to-string source-mln :language :en))
           ""))
    (setq new-source (or (get-field "source" parms) ""))
    (format t "~%;--- old-source: ~S" old-source)
    (format t "~%;--- new-source: ~S" new-source)
    (unless (equal+ old-source new-source)
      (setq source-mln
            (if source-mln
                (mln::set-value source-mln :en (list new-source))
              (mln::make-mln `(:en ,new-source))))
      (setq source-flag t)
      )
    
    (unless (eql language :en)
      
      ;;=== now do the same with language but do not change possibly updated MLNs
      (format t "~%;- For language: ~S" language)
      (setq old-label-values 
            (when label-mln
              (mln::extract label-mln :language language)))
      (format t "~%;--- old-label-values: ~S" old-label-values)
      (setq new-label-values (omas::split-val new-label-foreign #\,))
      (format t "~%;--- new-label-values: ~S" new-label-values)
      (unless (omas::set-equal new-label-values old-label-values :test #'equal+)
        (setq label-mln 
              (if label-mln
                  (mln::set-values label-mln language new-label-values)
                (mln::make-mln `(,language ,new-label) :separator #\,)))
        ;; update values
        (setq label-flag t)
        )
      ;; compare English definitions
      (setq old-definition 
            (when definition-mln
              (mln::extract-to-string definition-mln :language language)))
      (format t "~%;--- old-definition: ~S" old-definition)
      (setq new-definition (or new-definition-foreign ""))
      (format t "~%;--- new-definition: ~S" new-definition-foreign)
      (unless (equal+ old-definition new-definition)
        (setq definition-mln 
              (if definition-mln
                  (mln::set-values definition-mln language (list new-definition))
                (mln::make-mln `(,language ,new-definition))))
        (setq definition-flag t)
        )
      ;; compare English source
      (setq old-source 
            (or
             (when source-mln
               (mln::extract-to-string source-mln :language language))
             ""))
      (setq new-source (or (get-field "foreign-source" parms) ""))
      (format t "~%;--- old-source: ~S" old-source)
      (format t "~%;--- new-source: ~S" new-source)
      (unless (equal+ old-source new-source)
        (setq source-mln
              (if source-mln
                  (mln::set-value source-mln language new-source)
                (mln::make-mln `(,language ,new-source))))
        (setq source-flag t)
        )	 
      )
    
    ;;== return now
    ;; if no flag active return nil
    (values `(,@(if label-flag `(("label" ,label-mln)))
                 ,@(if definition-flag `(("definition" ,definition-mln)))
                 ,@(if source-flag `(("source" ,source-mln))))
            nil)
    ))
	

;;;------------------------------------------------- MODIFY-LABEL-AND-DEFINITION
;;; if English label and definition are not provided, then :error
;;; if language is not English and foreign label and definition are not provided,
;;; then error
;;; if old English values are different new values are inserted
;;; if language is not English and foreign values are different, then new values
;;; are inserted

;;;(defun modify-label-and-definition (obj-l parms language)
;;;  "compares the values of old and new label and definition and returns modified ~
;;;   mlns if different or nil if same.
;;;Arguments:
;;;   obj-l: object as it is now
;;;   parms: changes from the edit page fields
;;;   language: language of the edit page
;;;Return:
;;;   2 values: :ERROR <message-list>
;;;             NIL NIL if no changes 
;;;             <list of changes> NIL"
;;;  (setq parms (omas.web::remove-empty-parameters parms))
;;;  (let* ((old-label-mln (car (get-field  "label" obj-l)))
;;;         (old-label (mln::extract-to-string old-label-mln :language :en))
;;;         (old-foreign-label 
;;;          (mln::extract-to-string old-label-mln :language language))
;;;         
;;;         (label (or (get-field "label" parms) ""))
;;;         (foreign-label (or (get-field "foreign-label" parms) ""))
;;;         ;---
;;;         (old-definition-mln (car (get-field "definition" obj-l)))
;;;         (old-definition (mln::extract-to-string old-definition-mln :language :en))
;;;         (old-foreign-definition 
;;;          (mln::extract-to-string old-definition-mln :language language))
;;;         
;;;         (definition (or (get-field "definition" parms) ""))
;;;         (foreign-definition (or (get-field "foreign-definition" parms) ""))
;;;         ;---
;;;         (old-source-mln (car (get-field "source" obj-l)))
;;;         (old-source (mln::extract-to-string old-source-mln :language :en))
;;;         (old-foreign-source 
;;;          (mln::extract-to-string old-source-mln :language language))
;;;         
;;;         (source (or (get-field "source" parms) ""))
;;;         (foreign-source (or (get-field "foreign-source" parms) ""))
;;;         ;--- 
;;;         label-flag definition-flag source-flag)
;;;    
;;;    (format t "~%;---------- CSCWD-ONTO: Entering modify-label-and-definition")
;;;    (format t "~%;--- obj-l:~%   ~S" obj-l)
;;;    (format t "~%;--- parms:~%   ~S" parms)
;;;    (format t "~%;--- language: ~S" language)
;;;    
;;;    ;(format t "~%;--- old-label-mln: ~S" old-label-mln)
;;;    ;(format t "~%;--- old-label: ~S" old-label)
;;;    ;(format t "~%;--- label: ~S" label)
;;;    
;;;    ;(format t "~%;--- old-definition-mln: ~S" old-definition-mln)
;;;    ;(format t "~%;--- old-definition: ~S" old-definition)
;;;    ;(format t "~%;--- definition: ~S" definition)
;;;    
;;;    ;(format t "~%;--- old-source-mln: ~S" old-source-mln)
;;;    ;(format t "~%;--- old-source: ~S" old-source)
;;;    ;(format t "~%;--- source: ~S" source)
;;;
;;;    ;(format t "~%;--- old-foreign-label: ~S" old-foreign-label)
;;;    ;(format t "~%;--- foreign-label: ~S" foreign-label)
;;;    
;;;    ;(format t "~%;--- old-foreign-definition: ~S" old-foreign-definition)
;;;    ;(format t "~%;--- foreign-definition: ~S" foreign-definition)
;;;    
;;;    ;(format t "~%;--- old-foreign-source: ~S" old-foreign-source)
;;;    ;(format t "~%;--- foreign-source: ~S" foreign-source)
;;;    
;;;    ;; if English reference is not given
;;;    (when (or (equal+ label "")(equal+ definition ""))
;;;      (return-from modify-label-and-definition 
;;;        (values :error '("Missing English definitions.")))
;;;      )
;;;    (unless (equal+ label old-label)
;;;      ;; replace English label in old-label-mln, creating it if it does not exist
;;;      (setq old-label-mln 
;;;            (if old-label-mln
;;;                (mln::set-values old-label-mln :en 
;;;                                 (omas::split-val label #\,))
;;;              (mln::make-mln `(:en ,label) :separator #\,)))
;;;      (setq label-flag t)
;;;      )
;;;    (unless (equal+ definition old-definition)
;;;      (setq old-definition-mln
;;;            (if old-definition-mln
;;;                ;; don't split definition text!
;;;                (mln::set-values old-definition-mln :en (list definition))
;;;              (mln::make-mln `(:en ,definition))))
;;;      (setq definition-flag t)
;;;      )
;;;    (unless (equal+ source old-source)
;;;      (setq old-source-mln
;;;            (if old-source-mln
;;;                (mln::set-values old-source-mln :en (list source))
;;;              (mln::make-mln `(:en ,source))))
;;;      (setq source-flag t)
;;;      )
;;;    
;;;    ;;== this block is for language different from English
;;;    (unless (eql language :en)
;;;      
;;;      ;; if foreign info is not given, then error
;;;      (when (or (equal+ foreign-label "")(equal+ foreign-definition ""))
;;;        (return-from modify-label-and-definition 
;;;          (values :error '("Missing foreign (non English) definitions.")))
;;;        )
;;;      ;; check label first
;;;      (unless (equal+ foreign-label old-foreign-label)
;;;        (setq old-label-mln 
;;;              (mln::set-values old-label-mln language 
;;;                               (omas::split-val foreign-label #\,)))
;;;        (setq label-flag t)
;;;        )
;;;      ;; then definition
;;;      (unless (equal+ foreign-definition old-foreign-definition)
;;;        (setq old-definition-mln
;;;              (mln::set-value old-definition-mln language foreign-definition))
;;;        (setq definition-flag t)
;;;        )
;;;      ;; then source
;;;      (unless (equal+ foreign-source old-foreign-source)
;;;        (setq old-source-mln
;;;              (mln::set-value old-source-mln language foreign-source))
;;;        (setq source-flag t)
;;;        )
;;;      )  ; end foreign language
;;;    
;;;    ;; if no flag active return nil
;;;    (values `(,@(if label-flag `(("label" ,old-label-mln)))
;;;                 ,@(if definition-flag `(("definition" ,old-definition-mln)))
;;;                 ,@(if source-flag `(("source" ,old-source-mln))))
;;;            nil)
;;;    ))


#|
(setq obj-l '(("label" ((:en "ea") (:fr "fa") (:it "ia")))
              ("definition" ((:en "dea")(:fr "dfa")(:it "dia")))
              ))

(setq parms '(("label" . "ea")))
(modify-label-and-definition obj-l parms :en)
:ERROR
("Missing English definitions.")

(setq parms '(("label" . "ea")("definition" . "dea")))
(modify-label-and-definition obj-l parms :en)
NIL
NIL

(setq parms '(("label" . "ea")("definition" . "dea, and more...")))
(modify-label-and-definition obj-l parms :en)
(("definition" ((:EN "dea, and more...") (:FR "dfa") (:IT "dia"))))
NIL

(setq parms '(("label" . "ea, new label")("definition" . "new definition")))
(modify-label-and-definition obj-l parms :en)
(("label" ((:EN "ea" "new label") (:FR "fa") (:IT "ia")))
 ("definition" ((:EN "new definition") (:FR "dfa") (:IT "dia"))))
NIL

(setq parms '(("label" . "ea")("definition" . "")))
(modify-label-and-definition obj-l parms :fr)
:ERROR
("Missing English definitions.")

(setq parms '(("label" . "ea")("definition" . "da")))
(modify-label-and-definition obj-l parms :fr)
:ERROR
("Missing foreign (non English) definitions.")

(setq parms '(("label" . "ea")("definition" . "dea")
              ("foreign-label" . "fa")("foreign-definition" . "dfa")))
(modify-label-and-definition obj-l parms :fr)
NIL
NIL

(setq parms '(("label" . "ea")("definition" . "dea")
              ("foreign-label" . "fa")("foreign-definition" . "new def")))
(modify-label-and-definition obj-l parms :fr)
(("definition" ((:EN "dea") (:FR "new def") (:IT "dia"))))
NIL

(setq parms '(("label" . "ea, new label")("definition" . "dea")
              ("foreign-label" . "fa")("foreign-definition" . "new def")))
(modify-label-and-definition obj-l parms :fr)
(("label" ((:EN "ea" "new label") (:FR "fa") (:IT "ia")))
 ("definition" ((:EN "dea") (:FR "new def") (:IT "dia"))))
NIL

;;; Same tests with nil input
(setq obj-l nil)

(setq parms '(("label" . "ea")))
(modify-label-and-definition obj-l parms :en)
:ERROR
("Missing English definitions.")

(setq parms '(("label" . "ea")("definition" . "dea")))
(modify-label-and-definition obj-l parms :en)
(("label" ((:EN "ea"))) 
 ("definition" ((:EN "dea"))))
NIL

(setq parms '(("label" . "ea, new label")("definition" . "dea")))
(modify-label-and-definition obj-l parms :en)
(("label" ((:EN "ea" "new label"))) 
 ("definition" ((:EN "dea"))))
 

(setq parms '(("label" . "ea, new label")("definition" . "new definition")))
(modify-label-and-definition obj-l parms :en)
(("label" ((:EN "ea" "new label")))
 ("definition" ((:EN "new definition"))))
NIL

(setq parms '(("label" . "ea")("definition" . "")))
(modify-label-and-definition obj-l parms :fr)
:ERROR
("Missing English definitions.")

(setq parms '(("label" . "ea")("definition" . "da")))
(modify-label-and-definition obj-l parms :fr)
:ERROR
("Missing foreign (non English) definitions.")

(setq parms '(("label" . "ea")("definition" . "dea")
              ("foreign-label" . "fa")("foreign-definition" . "dfa")))
(modify-label-and-definition obj-l parms :fr)
(("label" ((:EN "ea") (:FR "fa"))) 
 ("definition" ((:EN "dea") (:FR "dfa"))))
NIL

(setq parms '(("label" . "ea")("definition" . "dea")
              ("foreign-label" . "fa")("foreign-definition" . "new def")))
(modify-label-and-definition obj-l parms :fr)
(("label" ((:EN "ea") (:FR "fa")))
 ("definition" ((:EN "dea") (:FR "new def"))))
NIL

(setq parms '(("label" . "ea, new label")("definition" . "dea")
              ("foreign-label" . "fa")("foreign-definition" . "new def")))
(modify-label-and-definition obj-l parms :fr)
(("label" ((:EN "ea" "new label") (:FR "fa"))) 
 ("definition" ((:EN "dea") (:FR "new def"))))

(setq parms '(("label" . "ea, new label")("definition" . "dea")
               ("foreign-label" . "fa")("foreign-definition" . "new def")
               ("source" . "fs")("foreign-source" . "new source")))
(modify-label-and-definition obj-l parms :fr)
(("label" ((:EN "ea" "new label") (:FR "fa"))) 
 ("definition" ((:EN "dea") (:FR "new def")))
 ("source" ((:EN "fs") (:FR "new source"))))
NIL

(setq obj-l '(("label" ((:en "ea" "eb" "ec") (:fr "fa" "fb") (:it "ia")))
              ("definition" ((:en "dea")(:fr "dfa")(:it "dia")))
              ))
(setq parms '(("label" . "ea,eb")("definition" . "dea") 
              ("foreign-label" . "fb") ("foreign-definition" . "dfa")))
(modify-label-and-definition obj-l parms :fr)
(("label" ((:EN "ea" "eb") (:FR "fb") (:IT "ia"))))
NIL
|#    
;;;------------------------------------------------------------ MODIFY-ROLE-TYPE
;;;((:DATA 
;;;  "ROLE"
;;;
;;;  (("id" "$E-ROLE.1")
;;;   ("LABEL"
;;;    ((:EN "includes") (:FR "comprend" "est composé de")))
;;;   ("DEFINITION"
;;;    ((:EN
;;;      "Includes means that the target concept is part of the initial concept.")
;;;     (:FR
;;;      "Comprend signifie que les concepts cibles font partie du concept de départ.")))
;;;   ("target") ("subsuming concept")
;;;   ("author" ("Barthès : Jean-Paul" "author" "$E-PERSON.1"))
;;;   ("Creation date" "21/08/2014"))
;;;
;;;  (("LABEL" . "includes")
;;;   ("DEFINITION"
;;;    . "Includes means that the target concept is part of the initial concept.")
;;;   ("from" . "CSCW") ("target" . "CSCWD")
;;;   ("subsuming concept" . "")
;;;   ("author" . "Barthès : Jean-Paul")
;;;   ("Creation date" . "21/08/2014")
;;;   ("current-language" . "FR")
;;;   ("FOREIGN-LABEL" . "comprend, est composé de")
;;;   ("FOREIGN-DEFINITION"
;;;    . "Comprend signifie que les concepts cibles font partie du concept de départ.")))
;;; (:LANGUAGE . COMMON-LISP-USER::FR) (:TIMEOUT . 20))

(defun modify-role-type (agent args)
  "uses the values from the Web form to modify the role object.
Arguments:
   agent: CSCWD-ONTO
   args:
Return:
   2 values: id-ref and list of messages."
  (let* ((data (getvl args :data))
         (obj-l (cadr data))
         (language (getvl args :language))
         (parms (caddr data))
         ;(from-label (get-field "from" parms))
         ;(target-label (get-field "target" parms))
         (id-ref (car (get-field "id" obj-l)))
         (id (intern (moss::%string-norm id-ref)))
         changes object-list message-list
         ;from-concept target-concept
         )
    
    (format t "~2%;---------- Entering modify-role")
    ;(format t "~%;--- obj-l: ~S" obj-l)
    
    ;; in case language is not a keyword...
    (set-language language)
    
;;;    ;; because of the select option from and target labels have underscores instead
;;;    ;; istead of spaces; we must remove them
;;;    ;; no longer true but inoffensive
;;;    (setq from-label (substitute #\space #\_ from-label))
;;;    (setq target-label (substitute #\space #\_ target-label))
    
;;;    ;; first process from and to concepts (if not in parms assume ROOT)
;;;    (let ((*language* :all))
;;;      (setq from-concept 
;;;            (car (access `("CSCWD concept" ("label" :is ,from-label)))))
;;;      (setq target-concept 
;;;            (car (access `("CSCWD concept" ("label" :is ,target-label)))))
;;;      )		 
;;;    (unless from-concept (setq from-concept (moss::%%get-id "Root" :class)))
;;;    (unless target-concept (setq target-concept (moss::%%get-id "Root" :class)))
;;;    (format t "~%;--- from-concept: ~S" from-concept)
;;;    (format t "~%;--- target-concept: ~S" target-concept)
;;;    (push `("from" ,from-concept) object-list)
;;;    (push `("target" ,target-concept) object-list)
    
    ;; then process label and description
    (multiple-value-setq (changes message-list)
      (modify-label-and-definition obj-l parms *language*))
    (format t "~%;--- changes: ~S" changes)
    (format t "~%;--- arg 2 (message list): ~S" message-list)
    ;; analyze answer
    (cond
     ;; if answer is error return object reference and message list
     ((eql changes :error)
      (return-from modify-role-type (values id-ref message-list))
      ))
    
    (setq object-list (append changes object-list))
    
    ;; modification date must be set
    (push (list "modification date" (moss::get-current-date)) object-list)
    
    ;; author are read-only
    
    ;; subsuming role (a role can subsume another role only if its from concept
    ;; subsume the fromt concept and its to-concept subsumes the to-concept)
    ;;***** discarded since it obliges to insert into the role hierachy at the
    ;; fittest (lowest) level (too much work for an improbable gain)
    ;;;    (multiple-value-setq (changes msg-list)
    ;;;      (find-subsuming-role "Role" "subsumming role" 
    ;;;                           (get-field "subsuming role" parms) 
    ;;;                           from-concept to-concept))
    
    ;; here we should do something about annotations...
    
    (format t "~%;--- object-list (values to update): ~% ~S" object-list)
    
    ;; update-list contains at least modification date
    (dolist (item object-list)
      (with-transaction agent
        (send id '=replace (car item) (cdr item))))
    
    ;(format t "~%;--- properties to update:~%  ~S" object-list)
    ;; return standard list
    (values id-ref 
            (list (format nil "~{~A~^, ~}" 
                    (cons "Object has been updated" message-list))))
    ))

;;;-------------------------------------------------------------- SELECT-CONCEPT

(defun select-concept (class-ref name prop)
  "returns the list of category labels in alphabetic order, spaces being ~
   replaced by underscores"
  (declare (special *language*)(ignore name class-ref))
  (let ((*language* :EN)
        item-list)
    ;(break "select-concept")
    (format t "~2%;---------- Entering select-concept")
    (format t "~%;--- prop: ~S" prop)
    
    ;; get the list of lists of instances
    (setq item-list (access '("CSCWD Concept")))
    (setq item-list (broadcast item-list '=get prop))
    ;; remove the list and process the MLNs
    (setq item-list
          (remove nil
                  (mapcar #'(lambda (xx) 
                              (mln::filter-language xx *language*))
                    (mapcar #'car item-list))))
    ;; make a list of hyphenated names
    (setq item-list
          ;      (mapcar #'(lambda(xx) (select-norm (car xx))) item-list))
          (mapcar #'car item-list))
    ;; sort the result, include no choice as first element
    ;; copy-tree is used to make sure that sort does not break the original
    ;; structures
    (cons "" (sort (copy-tree item-list) #'string-lessp))))

#|
(select-concept nil nil "label")
("" "Administration" "Aerospace" "Agent" "Application" "Architecture"
 "Automotive_Industry" "Big_Data" "Business" "Business_Process" ...)
|#
;;;----------------------------------------------------- SELECT-CURRENT-LANGUAGE

(defun select-current-language (val)
  "uses a phony property to display current language"
  (declare (ignore val)(special *language*))
  (format nil "~A" *language*))

;;;------------------------------------------------------- SELECT-CURRENT-SOURCE

(defun select-current-source (mln)
    "returns the value associated to the current language except if it is English"
  (declare (special *language*))
  (if (eql *language* :en)
      " " 
    (mln::extract-to-string mln :language *language*)))

;;;---------------------------------------------------------- SELECT-CURRENT-TAG

(defun select-current-tag (mln)
  "returns the value associated to the current language except if it is English"
  (declare (special *language*))
  (if (eql *language* :en)
      " " 
    (mln::extract-to-string mln :language *language*)))

;;;------------------------------------------------------- SELECT-ENGLISH-SOURCE

(defun select-english-source (mln)
  "extract the English value of the source pointer"
  (mln::extract-to-string mln :language :en))

;;;---------------------------------------------------------- SELECT-ENGLISH-TAG

(defun select-english-tag (mln)
  "extracts English value from a layout MLN"
  (mln::extract-to-string mln :language :en))

;;;------------------------------------------------------------- SELECT-LANGUAGE

(defun select-language (class-ref name prop)
  "returns the list of possible languages"
  (declare (special moss::*language-tags*)(ignore class-ref name prop))
  (let ((item-list (remove :unknown moss::*language-tags*)))
    (setq item-list (mapcar #'(lambda(xx) (format nil "~A" xx)) item-list))
    (cons "" (sort (copy-tree item-list) #'string-lessp))))

#|
(select-language nil nil nil)
("" "BR" "CN" "EN" "ES" "FR" "IT" "JP" "PL" "PT")
|#
    
;;;----------------------------------------------------------------- SELECT-NORM

(defun select-norm (text)
  "function that removes spaces and replace them by an underscore"
  (substitute #\_ #\space text))
;(string-capitalize (moss::%string-norm text)))

#|
(select-norm "Computer-supported collaborative work in Design")
"Computer-supported_collaborative_work_in_Design"
|#
;;;------------------------------------------------------------ SELECT-ROLE-TYPE

(defun select-role-type (class-ref name prop)
  "returns the list of the different role types in English"
  (declare (ignore class-ref name prop)(special *language*))
  (let ((role-type-list (access '("Role type")))
        (*language* :EN))
    (cons " " (mapcar #'car (broadcast role-type-list '=summary)))))

#|
(select-role-type nil nil nil)
(" " "includes" "similar to")
|#
;;;------------------------------------------------------------ SHOW-ANNOTATIONS

(defun show-annotations (ip)
  "builds an HTTP pointer to call the page"
  (declare (ignore ip))
  "annotations")

;;;------------------------------------------------------- SHOW-CURRENT-LANGUAGE

(defun show-current-language ()
  (declare (special *language*))
  (list (format nil "~A" *language*)))

;;;=============================================================================
;;;================================ SKILLS =====================================
;;;=============================================================================

;;;========================================================================= SKILL
;;;                          CREATE-OBJECT
;;;===============================================================================
;;; transfers the message to the owner of the referenced class
;;; args contain e.g. (:DATA "CSCWD CONCEPT" ("label" . "cat-test"))))

(defskill :create-object :CSCWD-ONTO
  :static-fcn static-create-object)

(defun static-create-object (agent message args)
  "create the requested objects.
Arguments:
   ...
   args: ((:data <class ref> <data>))
Return:
   whatever the staff agent returns."
  (declare (ignore message))
  (format t "~%;--- args: ~S" args)
  (let* ((data (getvl args :data))
         (language (getvl args :language))
         (class-ref (car data))
         id message-list)
    ;; language is extracted from the message and set as a global variable for
	;; further processing
    (format t "~%;--- class-ref: ~S" class-ref)
    (set-language language)

    (cond
     ((equal+ class-ref "CSCWD-Concept")
      (multiple-value-setq (id message-list)
        (create-cscwd-concept agent args)))
     ((equal+ class-ref "Role")
      (multiple-value-setq (id message-list)
        (create-role agent args)))
     ((equal+ class-ref "Annotation")
      (multiple-value-setq (id message-list)
        (create-annotation agent args)))
     ((equal+ class-ref "Role-Type")
      (multiple-value-setq (id message-list)
        (create-role-type agent args)))
     (t
      (return-from static-create-object
        (static-exit agent :failure 
                     :reason (list "Can't create such objects...")))))
    
    (if id
        (static-exit agent (list id message-list))
      (static-exit agent :failure 
                   :reason (list (format nil "~{~A~^, ~}" message-list))))
    ))

;;;========================================================================= SKILL
;;;                              GET-ANNOTATIONS
;;;===============================================================================
;;; builds an HTML string to display into a textarea

(defskill :get-annotations :cscwd-onto
	:static-fcn static-get-annotations)

(defun static-get-annotations (agent message args)
  (declare (ignore message))
  (let ((ref (cdr (assoc :data args)))
        (result "*no annotations yet* ")
        annotation-list id)
    ;; read object id
    (when ref
      (setq id (read-from-string ref nil nil))
      (setq annotation-list (send id '=get "annotation"))
      (when annotation-list
        (setq result
              (format nil "~{~{~A~}~%~}"
                (reverse (broadcast annotation-list '=summary))))
        )
      )
    (static-exit agent result)))

#|
(static-get-annotations sa_cscwd-onto nil '((:data . "$E-CSCWD-CONCEPT.70")))
"Barthès : Jean-Paul (30/10/2014): Annotation 2 à fin de test
Barthès : Jean-Paul (30/10/2014): Sample annotation 1 for tests
"
NIL

(static-get-annotations sa_cscwd-onto nil '((:data . "$E-CSCWD-CONCEPT.1")))
"*no annotations yet* "
NIL
|#
;;;========================================================================= SKILL
;;;                            GET-ALL-CONCEPTS
;;;===============================================================================
;;; this kill is used to retrieve all concepts to feed the pop up window

(defskill :get-all-concepts :cscwd-onto
	:static-fcn static-get-all-concepts)

(defun static-get-all-concepts (agent message args)
  (declare (ignore message)(special *language*))
  (let (result language)
    ;; get language
    (setq language (cdr (assoc :language args)))
    (omas::set-language language)
    (format t "~%; static-get-all-concepts / language: ~S" *language*)
    ;; get all concepts
    (setq result (cdr (select-concept "CSCWD Concept" nil "label")))
    (static-exit agent result)))

#|
(static-get-all-concepts sa_cscwd-onto :dummy '((:language . :EN)))
("Administration" "Aerospace" "Agent" "Application" "Architecture"
 "Automotive Industry" "Big Data" "Business" "Business Process" "Cloud" ...)
NIL
|#

;;;========================================================================= SKILL
;;;                            GET-ALL-CONCEPTS-JSON
;;;===============================================================================
;;; this skill is used to retrieve all concepts in the json format
;;; used for rendering (mfk)

(defskill :get-all-concepts-json :cscwd-onto
	:static-fcn static-get-all-concepts-json)

(defun static-get-all-concepts-json (agent message args)
  (declare (ignore message))
  (let (result language)
    ;; get language
    (setq language (cadr (assoc :language (cdr (assoc :data args)))))
    (setq *language* language)
    (format t "~%@static-get-all-concepts-json language = ~S" language)
    ;; get all concepts
    (setq result (access '("cscwd-concept")))
    (when result
      (setq result 
        ;; convert concepts to the json format (easily parsed in the web environment)
        (moss-list-to-json (fill-pattern result '("CSCWD Concept" ("label"))) language)
      )
    )
    (static-exit agent result) 
  )
)

;;;========================================================================= SKILL
;;;                              GET-CONCEPT-JSON
;;;===============================================================================
;;; this skill is used to retrieve a concept in the json format
;;; used for rendering (mfk)

(defskill :get-concept-json :cscwd-onto
	:static-fcn static-get-concept-json)

(defun static-get-concept-json (agent message args)
  (declare (ignore message))
  (let (label language result)
    ;; get language
    (setq language (cadr (assoc :language (cdr (assoc :data args)))))
    (setq *language* language)
    (format t "~%@static-get-concept-json language = [~S]" language)
    (setq label (cadr (assoc :label (cdr (assoc :data args)))))
    (format t "~%@static-get-concept-json label = [~S]" label)   
    ;; get the specified concept
    (setq result (access `("CSCWD Concept" ("label" :is ,label))))
    (format t "~%@static-get-concept-json result = [~S]" result)
    (when result
      (setq result 
            ;; convert concepts to the json format (easily parsed in the web environment)
            (moss-list-to-json 
             (marshall-moss-object result 
                                   '($S-CSCWD-CONCEPT-SUBSUMING-CONCEPT $S-ROLE-FROM $S-ROLE-TARGET))
             language)
          )
      )
    (static-exit agent result)
    )
  )

;;;========================================================================= SKILL
;;;                          MODIFY-OBJECT
;;;===============================================================================
;;; this skill is a handle onto the OMAS Web editing process. It takes an object
;;; description of the object to modify and a set of parms, i.e. pairs describing
;;; the edited Web page. It then should update the original object and save the
;;; result

(defskill :modify-object :CSCWD-ONTO
  :static-fcn static-modify-object)

(defun static-modify-object (agent message args)
  "modify the specified object.
Arguments:
   ...
   args: ((:data <class ref> <object-to-modify> <parms>))
Return:
   a list (<id-ref> <message-list>)."
  (declare (ignore message))
  (format t "~%;--- args: ~S" args)
  (let* ((data (getvl args :data))
         (class-ref (car data))
         id-ref message-list)
    (format t "~%;--- class-ref: ~S" class-ref)
    (cond
     
     ((equal+ (moss::%string-norm class-ref) "CSCWD-CONCEPT")
      (multiple-value-setq (id-ref message-list)
        (modify-cscwd-concept agent args)))
     
     ((equal+ class-ref "ROLE-TYPE")
      (multiple-value-setq (id-ref message-list)
        (modify-role-type agent args)))
     )
    
    (format t "~%;--- id-ref: ~S" id-ref)
    (format t "~%;--- message-list ~S" message-list)
    (if id-ref
        (static-exit agent (list id-ref message-list))
      (static-exit agent :failure 
                   :reason (list (format nil "~{~A~^, ~}" message-list))))
    )
  )

;;;=============================================================================
;;;                        WEB-EDIT-USER-ACTION
;;;======================================================================= skill
;;; called by the server for dipslaying annotations or annig a new annotation or
;;; refreshing the page, e.g. with a different non English language

(defskill :web-edit-user-action :CSCWD-ONTO
  :static-fcn static-web-edit-user-action
  )

(defun static-web-edit-user-action (agent message args)
  "function to handle annotations or to refresh the page.
Arguments:
   agent: CSCWD-ONTO
   message: unused
   args: a list of pairs corresponding to the fields of the edit page, e.g.
        ((:data <class-ref> <parms> <ref> <obj-desc>)
Return:
   :done."
  (declare (ignore message))
  (let* ((data (cdr (assoc :data args)))
         (parms (cadr data))
         (action (get-field "user-action" parms))
         )
    (format t "~%---args---")
    (format t "~%---parms:~% ~S"  parms)
    
    (cond
     ;; display annotations
     ((equal+ action "annotations")
      )
     
     ;; add an English annotation
     ((equal+ action "add-English-Annotation")
      )
     
     ;; add a non English annotation
     ((equal+ action "add-non-English-annotation")
      )
     
     ;; refresh page
     ((equal+ action "refresh-page")
      )
     )
    ;; get out
    (static-exit agent :done)))
	

;;;=============================================================================
;;;================================ GOALS ======================================
;;;=============================================================================

;;; ================================ goal section ==============================
;;; defgoal: macro that builds a goal structure for an agent.
;;;Arguments:
;;;   goal-name:              name of the goal to be used to create task-ids
;;;   agent-name:             name of the agent to receive the goal
;;;keys arguments:
;;;   mode:                   activation mode (:rigid or :flexible)
;;;   type:                   type of goal (:permanent :cyclic :1-shot)
;;;   period:                 period for cyclic goals (default is 20s)
;;;   expiration-date:        date at which the goal dies
;;;   expiration-delay:       time to wait until we kill the goal
;;;   activation-date:        date at which the goal should fire (default: now)
;;;   activation-delay:       time to wait before activating the goal
;;;   status:                 waiting, active, dead,... 
;;;                              may not be useful with activation
;;;   goal-enable-fcn:        function checking the goal enabling conditions
;;;   script:                 script describing the goal, name of a function 
;;;                              returning a list of messages
;;;Return:
;;;   goal-name.

;;; defgoal must be called last since the goal takes effect immediately, thus 
;;; the enable function must be defined
;;; return T to enable the goal NIL to inhibit it

;;;=============================================================================
;;; 
;;;                             initial conditions 
;;;
;;;=============================================================================

;;; when the agent is loaded it checks for the existence of a "next issue" paper
;;; if not there, it creates it

#|
(deffact SA_SAAA <any value> <key>)
|#

:EOF