﻿;;;-*- Mode: Lisp; Package: "XXXXX" -*-
;;;===============================================================================
;;;14/08/03
;;;             SHARED ENGLISH DIALOG (file SHARED-DIALOG-EN.lisp)
;;;             Copyright Barthes@UTC, 2014
;;;
;;;=============================================================================== 

;;; This file contains the dialog shared by English speaking PAs. It is loaded
;;; into their package when creating the agent
#|
2014
 0303 Creation by cloning STEVENS-DIALOG
 0528 adding tables for displaying categories and  indexes
 0531 add-category returns a warning now
 0803 tests with Stevens
|#

(in-package :XXXXX)

(format t "~%;*** Loading XXXXX dialogs, package is: ~S" *package*)

(defparameter *empty-words* 
  '("a" "an" "the" "that" "those" "these" "what" "is" "are"))

(defparameter *topic-words*
  '("topic" "label" "add"))


;;;================================================================================
;;;
;;;                              Sub-Conversations
;;;
;;;================================================================================

;;; Sub-conversations are defined by a conversation header, that points to the
;;; entry state.
;;; There are two output states:
;;;   SUCCESS: the result is stored into the RESULTS slot of the state-context
;;;   FAILURE: no result could be achieved
;;; In case of bad error there is a throw to an :error label
;;; data are obtained from the HAS-DATA slot of the context object

;;;================================================================================
;;;
;;;                          NEWS CONVERSATIONS
;;;
;;;================================================================================

;;; News conversations deal with the following tasks:
;;;     - adding a category (returns a warning message)
;;;     - adding a piece of information (news item)
;;;     - adding an index (can't add synonyms)
;;;     - canceling a subscription using its tag
;;;     - deleting a keyword (returns warning message)
;;;     ? deleting a private news item
;;;     - getting info on some category from publisher (uses get-news-item)
;;;     - getting info on some indexes from publisher
;;;     - getting info from some author
;;;     - getting the last news from publisher
;;;     - making a private news item public (to do)
;;;     - obtaining the list of categories
;;;     - obtaining the list of indexes
;;;     - subscribing on categories and/or indexes and/or authors
;;;     - viewing local news items (to do)
;;;     - viewing the last local news
;;;     - viewing subscriptions
;;; + : still to be implemented (dialog has to be modified)
;;; ? : untested

;;;===============================================================================
;;;
;;;                      NEWS: ADD CATEGORY CONVERSATION
;;;
;;;===============================================================================
;;; Adding a category can only be done by the administrator

(moss::defsubdialog  
    _add-category-conversation
    (:label "News: add category conversation")
  (:explanation 
   "Master is adding a new category for grouping news items.")
  (:states _add-category-conversation ; required by defstate
           _nac-new-entry-state)
  )

(defun print-categories (data web-flag)
  (declare (ignore web-flag))
  (omas::assistant-display-text 
   PA_XXXXX
   (format nil "Sorry, you are not allowed to add a new category, ~
                please contact the NEWS application administrator. ~
                Available categories at PUBLISHER's are:~% ~A" data)))

;;;------------------------------------------------------------- (NAC) ENTRY-STATE

(defstate 
    _nac-new-entry-state
    (:entry-state _add-category-conversation)
  (:label "NEWS: add category dialog entry state")
  (:explanation "Send a message saying that it is not possible.")
  (:text
    "Sorry, you are not allowed to add a new category, ~
     Please contact the NEWS application administrator.")
  ;; clear memory slots
  (:transitions
   (:always :success))
  )

#| 
;;;--------------------------------------------------------------- FORMAT-CATEGORY

(defun format-categories (category-list web-flag)
  "produces either a string to print into a window pane or part of a web page if ~
   the value of the web-flag (second arg) is true."
  (if web-flag
      (format-category-html category-list)
    (format-category-text category-list)))

;;;---------------------------------------------------------- FORMAT-CATEGORY-HTML

(defUn format-category-html (string-list)
  "returns a list of categories" 
  (let ((prop-titles '(("label" "  Categories"))))
    ;; printing the list of categories in a single table entry
    (omas::make-html-table prop-titles 
                           `((("label" ,(format nil "~{~A~^, ~}" string-list))))
                           :header-color "green" :text-color "white"
                           :body-bgcolor "white")))

;;;---------------------------------------------------------- FORMAT-CATEGORY-TEXT

(defUn format-category-text (string-list)
  "print a list of categories in alphabetical order"
  (omas::assistant-display-text 
   XXXXX::PA_XXXXX
   (format nil "~{~A~^ / ~}" 
     (sort string-list #'string-lessp))))
|#
#|
;;;================================================================================
;;;
;;;                      NEWS: ADD CATEGORY CONVERSATION
;;;
;;;================================================================================
;;; this subdialog is used to create a new category

(moss::defsubdialog  
    _add-category-conversation
    (:label "News: add category conversation")
  (:explanation 
   "Master is adding a new category for grouping news items.")
  (:states _add-category-conversation ; required by defstate
           _nac-new-entry-state
           _nac-add
           _nac-check-local-category
           _nac-confirmed
           _nac-sorry)
  )

;;;===============================================================================
;;;               NEWS: ADD CATEGORY SERVICE FUNCTIONS
;;;===============================================================================

(defun print-category-exists (data)
  "get a multilingual index in a string, prints it."
  (if data
      (let ((mln (read-from-string (car data) nil nil)))
        (omas::assistant-display-text 
         PA_XXXXX
         ;; must be a multilingual name
         (if (mln::mln? mln)
             (format nil "Category ~A already exists."
               (mln::extract mln :language :en))
           (format nil "I can't figure out what was returned by the staff agent,~
                        namely: ~S" data))))))
|#
#|
(print-category-exists 
 '("(:FR \"conférence\" :EN \"conference\")"))
 assistant moss::display-text /text: "Category conference already exists."
:DONE
|#
#|
;;;===============================================================================
;;;                       NEWS: ADD CATEGORY STATES
;;;===============================================================================
	
;;;------------------------------------------------------------- (NAC) ENTRY-STATE

(defstate 
    _nac-new-entry-state
    (:entry-state _add-category-conversation)
  (:label "NEWS: add category dialog entry")
  (:explanation "Get category new name.")
  (:question
   "Adding a new category is an important decision that will chage the global ~
	   structure of the system. Are you sure you want to add a new category?")
  ;; clear memory slots
  (:clear-facts :label)
  (:transitions
   (:yesp :target _nac-confirmed)
   (:otherwise :failure))
  )

;;;--------------------------------------------------------------------- (NAC) ADD

(defstate 
    _nac-add
    (:label "NEWS: send to staff to add")
  (:explanation "Ask NEWS to create the category.")
  ;; clean answer area
  (:clear-fact :answer)
  (:send-message :to :XXXXX-NEWS :self PA_XXXXX :action :add-category
                 :args 
                 ;; category should be a list because it is one when using 
                 ;; a language different from English
                 `(((:data ,(read-fact moss::conversation :label))
                    (:language . :EN))))
  (:execute (move-fact moss::conversation :from :raw-input :to :answer))
  (:transitions
   (:on-failure :target _nac-sorry)
   (:otherwise :display-answer :success))
  )

;;;---------------------------------------------------- (NAC) CHECK-LOCAL-CATEGORY

(defstate
    _nac-check-local-category
    (:label "NEWS: checking local category")
  (:explanation "Ask NEWS staff for existing local category.")
  (:send-message :to :XXXXX-NEWS :self PA_XXXXX :action :check-category
                 ;; Recall gets data stored in FACT/LABEL
                 :args 
                 `(((:data . ,(read-fact moss::conversation :label)))))
  (:transitions
   (:on-failure :target _nac-add)
   (:otherwise :print-answer #'print-category-exists :success))
  )

;;;--------------------------------------------------------------- (NAC) CONFIRMED

(defstate
    _nac-confirmed
    (:label "NEWS: getting category label")
  (:explanation "Ask master for a category label")
  (:question
   ("What name?"
    "Name?"
    "What is the label (name) of the category you want to add?"))
  (:execute (move-fact moss::conversation :from :raw-input :to :label))
  (:transitions
   (:always :target _nac-check-local-category))
  )

;;;-------------------------------------------------------------------- (NAC) SORRY

(defstate 
    _nac-sorry
    (:label "Publish failure")
  (:explanation "There was an error while publishing the category.")
  (:text "Sorry, the NEWS agent could not create the specified category for some ~
          reason.")
  (:transitions (:failure))
  )
|#
;;;================================================================================
;;;
;;;                   NEWS: ADD GIVEN index CONVERSATION
;;;
;;;================================================================================
;;; this subdialog is a different entry point to the add-keyword conversation

(defsubdialog 
    _add-given-index-conversation
    (:label "News: add index conversation")
  (:explanation 
   "Master is adding a new index for indexing news items.")
  (:states _add-given-index-conversation; required by defstate
           _nak-given-entry-state
           )
  )

;;;================================================================================
;;;
;;;                      NEWS: ADD INDEX CONVERSATION
;;;
;;;================================================================================
;;; this subdialog is used to create a new index
;;; Done as follows:
;;; 1. ask for a new index
;;; 2. check if it exists already locally; if so, quit, success
;;; 3. send info to PUBLISHER, if down, create order to update
;;; 

(defsubdialog 
    _add-index-conversation
    (:label "News: add index conversation")
  (:explanation 
   "Master is adding a new index for indexing news items.")
  (:states _add-index-conversation; required by defstate
           _nak-new-entry-state ; stp 1
           _nak-add ; 9
           _nak-check-local-index ; 2
           _nak-sorry
           )
  )

;;;===============================================================================
;;;                   NEWS: ADD INDEX SERVICE FUNCTIONS
;;;===============================================================================


(defun print-index-exists (data)
  "prints a message telling master that the index already exists."
  (declare (ignore data))
  (omas::assistant-display-text 
   PA_XXXXX
   (format nil "This index already exists.")))

;;;===============================================================================
;;;                       NEWS: ADD INDEX STATES
;;;===============================================================================
	
;;;------------------------------------------------------- (ADD-INDEX) ENTRY-STATE

(defstate 
    _nak-new-entry-state
    (:entry-state _add-index-conversation)
  (:label "NEWS: add index dialog entry")
  (:explanation "Get index name.")
  (:clear-facts :index :english-index)
  (:question "What keyword(s) or index(es) do you want to add?")
  (:transitions 
   (:always :exec (copy-fact :raw-input :index)
            :target _nak-check-local-index))
  )

;;;------------------------------------------------- (ADD-INDEX) GIVEN-ENTRY-STATE
;;; this second entry point is used when specifying indexes while adding a news
;;; item and when the index does not exist.

(defstate 
    _nak-given-entry-state
    (:entry-state _add-given-index-conversation)
  (:label "NEWS: add index dialog entry")
  (:explanation "Go create index. index must be in FACT/index.")
  (:transitions 
   (:always :target _nak-add))
  )

;;;--------------------------------------------------------------- (ADD-INDEX) ADD

(defstate 
    _nak-add
    (:label "NEWS: ask staff to add index")
  (:explanation "Ask NEWS to create the index.")
  (:send-message 
   :to :XXXXX-NEWS :self PA_XXXXX :action :add-index-from-dialog
   ;; get data stored in FACT base
   :args `(((:data . ,(get-fact :index))
            (:language . :en))))
  ;(:execute (print (CAR (READ-FACT MOSS::CONVERSATION :INPUT))))
  (:transitions
   (:on-failure :target _nak-sorry)
   (:otherwise :exec 
               ;; what do we post here anyway?
               (send moss::conversation '=display-text (get-fact :raw-input))
               :success))
  )

;;;------------------------------------------------- (ADD-INDEX) CHECK-LOCAL-INDEX

(defstate
    _nak-check-local-index
    (:label "NEWS: checking local index")
  (:explanation "Ask NEWS staff for existing local index.")
  (:send-message :self PA_XXXXX :to :XXXXX-NEWS :action :check-index
                 ;; Recall gets data stored in FACT/LABEL
                 :args 
                 `(((:data . ,(get-fact :index))
                    (:language . :en))))
  (:transitions
   (:on-failure :target _nak-add)
   (:otherwise :print-answer #'print-index-exists :success))
  )

;;;*------------------------------------------------------------ (ADD-INDEX) SORRY

(defstate 
    _nak-sorry
    (:label "Publish failure")
  (:explanation "It was an error while creating the index.")
  (:text "Sorry the NEWS agent could not create the specified index.")
  (:transitions (:always :failure))
  )


;;;X===============================================================================
;;;
;;;                      NEWS: ADD-INFO CONVERSATION
;;;
;;;================================================================================

;;; this subdialog is used to record a piece of information into the local NEWS
;;; database, as managed by the NEWS staff agent.

(defsubdialog 
    _add-info-conversation
    (:label "News: add info conversation")
  (:explanation 
   "Master is trying to record a new piece of info.")
  (:states _add-info-conversation; required by defstate
           _nai-entry-state
           _nai-check-category
           _nai-get-category
           _nai-get-content
           _nai-get-indexes
           _nai-get-title
           _nai-indexes?
           _nai-privacy
           _nai-publish
           _nai-select-index
           _nai-select-category
           _nai-sorry)
  )

;;;*------------------------------------------------------- (ADD-INFO) ENTRY-STATE

(defstate 
    _nai-entry-state
    (:entry-state _add-info-conversation)
  (:explanation 
   "Standard entry for adding a piece of information.")
  (:label "NEWS: add info dialog entry")
  (:clear-facts :news-section :news-title :news-content)
  (:transitions (:always :target _nai-get-title)))

;;;*---------------------------------------------------- (ADD-INFO) CHECK-CATEGORY

(defstate
    _nai-check-category
    (:label "NEWS: check category")
  (:explanation 
   "PA asks staff about existing category. If non existing, prints the list of ~
    existing categories.")
  (:send-message :to :XXXXX-NEWS :self PA_XXXXX :action :check-category
                 :args `(((:language . :fr) 
                          (:data . ,(get-fact :news-section)))))
  (:transitions
   (:on-failure
    ;; go print the list of categories
    :sub-dialog _get-all-categories-conversation
    :success _nai-select-category
    ;; should not be a failure unless NEWS staff is dead
    :failure _nai-sorry)
   (:otherwise :target _nai-get-content)
   )
  )

;;;*------------------------------------------------------ (ADD-INFO) GET-CATEGORY

(defstate 
    _nai-get-category
    (:label "Ask master in which category we want to put the news item.")
  (:explanation "Obtained title, ask for category")
  (:question-no-erase 
   "In what category (topic) do you want to file your item?")
  (:transitions
   (:patterns (("?")
               (* "topics" *)
               (* "categories" *)
               (* "category" *)
               (* "topic" *)
               (* "subject" *))
              :sub-dialog _get-all-categories-conversation
              :success _nai-select-category
              :failure _nai-sorry)
   (:otherwise 
    :exec (copy-fact :raw-input :news-section) ; why copying to :news-section?
    :target _nai-check-category))
  )


;;;*------------------------------------------------------- (ADD-INFO) GET-CONTENT

(defstate 
    _nai-get-content
    (:label "Ask master the content.")
  (:explanation "Obtain category, ask for content")
  
  (:execute-preconditions 
   ;; include mark in the agent structure
   ;; this won't work for web access (no window)
   (setf (omas::pass-every-char 
          (car (has-moss-input-window moss::conversation))) t)
   ;; include :verbatim flag into the FACT base (?)
   (set-fact :verbatim t)
   )
  (:question-no-erase
   "Write the content of the news, then click terminate or type \"?.\".")
  (:answer-type :answer)
  (:execute 
   (progn
     ;; Store category in agent´s short term memory						   
     (copy-fact :input :news-content)
     ;; remove :verbatim
     (set-fact :verbatim nil)
     ))
  (:transitions
   (:always :target _nai-indexes?)))

;;;*------------------------------------------------------ (ADD-INFO) GET-INDEXES

(defstate 
    _nai-get-indexes
    (:label "Master wants to know indexes.")
  (:explanation "Obtain indexes, and go check/create them")
  (:question-no-erase 
   "Give now indexes separated by commas. If they do not exist I'll ask ~
   you if you want to create them. To view all indexes simply say \"show.\"")
  (:answer-type :answer)
  (:transitions
   (:patterns
    ((* "show" *))
    :target 
    :sub-dialog _get-all-indexes-conversation
    :success _nai-select-index
    :failure _nai-index-sorry)
   (:otherwise 
    :exec 
    (progn
      ;; Store indexes (a string with possible commas) into a safe place						   
      (copy-fact :raw-input :news-indexes)
      ;; and copy for sub dialog
      (copy-fact :raw-input :index-list)
      ;; reset checked indexes
      (set-fact :checked-indexes nil)
      )
    ;; and call the checking process
    :sub-dialog _check-create-indexes-conversation
    :success _nai-privacy
    :failure _nai-privacy))
  )

;;;*--------------------------------------------------------- (ADD-INFO) GET-TITLE

(defstate 
    _nai-get-title
    (:label "NEWS: get title")
  (:explanation "Ask for the title of the new.")  
  (:question-no-erase "Please give a title to the news item.")
  (:answer-type :answer)
  (:execute (copy-fact :raw-input :news-title)) 
  (:transitions
   (:always  :target _nai-get-category)))

;;;*--------------------------------------------------------- (ADD-INFO) INDEXES?

(defstate 
    _nai-indexes?
    (:label "Ask master about indexes.")
  (:explanation "Ask for indexes")
  (:question-no-erase "Do you want to specify indexes?")
  (:transitions
   (:no :target _nai-privacy)
   (:otherwise :target _nai-get-indexes))
  )

;;;*----------------------------------------------------------- (ADD-INFO) PRIVACY

(defstate 
    _nai-privacy
    (:label "NEWS: get privacy")
  (:explanation "Ask if the news is private.")  
  (:question "Do you want to publish this news item?")
  (:transitions
   (:patterns
    ((* "error" *)
     (* "go" "back" *))
    :target _nai-indexes?)
   (:no
    :exec (set-fact :news-privacy t) :target _nai-publish)   
   (:otherwise 
    :exec (set-fact :news-privacy nil) :target _nai-publish))
  )

;;;*----------------------------------------------------------- (ADD-INFO) PUBLISH

(defstate 
    _nai-publish
    (:label "NEWS: Ask master the content.")
  (:explanation "Ask NEWS to create the news")
  (:send-message
   :to :XXXXX-NEWS :self PA_XXXXX :action :add-news-item-from-dialog
   ;; Recall gets data stored in FACTS
   :args `(((:data 
            ("title" ,(get-fact :news-title))
            ("category"  ,(get-fact :news-section))
            ("text" . ,(get-fact :news-content))
            ("indexes" . ,(get-fact :checked-indexes))
			;; in case there are several "Barthès" give first name
			("author" ("person" ("name" :is "Barthès")
			                    ("first name" :is "Jean-Paul")))
             ;("author"  "Barthès")
            ,@(if (get-fact :news-privacy)
                  `(("private" ,(get-fact :news-privacy))))
             )
            (:language . :en))))
  (:transitions
   (:on-failure :target _nai-sorry)
   (:otherwise :exec 
               (send moss::conversation '=display-text (get-fact :raw-input))
               :success))
  )

;;;*--------------------------------------------------- (ADD-INFO) SELECT-CATEGORY

(defstate
    _nai-select-category
    (:label "NEWS: Select Category")
  (:explanation "master must choose one of the terms in the list")
  (:question-no-erase
   "Please select one of the terms or specify: quit.")
  (:transitions
   (:patterns
    (("quit" *)
     ("none" *)
     (* "quit" *))
    :target _nai-sorry)
   (:otherwise 
    :exec (move-fact moss::conversation :from :raw-input :to :news-section)
    :target _nai-check-category))
  )

;;;*------------------------------------------------------ (ADD-INFO) SELECT-INDEX

(defstate
    _nai-select-index
    (:label "NEWS: Select index")
  (:explanation "master must choose one of the terms in the list")
  (:question-no-erase
   "Select one of the expressions or say: quit.")
  (:transitions
   (:patterns
    (("quit" *)
     ("none")
     (* "quit" *))
    :target _nai-sorry)
   (:otherwise 
    :exec 
    (progn
      ;; Store indexes in a safe place						   
      (move-fact moss::conversation :from :raw-input :to :news-indexes)
      ;; and copy for sub dialog
      (move-fact moss::conversation :from :raw-input :to :index-list)
      )
    ;; and call the checking process
    :sub-dialog _check-create-indexes-conversation
    :success _nai-privacy
    :failure _nai-privacy))
  )

;;;*------------------------------------------------------ (ADD-INFO) SHOW-INDEXES

(defstate
    _nai-show-indexes
    (:label "Ask NEWS about indexes")
  (:explanation "Master wants to view the list of indexes")
  (:question-no-erase
   "Give indexes separated by commas. ~%
    If you don't want indexes just type \"none.\"")
  (:transitions 
   (:patterns ((* "none" *)) :target _nai-privacy)    
   (:otherwise 
    :exec 
    ;; Store indexes in a safe place						   
    (move-fact moss::conversation :from :input :to :news-indexes)
    ;; and call the checking process
    :sub-dialog _check-create-indexes
    :success _nai-privacy
    :failure _nai-privacy))
  )

;;;*------------------------------------------------------------- (ADD-INFO) SORRY

(defstate 
    _nai-sorry
    (:label "Publish failure.")
  (:explanation "It was an error while publishing the new.")
  (:text "Sorry, the NEWS agent could not record your text.~%")
  (:reset)
  (:transitions (:failure))
  )

;;;================================================================================
;;;
;;;                   NEWS: CHECK-CREATE INDEXES CONVERSATION
;;;
;;;================================================================================
;;; this subdialog takes a list of strings or a string with indexes separated by
;;; commas. Then, checks if they already exist, if they don't ask user if he wants
;;; to create them. The list of strings must correspond to distinct keywords...
;;; called by the add-info conversation
;;; when returning success, checked indexes (created or existing) are available in
;;; FACTS/:checked-indexes

(defsubdialog
    _check-create-indexes-conversation
    (:label "NEWS: check indexes conversation")
  (:explanation "PA is checking whether proposed indexes exist. If not, offers to ~
    create them.")
  (:states _check-create-indexes-conversation
          _ncck-entry-state
          _ncck-check
          _ncck-create
          _ncck-more?
          _ncck-save-index
          )
  )

;;;*---------------------------------------------- (CHECK-CREATE-INDEX) ENTRY-STATE

(defstate
    _ncck-entry-state
    (:entry-state _check-create-indexes-conversation)
  (:label "NEWS: check create indexes dialog entry")
  (:explanation "check list of indexes")
  (:execute-preconditions
   (let ((text (get-fact :index-list)))
     (if (stringp text)
         (set-fact :index-list (mln::split-val text)))))
  (:transitions
   (:always :target _ncck-more?))
  )

;;;----------------------------------------------------- (CHECK-CREATE-INDEX) CHECK

(defstate
    _ncck-check
    (:label "NEWS: checking first index")
  (:explanation "check the first index in the list")
  (:execute-preconditions
   ;; get the nextt index to process
   (let ((index-list (get-fact :index-list)))
     (set-fact :index (car index-list))
     (set-fact :index-list (cdr index-list))))
  (:send-message 
   :self PA_XXXXX :to :XXXXX-NEWS :action :check-index
   :args `(((:language . :fr)
            (:data . ,(get-fact :index)))))
  (:transitions
   ;; index is not found by staff
   (:on-failure :target _ncck-create)
   ;; index exists (returned as a list of a string) add to checked list
   (:otherwise :target _ncck-save-index))
  )

;;;---------------------------------------------------- (CHECK-CREATE-INDEX) CREATE

(defstate
    _ncck-create
    (:label "NEWS: calling the process to create a new index")
  (:explanation "ask for permission to create index.")
  (:text-exec (format nil "The index ~A does not exist." (get-fact :index)))
  (:question-no-erase
   "Do you want to create it?")
  (:transitions
   (:no :target _ncck-more?) ; go see if there are more
   (:otherwise :sub-dialog _add-given-index-conversation
               :success _ncck-save-index :failure _ncck-more?))
  )

;;;----------------------------------------------------- (CHECK-CREATE-INDEX) MORE?

(defstate
    _ncck-more?
    (:label "NEWS: check-create indexes more? state")
  (:explanation "check list of indexes")
  (:transitions
   ;; if no more indexes, we quit
   (:test (null (get-fact :index-list)) :success)
   (:otherwise :target _ncck-check))
  )

;;;------------------------------------------------ (CHECK-CREATE-INDEX) SAVE-INDEX

(defstate
    _ncck-save-index
    (:label "NEWS: save new index")
  (:explanation "we created the new index, must save it")
  (:execute (let ((val (get-fact :index))
                  (val-list (get-fact :checked-indexes)))
              (set-fact :checked-indexes (append val-list (list val)))))
  (:transitions (:always :target _ncck-more?))
  )

;;;===============================================================================
;;;
;;;                           NEWS: DELETE KEYWORD
;;;
;;;===============================================================================
;;; Deleting a keyword can only be done by the administrator

(defsimple-subdialog "delete-keyword" "_ndk"
  :explanation "user wants to delete a keyword."
  :format-fcn #'print-delete-keywords
  :sorry "Sorry, you are not authorized to delete a keyword (index), this can ~
          only be done by the administrator."
  )

(defun print-delete-keywords (data web-flag)
  (declare (ignore web-flag data))
  (omas::assistant-display-text 
   PA_XXXXX
   (format nil "Sorry, you are not allowed to delete a keyword (index), ~
                because it could be used by other people.
                Please contact the NEWS application administrator.")))

;;;================================================================================
;;;
;;;                         NEWS: DELETE NEWS ITEM
;;;
;;;================================================================================
;;; info is deleted using the info id number
;;; <to do>
;;; this subdialog is used to create a new index

(defsubdialog 
    _delete-news-item-conversation
    (:label "News: add index conversation")
  (:explanation 
   "Master is adding a new index for indexing news items.")
  (:states _delete-news-item-conversation; required by defstate
           _ndni-entry-state
           _ndni-cannot-do
           _ndni-confirm
           _ndni-done
           _ndni-erase
           _ndni-failure
           _ndni-show-info
           _ndni-try-again
           _ndni-sorry)
  )

;;;================================================================================
;;;               DELETE NEWS ITEM CONVERSATION SERVICE FUNCTIONS
;;;================================================================================

;;;------------------------------------------- (DELETE-NEWS-ITEM)- PRINT-INFO-TITLE
;;; gets a list (("info" ("titre" <titre>) ("auteur" <auteur>) ("date" <date>))*)

(defun print-info-title (data)
  "prints a list of titles returned by a call to the staff agent"
  (omas::assistant-display-text 
   PA_XXXXX
   (format nil "~{~A~^~%~}"
     (mapcar #'print-single-info-title data))))

#|
(XXXXX::print-info-title 
 `(("info" ("titre" "Blanche Neige")("auteur" "Disney")("date" ,(get-universal-time)))
   ("info" ("titre" "Zorro")("auteur" "Disney")("date" ,(get-universal-time)))
   ("info" ("titre" "Peau d'Âne")("auteur" "Perrault")("date" ,(get-universal-time)))))
|#   
;;;------------------------------------- (DELETE-NEWS-ITEM) PRINT-SINGLE-INFO-TITLE
	   
(defun print-single-info-title (data)
  (let* ((obj-l (cdr data))
         (title (cadr (assoc "title" obj-l :test #'string-equal)))
         (author (cadr (assoc "author" obj-l :test #'string-equal)))
         (date (cadr (assoc "date" obj-l :test #'string-equal)))
         )
    (if obj-l
        (format nil "~A (~A, ~A)" 
          title author 
          (if date (moss::get-current-date :time date) "?"))
      "*no information*")))

#|
(XXXXX::print-single-info-title 
 `("info" ("titre" "Blanche Neige")("auteur" "Disney")("date" ,(get-universal-time))))
"Blanche Neige (Disney, 28/07/2011)"
|#
;;;================================================================================
;;;                 NEWS: DELETE NEWS ITEM CONVERSATION STATES
;;;================================================================================
;;; Can only delete local news items. Those sent to PUBLISHER are out of control

;;;------------------------------------------------- (DELETE-NEWS-ITEM) ENTRY-STATE

(defstate 
    _ndni-entry-state
    (:entry-state _delete-news-item-conversation)
  (:label "NEWS: removing a news item")
  (:explanation "Get item id.")
  (:question "What is the info ID? (e.g. N-345)")
  (:answer-type :answer)
  (:execute (move-fact moss::conversation :from :raw-input :to :info-id))
  (:transitions 
   (:always :target _ndni-show-info))
  )

;;;--------------------------------------------------- (DELETE-NEWS-ITEM) CANNOT-DO

(defstate
    _ndni-cannot-do
    (:label "NEWS: failed to erase")
  (:explanation "Staff could not erase info.")
  (:question
   "For some reason I could not erase the news item.
	  Do you want to try again?")
  (:transitions
   (:yes :target _ndni-entry-state)
   (:otherwise :failure))
  )
;;;----------------------------------------------------- (DELETE-NEWS-ITEM) CONFIRM

(defstate
    _ndni-confirm
    (:label "NEWS: confirm")
  (:explanation "Ask master to confirm the info to delete.")
  (:question-no-erase
   ("Do you want to remove this news item?"
    "This item?"
    "I erase this item?"))
  (:transitions
   (:yes :target _ndni-erase)
   (:otherwise :target _ndni-try-again))
  )
;;;-------------------------------------------------------- (DELETE-NEWS-ITEM) DONE

(defstate
    _ndni-done
    (:label "NEWS: done")
  (:explanation "Ask if want to erase more.")
  (:question
   ("Item was erase. Do you want to erase more?"
    "Done. More?"
    "OK. Do we erase more?"))
  (:transitions
   (:yes :target _ndni-entry-state)
   (:otherwise :success))
  )

;;;------------------------------------------------------- (DELETE-NEWS-ITEM) ERASE

(defstate
    _ndni-erase
    (:label "NEWS: erase")
  (:explanation "Ask staff to erase news item.")
  (:send-message :to :XXXXX-NEWS :self PA_XXXXX :action :delete-news-item
                 ;; Recall gets data stored in FACT
                 :args `(((:data ,(read-fact moss::conversation :info-id))
                          )))
  (:transitions
   (:on-failure :target _ndni-cannot-do)
   (:otherwise :target _ndni-done))
  )

;;;----------------------------------------------------- (DELETE-NEWS-ITEM) FAILURE

(defstate
    _ndni-failure
    (:label "NEWS: failed to find info")
  (:explanation "Staff could not find info.")
  (:question
   ("I can't find the specific info. Do you want to try again?"
    "Can't find info, maybe already erased...Try again?"))
  (:transitions
   (:yes :target _ndni-entry-state)
   (:otherwise :failure))
  )

;;;--------------------------------------------------- (DELETE-NEWS-ITEM) SHOW-INFO

(defstate
    _ndni-show-info
    (:label "NEWS: show info.")
  (:explanation "Show info item and ask for confirmation")
  (:send-message :to :XXXXX-NEWS :self PA_XXXXX :action :get-news-by-id
                 ;; Recall gets data stored in FACT
                 :args `(((:data . ,(read-fact moss::conversation :info-id))
                          (:pattern "Info" ("title")("author")("date"))
                          )))
  (:transitions
   (:on-failure :target _ndni-failure)
   (:otherwise :print-answer #'print-info-title :target _ndni-confirm))
  )

;;;--------------------------------------------------- (DELETE-NEWS-ITEM) TRY-AGAIN

(defstate
    _ndni-try-again
    (:label "NEWS: try again")
  (:explanation "Wrong info ID. Ask master if we try again.")
  (:question 
   ("We try another ID? (No or new ID)?"
    "Try again?(No or new ID)"))
  (:execute (move-fact moss::conversation :from :raw-input :to :info-id))
  (:transitions
   (:no :reset :failure)
   (:otherwise :target _ndni-show-info))
  )

;;;================================================================================
;;;
;;;                              NEWS: CREATE INFO
;;;
;;;================================================================================
;;; this conversation is a test for calling a web editor page

(defsubdialog 
    _create-info-conversation
    (:label "News: add index conversation")
  (:explanation 
   "Master is adding a new index for indexing news items.")
  (:states _create-info-conversation; required by defstate
           _nci-entry-state
           )
  )

;;;------------------------------------------------------- (CREAT-INFO) ENTRY-STATE
;;; this is the entry and axit state. It sends back info to call the web editor

(defstate 
    _nci-entry-state
    (:entry-state _create-info-conversation)
  (:label "NEWS: calling the editor to create an info")
  (:explanation "Send the right parameters to the editor")
  (:transitions 
   (:exec 
    (let ((index (moss::web-get-gate moss::conversation)))
      ;; should set answer into *answer-list*
      (send moss::conversation '=print-self)
      (moss::web-set-text moss::conversation '(:create "XXXXX-news" "news item"))
      (format t "~%; _create-info entry state/index (gate): ~%  ~S" index)
      (format t "~%; _create-info entry state/*answer-list*: ~%  ~S" 
        omas.web::*web-answer-list*)
      ;; clean up conversation
      (web-clear-gate moss::conversation)
      (web-clear-tag moss::conversation)
      ;; and open gate to let PA return an answer to WEBNEWS
      (omas::web-open-gate index)
      ))
   (:always :success))
  )

;;;================================================================================
;;;
;;;                          NEWS: GET ALL AUTHORS
;;;
;;;================================================================================
;;; this conversation allows to recover information about all authors that wrote
;;; or can write news items. They are listed at the PUBLISHER site

(defsimple-subdialog "get-all-authors" "_gaa"
  :explanation "Master is trying to obtain the list of all authors."
  :from PA_XXXXX :to :PUBLISHER :action :get-all-authors 
  ;:print-fcn #'print-authors
  :sorry "Sorry, I can't find the requested information."
  )
  
;;;================================================================================
;;;
;;;                          NEWS: GET ALL CATEGORIES
;;;
;;;================================================================================
;;; this conversation is intended to print news about a specific topic. It prints 
;;; a list of formatted strings returned by the :get-news-items skill


(defsimple-subdialog "get-all-categories" "_ngac"
  :explanation "Master is trying to obtain the list of categories."
  :from PA_XXXXX :to :XXXXX-NEWS :action :get-all-categories	 
  :format-fcn #'format-categories
  :sorry "Sorry I can't find the list of all categories."
  )

;;;--------------------------------------------------------------- FORMAT-CATEGORY

(defun format-categories (category-list web-flag)
  "produces either a string to print into a window pane or part of a web page if ~
   the value of the web-flag (second arg) is true."
  (if web-flag
      (format-category-html category-list)
    (format-category-text category-list)))

;;;---------------------------------------------------------- FORMAT-CATEGORY-HTML

(defUn format-category-html (string-list)
  "returns a list of categories" 
  (let ((prop-titles '(("label" "  Categories"))))
    ;; printing the list of categories in a single table entry
    (omas::make-html-table prop-titles 
                           `((("label" ,(format nil "~{~A~^, ~}" string-list))))
                           :header-color "green" :text-color "white"
                           :body-bgcolor "white")))

;;;---------------------------------------------------------- FORMAT-CATEGORY-TEXT

(defUn format-category-text (string-list)
  "print a list of categories in alphabetical order"
  (omas::assistant-display-text 
   XXXXX::PA_XXXXX
   (format nil "~{~A~^ / ~}" 
     (sort string-list #'string-lessp))))

;;;================================================================================
;;;
;;;                         NEWS: GET ALL INDEXES
;;;
;;;================================================================================
;;; this conversation is intended to print news about a specific topic. It prints 
;;; a list of formatted strings returned by the :get-news-items skill

;;;------------------------------------------------------------------ PRINT-INDEXES

;;;(defUn print-indexes (string-list)
;;;  "print a list of indexes"
;;;  (omas::assistant-display-text 
;;;   PA_XXXXX 
;;;   (format nil "~{~A~^ / ~}" string-list)))

(defsimple-subdialog "get-all-indexes" "_ngak"
  :explanation "Master is trying to obtain the list of indexes."
  :from PA_XXXXX :to :XXXXX-NEWS :action :get-all-indexes
  :format-fcn #'format-indexes   
  :sorry "Sorry, I can't find the list of news indexes."
  )
;;;----------------------------------------------------------------- FORMAT-INDEXES

(defun format-indexes (index-list web-flag)
  "produces either a string to print into a window pane or part of a web page if ~
   the value of the web-flag (second arg) is true."
  (if web-flag
      (format-index-html index-list)
    (format-index-text index-list)))

;;;-------------------------------------------------------------- FORMAT-INDEX-HTML

(defUn format-index-html (string-list)
  "returns a list of categories" 
  (let ((prop-titles '(("label" "  Indexes"))))
    ;; printing the list of indexes in a single table entry
    (omas::make-html-table prop-titles 
                           `((("label" ,(format nil "~{~A~^, ~}" string-list))))
                           :header-color "green" :text-color "white"
                           :body-bgcolor "white")))

;;;------------------------------------------------------------- FORMAT-INDEX-TEXT

(defUn format-index-text (string-list)
  "print a list of categories in alphabetical order"
  (omas::assistant-display-text 
   XXXXX::PA_XXXXX
   (format nil "~{~A~^ / ~}" 
     (sort string-list #'string-lessp))))

;;;===============================================================================
;;;
;;;                   NEWS: GET ALL LOCAL NEWS CONVERSATION
;;;
;;;===============================================================================

(defsimple-subdialog "get-all-local-news" "_galni"
  :explanation "Master is trying to obtain all local news from NEWS staff."
  :from PA_XXXXX :to :XXXXX-NEWS :action :get-all-local-news 
  :pattern ("info" ("id number")("creation date")("author")("category")("title")
            ("text"))
  :format-fcn #'format-news
  :sorry "Sorry, I can't find any local news."
  )
  
;;;===============================================================================
;;;
;;;                   NEWS: GET LAST LOCAL NEWS CONVERSATION
;;;
;;;===============================================================================
;;; this conversation is intended to show the 5 last private news

(defUn print-news (string-list)
  "print a list of news"
  (omas::assistant-display-text 
   PA_XXXXX 
   (format nil "~{~A ~2%~}" string-list)))

(defsimple-subdialog "get-last-local-news" "_gllni"
  :explanation "Master is trying to obtain last local news from NEWS staff."
  :from PA_XXXXX :to :XXXXX-NEWS :action :get-last-local-news 
  :pattern ("info" ("id number")("creation date")("author")("category")("title")
            ("text"))
  :format-fcn #'format-news
  :sorry "Sorry, I can't find the requested last local news."
  )

;;;===============================================================================
;;;
;;;                      NEWS: GET LAST NEWS CONVERSATION
;;;
;;;===============================================================================
;;; this conversation is intended to show the 5 last public news

(defsimple-subdialog "get-last-news" "_glni"
  :explanation "Master is trying to obtain last news from NEWS staff."
  :from PA_XXXXX :to :PUBLISHER :action :get-last-news 
  :pattern ("info" ("id number")("creation date")("author")("category")("title")
            ("text"))
  :format-fcn #'format-news
  :sorry "Sorry, I can't find the requested last news."
  )

;;;===============================================================================
;;;
;;;                           NEWS: GET NEWS ITEMS
;;;
;;;===============================================================================
;;; this conversation is intended to print news about a specific topic. It prints 
;;; a list of formatted strings returned by the :get-news-items skill

(defsimple-subdialog "get-news" "_gni"
  :explanation "Master is trying to obtain some info from NEWS staff."
  :from PA_XXXXX :to :PUBLISHER :action :get-news-items	 
  :format-fcn #'format-news
  :language :EN
  :pattern ("info" ("id number")("creation date")("author")("category")("title")
            ("text"))
  :sorry "Sorry, I can't find the requested news."
  )

;;;-------------------------------------------------------------------------------
;;;                  GET CONTACT CONVERSATION SERVICE FUNCTIONS
;;;-------------------------------------------------------------------------------

;;;------------------------------------------------------------------- FORMAT-DATE
;;; handles both dates like "15/12/2013" and universal time

(defun format-date (val)
  "used to format the date value for correct printing"
  (if (stringp (car val))
      val
  (list (moss::get-current-date :time (car val)))))

;;;------------------------------------------------------------------- FORMAT-NEWS

(defun format-news (news-list web-flag)
  "produces either a string to print into a window pane or part of a web page if ~
   the value of the web-flag (second arg) is true."
  ;; sort contact-list by country
  (if web-flag
      (format-news-html news-list)
    (format-news-text news-list)))

;;;-------------------------------------------------------------- FORMAT-NEWS-HTML

(defUn format-news-html (string-list)
  "returns a list of news" 
  ;(format nil "~{~A <BR>~}" string-list))
  (let ((prop-titles '(("id number" "ID")
                       ("creation date" "Date" nil format-date)
                       ("author" "Author(s)")
                       ("category" "Category")("title" "Title")
                       ("text" "Content"))))
    (omas::make-html-table prop-titles (mapcar #'cdr string-list) 
                           :header-color "green" :text-color "white"
                           :body-bgcolor "white")))

;;;-------------------------------------------------------------- FORMAT-NEWS-TEXT

(defUn format-news-text (string-list)
  "returns a list of news" 
   (format nil "~{~A ~2%~}" string-list))

;;;===============================================================================
;;;
;;;                       NEWS: GET NEWS ITEMS FROM AUTHOR
;;;
;;;===============================================================================
;;; included in get-news-item


;;;===============================================================================
;;;
;;;                       NEWS: PUBLISH PRIVATE NOTE
;;;
;;;===============================================================================
;;; 
;;; 1. Test pattern of input:
;;;    - if ("publish") or ("publish" "note") or ("publish" "private" *) go to 3
;;;    - otherwise we have ("publish" ?x *); save ?x into :to-check, go to check 
;;; 2. check if :to-check is a note tag of an unpublished note, or if XXXXX/?x is
;;;    one
;;;    - if so go to 8
;;;    - otherwise go to 3             
;;; 3. "You want to publish what? If you want to see all your notes type/say 
;;;     show all (-> 4), otherwise type the id of the note you want to publish
;;;     (-> 2)."
;;; 4. Display the list of all unpublished private notes, ask to select one or
;;;    quit. If one is slelected, go check it (2).
;;; 5. "The data you gave me does not correspond to an unpublished note, if you
;;;    want to quit, type/say quit."
;;; 6. "This tag corresponds to an already published note."
;;; 7. Ask XXXXX-NEWS to publish the note
;;; 8. Post the result

(defsubdialog 
    _publish-note-conversation
    (:label "News: publish note conversation")
  (:explanation 
   "Master wants to publish one of its private notes.")
  (:states _publish-note-conversation; required by defstate
           _npn-entry-state
           _npn-already-published
           _npn-ask-tag
           _npn-check-tag
           _npn-display-and-ask
           _npn-publish
		   _npn-select-tag
           _npn-sorry
           _npn-tag-failure
           )
  )

;;;----------------------------------------------------- (PUBLISH NOTE) ENTRY-STATE

(defstate 
    _npn-entry-state
    (:entry-state _publish-note-conversation)
  (:label "NEWS: publish private node: entry state")
  (:explanation "publish an unpublished private note.")
  ;; we try to extract reference from the input data
  (:transitions
     (:patterns (
	   ("publish")
	   ("publish" "note")
	   ("publish" "private" *)
	   )
	   :target _npn-ask-tag)
	 (:patterns (("publish" ?x))	  
	   :keep ?x :exec (copy-fact '?x :to-check) :target _npn-check-tag)
	 )
   )

;;;----------------------------------------------- (PUBLISH NOTE) ALREADY-PUBLISHED

(defstate _npn-already-published
    (:label "News: Note already published")
  (:explanation "Tell master that the note has already been published and quit.")
  (:text "This note has already been published.")
  (:transitions
   (:always :success))
  )

;;;--------------------------------------------------------- (PUBLISH NOTE) ASK-TAG

(defstate _npn-ask-tag
    (:label "News: Ask note tag")
  (:explanation "Ask master to give the tag identifying the note to publish.")
  (:question "You want to publish what? If you want to see all your notes type/say 
     show all, otherwise type the id of the note you want to publish.")
  (:transitions
   (:patterns (("show" *)) :target _npn-display-and-ask)
   (:otherwise :exec (copy-fact :raw-input :to-check) :target _npn-check-tag))
  )

;;;------------------------------------------------------- (PUBLISH NOTE) CHECK-TAG

(defstate _npn-check-tag
    (:label "Ask XXXXX-NEWS to check note tag")
  (:explanation "Ask staff to check if the current data is that of an unpublished ~
  note.")
  (:send-message 
   :self PA_XXXXX :to :XXXXX-NEWS :action :check-note-tag
   :args `(((:language . :en)
            (:data . ,(get-fact :to-check)))))
  (:transitions 
   (:on-failure :target _npn-tag-failure)
   (:otherwise :target _npn-publish))
  )

;;;------------------------------------------------- (PUBLISH NOTE) DISPLAY-AND-ASK

(defstate _npn-display-and-ask
    (:label "Ask XXXXX-NEWS to display unpublished notes")
  (:explanation "Ask staff to display unpublished notes, and ask for a choice.")
  (:send-message 
   :self PA_XXXXX :to :XXXXX-NEWS :action :get-unpublished-notes :args)
  (:transitions 
   (:always :display-answer :target _npn-select-tag))
  )

;;;--------------------------------------------------------- (PUBLISH NOTE) PUBLISH

(defstate _npn-publish
    (:label "Ask XXXXX-NEWS to publish note")
  (:explanation "We checked that the note id is correct and ask XXXXX-NEWS to ~
                 publish it.")
  (:send-message 
   :self PA_XXXXX :to :XXXXX-NEWS :action :publish-item
   :args `(((:language . :en)
            (:data . ,(get-fact :answer)))))
  (:transitions 
   (:on-failure :target _npn-sorry)
   (:otherwise 
    :exec (send moss::conversation '=display-text 
                (format nil "~{~A~^, ~}" (cadr (get-fact :raw-input))))
    :success))
  )

;;;------------------------------------------------------ (PUBLISH NOTE) SELECT-TAG

(defstate _npn-select-tag
    (:label "News: Select tag")
  (:explanation "Ask master to give the tag identifying the note to publish.")
  (:question "Please select the note you want to publish.")
  (:transitions
   (:patterns (("quit" *)
               ("none")) :failure)
   (:otherwise :exec (copy-fact :input :to-check) :target _npn-check-tag))
  )

;;;----------------------------------------------------------- (PUBLISH NOTE) SORRY

(defstate _npn-sorry
    (:label "News: aborting publishing process.")
  (:explanation "The note cannot be published for some reqson.")
  (:text 
   "Sorry, the note could not be published for some reason.")
  ;; clean facts (the conversation is supposed to be toplevel
  (:clear-all-facts)
  (:transitions
   (:always :failure))
  )

;;;----------------------------------------------------- (PUBLISH NOTE) TAG-FAILURE

(defstate _npn-tag-failure
    (:label "News: Tag failure")
  (:explanation "The given data does not correspond to a note.")
  (:question 
     "Sorry, this does not correspond to a note. If you want to quit say/type ~
	 \"quit\", otherwise we try again.")
  (:transitions
   (:patterns (("quit" *)) :failure)
   (:otherwise :target _npn-ask-tag))
  )


;;;===============================================================================
;;;
;;;                               NEWS: SUBSCRIBE
;;;
;;;===============================================================================
;;; this conversation should allow master to subscribe to NEWS to receive the
;;; monthly journal, info concerning some category or indexes, or info posted by
;;; somebody in particular.
;;; We send a message to PUBLISHER including our name, email address, and type of
;;; subscription
;;; The states are
;;;   - choice of the type of subscription category/indexes, monthly journal, author
;;;   - show name first name and email, ask for email confirmation
;;;   - ask for email if not there
;;;   - change email if not right
;;;   - send acknowledgment and info for unsubscribing
;;; Subscriptions have an ID number

(defsubdialog 
    _subscribe-conversation
    (:label "News: subscribe conversation")
  (:explanation 
   "Master wants to subscribe to receive information.")
  (:states _subscribe-conversation; required by defstate
           _nsi-entry-state
           _nsi-ask-choice
           _nsi-ask-members-for-email
           _nsi-author
           _nsi-category
           _nsi-check-data
           _nsi-check-email
           _nsi-correct-data
           _nsi-get-email
           _nsi-indexes
           _nsi-journal
           _nsi-more?
           _nsi-record-subscription
           _nsi-register
           _nsi-select-category
           _nsi-select-indexes
           _nsi-sorry
           )
  )

;;;================================================================================
;;;                      NEWS:SUBSCRIBE SERVICE FUNCTIONS
;;;================================================================================

;;;-------------------------------------------------------------- EXPAND-CATEGORIES

(defun expand-name-string (string)
  "takes a string formatted like synonyms and builds a sequence of mlns"
  (mapcar #'(lambda (xx) (list :en xx)) (moss::%synonym-explode string)))

#|
(expand-name-string "Sports;Arts")
((:EN "Sports") (:EN "Arts"))
|#
;;;================================================================================
;;;                          NEWS:SUBSCRIBE STATES
;;;================================================================================

;;;-------------------------------------------------------- (SUBSCRIBE) ENTRY-STATE

(defstate 
    _nsi-entry-state
    (:entry-state _subscribe-conversation)
  (:label "NEWS: subscribing")
  (:explanation "subscribe to receive info.")
  ;(:text "Sorry, this feature is not yet available.")
  (:execute-preconditions
   ;; we recover master's name and email from the name of the PA and save data
   ;; into the fact base for later processing
   (let ((subscriber 
          (car (access '("person"
                   ("personal assistant" 
                    ("omas agent" ("ag-name" :is "XXXXX"))))))))
     (if (null subscriber)
         (error "subscription process: XXXXX has no master...")
       (progn
         (set-fact :name (send subscriber '=get "name"))
         (set-fact :first-name (send subscriber '=get "first name"))
         (set-fact :master-email (send subscriber '=get "email"))
         ))))
  (:transitions 
   (:always :target _nsi-ask-choice))
  )

;;;--------------------------------------------------------- (SUBSCRIBE) ASK-CHOICE

(defstate
    _nsi-ask-choice
    (:label "News: ask for choice of subscription")
  (:explanation "Master can subscribe for several kinds of subscriptions.")
  (:question
   ("What kind of subscription do you want? Choices are: monthly journal, specific ~
    category or indexes, specific author."
    "Monthly journal? Category or indexes? Author of news?"))
  (:transitions
   (:patterns
    ((* "journal" *)) :target _nsi-journal)
   (:patterns
    ((* "category" *)) :target _nsi-category)
   (:patterns
    ((* "indexes" *)(* "index" *)) :target _nsi-indexes)
   (:patterns
    ((* "author" *)(* "person" *)) :target _nsi-author)
   (:otherwise :failure))
  )

;;;---------------------------------------------- (SUBSCRIBE) ASK-MEMBERS-FOR-EMAIL

(defstate _nsi-ask-members-for-email
    (:label "Ask MEMBERS for email")
  (:explanation "We don't have master's email and ask MEMBERS to send it to us if ~
   it has one.")
  (:send-message 
   :self PA_XXXXX :to :MEMBERS :action :get-email
   :args `(((:language . :en)
            (:data ("name" ,@(get-fact :name))
                   ("first-name" ,@(get-fact :first-name))
                   ))))
  (:execute (copy-fact :raw-input :master-email))
  (:transitions 
   (:test (get-fact :master-email) :target _nsi-check-data)
   (:otherwise :target _nsi-get-email))
  )

;;;------------------------------------------------------------- (SUBSCRIBE) AUTHOR

(defstate _nsi-author
    (:label "News: Select author for notification")
  (:explanation "Master wants to be notified when somebody is posting something.")
  (:question "please give me the name and/or initials of the author you want to ~
    follow (only one person).")
  (:execute (progn
              (set-fact :subscription :author)
              (copy-fact :raw-input :author)))
  (:transitions
   (:always :target _nsi-record-subscription))
  )
;;;----------------------------------------------------------- (SUBSCRIBE) CATEGORY

(defstate _nsi-category
    (:label "News: selecting a category for notification")
  (:explanation "Master can select several categories.")
  (:execute (set-fact :subscription :category))
  (:transitions
   (:always :sub-dialog _get-all-categories-conversation
              :success _nsi-select-category
            :failure _nsi-sorry))
  )

;;;--------------------------------------------------------- (SUBSCRIBE) CHECK-DATA

(defstate _nsi-check-data 
    (:label "News: checking the info.")
  (:explanation "We ask for confirmation on name and email.")
  (:text-exec 
   (format nil "I have the following information concerning the subscriber:~%
   name: ~A, first-name: ~A, email: ~A" 
     (get-fact :name) (get-fact :first-name)(get-fact :master-email)))
  (:question-no-erase
   ("Is it OK?"
    "Can I use that?"))
  (:transitions
   (:yes :target _nsi-record-subscription)
   (:otherwise :target _nsi-correct-data))
  )

;;;-------------------------------------------------------- (SUBSCRIBE) CHECK-EMAIL

(defstate _nsi-check-email 
    (:label "Check email")
  (:explanation 
   "If an email is not available, asks MEMBERS using the name and first-name. If ~
	  still not found, ask master.")
  (:transitions
   (:test (get-fact :master-email) :target _nsi-check-data)
   (:otherwise :target _nsi-ask-members-for-email)
   )
  )
         
;;;------------------------------------------------------- (SUBSCRIBE) CORRECT-DATA

(defstate _nsi-correct-data
    (:label "News: correcting data for subscriptions")
  (:explanation "Some of the data for the subscription was incorrect.")
  (:question
   "What do you want to modify? email? category? indexes? author?")
  (:transitions
   (:patterns
    ((* "email" *)) :target _nsi-get-email)
   (:patterns
    ((* "category" *)) :target _nsi-category)
   (:patterns
    ((* "indexes" *)) :target _nsi-indexes)
   (:patterns
    ((* "author" *)) :target _nsi-author)
   (:otherwise :target _nsi-sorry))
  )

;;;---------------------------------------------------------- (SUBSCRIBE) GET-EMAIL

(defstate _nsi-get-email
    (:label "News: ask for registration email")
  (:explanation "We need an email to send the subscribed info.")
  (:text-exec
   (format nil "Subscriber is: name: ~{~A~^ ~}, first-name: ~{~A~^ ~}"
     (get-fact :name)(get-fact :first-name)))
  (:question
   ("I need an email address for sending the information. What is it?"
    "Please give me an email address for sending the information."))
  (:execute-preconditions 
   ;; necessary because mails contain periods!
   (setf (omas::pass-every-char (car (has-moss-input-window moss::conversation))) t)
   ;; include :verbatim flag into the FACT base (?)
   (set-fact :verbatim t)
   )
  (:question-no-erase
   "Write the email address, then click terminate button or type \"?.\".")
  (:answer-type :answer)
  (:execute 
   (progn
     (copy-fact :raw-input :master-email)
     (set-fact :verbatim nil))
   )
  (:transitions
   (:always :target _nsi-check-data))
  )

;;;------------------------------------------------------------ (SUBSCRIBE) JOURNAL

(defstate
    _nsi-journal 
    (:label "News: journal subscription")
  (:explanation "Master wants to subscribe to journal.")
  (:execute (set-fact :subscription  :journal))
  (:transitions 
   (:always :target _nsi-register))
  )

;;;------------------------------------------------------------ (SUBSCRIBE) INDEXES

(defstate _nsi-indexes
    (:label "News: selecting indexes for notification.")
  (:explanation "Master wants to be notified on some indexes.")
  (:execute (set-fact :subscription :indexes))
  (:transitions
   (:always :sub-dialog _get-all-indexes-conversation
            :success _nsi-select-indexes
            :failure _nsi-sorry))
  )
 
;;;-------------------------------------------------------------- (SUBSCRIBE) MORE?

(defstate _nsi-more?
    (:label "News: more subscriptions?")
  (:explanation "Asking master if she wants to subscribe to something else.")
  (:question
   "Do you want to make another type of subscription?")
  (:transitions
   (:yes :target _nsi-ask-choice)
   ;; clearing the fact base assumes that conversation is toplevel
   (:otherwise :exec (clear-all-facts moss::conversation) :success))
  )

;;;------------------------------------------------ (SUBSCRIBE) RECORD-SUBSCRIPTION
;;; Here XXXXX must use its ontology to forwar information to the other agents
           
(defstate _nsi-record-subscription
    (:label "News: recording subscription")
  (:explanation "we ask the NEWS agent to do the job")
  (:send-message 
   :self PA_XXXXX :to :XXXXX-NEWS :action :add-subscription
   :args `(((:language . :en)
            (:data  ("name" ,@(get-fact :name))
                   ("first-name" ,@(get-fact :first-name))
                   ("email" ,(get-fact :master-email))
                   ("start date" ,(moss::get-current-date))
                   ("subscription type" ,(get-fact :subscription))
                   ("category" ,@(mln::split-val (get-fact :category)))
                   ("indexes" ,(get-fact :indexes))
                   ("author" ,(get-fact :author))))))
  (:transitions
   (:always :display-answer :target _nsi-more?))
  )
  
;;;----------------------------------------------------------- (SUBSCRIBE) REGISTER

(defstate _nsi-register
    (:label "News: registering subscription")
  (:explanation "Check name, first name of the person asssociated with the PA.
If there is no email go ask for email, otherwise ask for check.")
  (:transitions
   (:test (null (get-fact :master-email)) :target _nsi-get-email)
   (:otherwise :target _nsi-check-data))
  )

;;;---------------------------------------------------- (SUBSCRIBE) SELECT-CATEGORY

(defstate _nsi-select-category
    (:label "NEWS: Select Category for subsciption")
  (:explanation "master must choose one of the terms or more in the list")
  (:question-no-erase
   "Please select some term(s) (separate with semi-columns) or specify: quit.
The choice corresponds to a logical AND.")
  (:transitions
   (:patterns
    (("quit" *)
     ("none" *)
     (* "quit" *))
    :target _nsi-sorry)
   (:otherwise 
    :exec (copy-fact :raw-input :category)
    :target _nsi-record-subscription))
  )

;;;----------------------------------------------------- (SUBSCRIBE) SELECT-INDEXES
;;; indexes can be grouped as AND indexes or OR indexes. Currently we use only
;;; end indexes, knowing that OR indexes can be specified as a separate request
;;; indexes are packed into a string, separated with ";"

(defstate _nsi-select-indexes
    (:label "NEWS: Select indexes for notification")
  (:explanation "master must choose one of the terms or more in the list")
  (:question-no-erase
   "Please select some term(s) separated with commas to make an AND clause, or specify: quit.")
  (:transitions
   (:patterns
    (("quit" *)
     ("none" *)
     (* "quit" *))
    :target _nsi-sorry)
   (:otherwise 
    ;; copy AND clause to FACTS/indexes
    :exec (copy-fact :raw-input :indexes)
    :target _nsi-record-subscription))
  )

;;;-------------------------------------------------------------- (SUBSCRIBE) SORRY

(defstate _nsi-sorry
    (:label "News: aborting subscription process.")
  (:explanation "We can't complete the subscription process.")
  (:text 
   "Sorry, I have to abort the subscription process.")
  ;; clean facts (the conversation is supposed to be toplevel
  (:clear-all-facts)
  (:transitions
   (:always :failure))
  )
  
;;;================================================================================
;;;
;;;                              NEWS: UNSUBSCRIBE
;;;
;;;================================================================================
;;; this conversation allows to cancel a subscription
;;; Canceling a subscription consists in deleting the corresponding subscription
;;; object. A person may have several types of subscriptions corresponding to 
;;; different subscription objects

(defsubdialog 
    _unsubscribe-conversation
    (:label "News: unsubscribe conversation")
  (:explanation 
   "Master wants to unsubscribe to information programs.")
  (:states _unsubscribe-conversation; required by defstate
            _nusi-entry-state
		   _nusi-check
		   _nusi-do-it
		   _nusi-get-ref
		   _nusi-sorry
		   )
  )

;;;------------------------------------------------------ (UNSUBSCRIBE) ENTRY-STATE

(defstate 
    _nusi-entry-state
    (:entry-state _unsubscribe-conversation)
  (:label "NEWS: unsubscribing")
  (:explanation "unsubscribe to receive info.")
  ;; we try to extract reference from the input data
  (:transitions
     (:patterns (
	   ("unsubscribe" "to" ?x)
	   ("cancel" ?x)
	   (* "subscription" "to" ?x)
	   (* "cancel" ?x))
	   :keep ?x :target _nusi-check)
	 (:otherwise :target _nusi-get-ref)))
	 
;;;------------------------------------------------------------ (UNSUBSCRIBE) CHECK

(defstate _nusi-check
    (:label "NEWS: get the subscription reference")
  (:explanation "master must give the reference of the subscription to cancel.")
  (:execute-preconditions
     (let ((ref (car (last (get-fact '?x)))))
	   ;; check reference
	   (cond
	    ((not (stringp ref)) (set-fact nil :reference))
	    ((or (string-equal "JS-" (subseq ref 0 3))
              (string-equal "CS-" (subseq ref 0 3))
              (string-equal "KS-" (subseq ref 0 3))
              (string-equal "PS-" (subseq ref 0 3)))
		 (set-fact :reference ref))
		(t (set-fact :reference nil)))))
  (:transitions
   (:test (get-fact :reference) :target _nusi-do-it)
   (:otherwise :target _nusi-get-ref))
   )

;;;------------------------------------------------------------ (UNSUBSCRIBE) DO-IT
  
(defstate _nusi-do-it
    (:label "News: send unsubscrie request")
  (:explanation "we ask the NEWS agent to do the job")
  (:send-message 
   :to :PUBLISHER :self PA_XXXXX :action :cancel-subscription
   :args `(((:language . :en)
            (:data  ("reference" ,(get-fact :reference))))))
  (:transitions
   (:always :display-answer :success))
  )
  
;;;---------------------------------------------------------- (UNSUBSCRIBE) GET-REF

(defstate _nusi-get-ref
    (:label "NEWS: get the subscription reference")
  (:explanation "master must give the reference of the subscription to cancel.")
  (:question
   ("Please give me the reference of your subscription."
    "What is the reference of the subscription you want to cancel?"
	"Reference?"))
  (:transitions
   (:patterns
    (("quit" *)
     ("none" *)
     (* "quit" *)
     ("forget it"))
    :target _nusi-sorry)
   (:otherwise :exec (copy-fact :raw-input :reference) :target _nusi-do-it))
  )
  
;;;------------------------------------------------------------ (UNSUBSCRIBE) SORRY

(defstate _nusi-sorry
    (:label "News: aborting unsubscription process.")
  (:explanation "We can't complete the unsubscription process.")
  (:text 
   "Sorry, I have to abort the unsubscription process.")
  ;; clean facts (the conversation is supposed to be toplevel
  (:clear-all-facts)
  (:transitions
   (:always :failure))
  )

;;;================================================================================
;;;
;;;                          NEWS: VIEW-SUBSCRIPTIONS
;;;
;;;================================================================================
;;; this dialog sends a request directly to the PUBLISHER agent to view the list
;;; of subscriptions. It must send enough data for locating subscriber.

(defsubdialog 
    _view-subscriptions-conversation
    (:label "News: view subscriptions conversation")
  (:explanation 
   "Master wants to view the list of current subscriptions.")
  (:states _view-subscriptions-conversation; required by defstate
           _nvs-entry-state
           _nvs-get-subscriptions
           _nvs-sorry
           )
  )
;;;================================================================================
;;;                  NEWS: VIEW-SUBSCRIPTIONS SERVICE FUNCTIONS
;;;================================================================================

;;;---------------------------------------------------------------- FORMAT-CATEGORY

(defun format-subscriptions (subscription-list web-flag)
  "produces either a string to print into a window pane or part of a web page if ~
   the value of the web-flag (second arg) is true."
  (if web-flag
      (format-subscription-html subscription-list)
    (format-subscription-text subscription-list)))

;;;----------------------------------------------------------- FORMAT-CATEGORY-HTML

(defUn format-subscription-html (string-list)
  "returns a list of subscriptions" 
  (let ((prop-titles '(("Reference" "reference")
                        ("subscription type" "subscription type")
                        ("category" "Category")
                        ("AND indexes" "And indexes")
                        ("Author" "author"))))
    ;; printing the list of categories in a single table entry
    (omas::make-html-table prop-titles (mapcar #'cdr string-list)
                           :header-color "red" :text-color "white"
                           :body-bgcolor "white")))

;;;----------------------------------------------------------- FORMAT-CATEGORY-TEXT

(defUn format-subscription-text (string-list)
  "print a list of subscriptions"
  (omas::assistant-display-text 
   XXXXX::PA_XXXXX
   (format nil "~{~A~^ / ~}" 
     (sort string-list #'string-lessp))))

;;;================================================================================
;;;                        NEWS: VIEW-SUBSCRIPTIONS STATES
;;;================================================================================

;;;----------------------------------------------- (VIEW-SUBSCRIPTIONS) ENTRY-STATE

(defstate 
    _nvs-entry-state
    (:entry-state _view-subscriptions-conversation)
  (:label "NEWS: view subscriptions")
  (:explanation "ask to view subscriptiosn, get master's info.")
  (:execute-preconditions
   ;; we recover master's name and email from the name of the PA and save data
   ;; into the fact base for later processing
   (let ((subscriber 
          (car (access '("person"
                   ("personal assistant" 
                    ("omas agent" ("ag-name" :is "XXXXX"))))))))
     (if (null subscriber)
         (error "subscription process: XXXXX has no master...")
       (progn
         (set-fact :name (send subscriber '=get "name"))
         (set-fact :first-name (send subscriber '=get "first name"))
         ))))
  (:transitions 
   (:always :target _nvs-get-subscriptions))
  )
  
;;;----------------------------------------- (VIEW-SUBSCRIPTIONS) GET-SUBSCRIPTIONS
           
(defstate _nvs-get-subscriptions
    (:label "News: get subscriptions")
  (:explanation "send message to PUBLISHER to get the list of ubscriptions")
  (:send-message 
   :to :PUBLISHER :self PA_XXXXX :action :view-subscriptions
   :args `(((:language . :en)
            (:data ("name" ,@(get-fact :name))
                   ("first-name" ,@(get-fact :first-name)))
            (:pattern "subscription" ("subscription type")("reference")
                      ("category")("and indexes")("author")))
           ))
  (:transitions
   (:on-failure :target _nvs-sorry)
   (:otherwise :format-answer #'format-subscriptions :success))
  )

;;;----------------------------------------------------- (VIEW-SUBSCRIPTIONS) SORRY

(defstate _nvs-sorry
    (:label "News: aborting viewing subscriptions.")
  (:explanation "We can't get unsubscriptions.")
  (:text 
   "Sorry, I cannot get the list of subscriptions.")
  ;; clean facts (the conversation is supposed to be toplevel
  (:clear-all-facts)
  (:transitions
   (:always :failure))
  )

;;;================================================================================
;;;
;;;                          PRINT HELP CONVERSATION
;;;
;;;================================================================================

;;; this conversation is intended to help the master by giving information:
;;;   - in general (what tasks are available
;;;      "what can you do for me?" "help." "what can I do?"
;;;   - on a particular topic
;;;      "how do I send a mail?" "help me with the mail?"
;;; specific help corresponding to special help tasks are caught by the task 
;;; task selection mechanism?

(defsubdialog 
  _print-help-conversation 
  (:label "Help conversation")
  (:explanation 
   "Help was asked.")
  (:states _ph-print-global-help)
  )

;;;--------------------------------------------------------- (PH) PRINT-GLOBAL-HELP

(defstate
  _ph-print-global-help
  (:label "Print general help")
  (:entry-state _print-help-conversation )
  (:explanation "No specific subject was included.")
  (:execute
   (let
     ((obj-id (car (send '>-global-help '=get 'is-title-of)))
      (*language* :fr)
      (task-list (access '(task)))
      (conversation (omas::conversation XXXXX::pa_XXXXX))      
      )
     (when obj-id (send obj-id '=get-documentation)
           (send conversation '=display-text *answer*))
     (send conversation '=display-text
           "~2%I can do the following:")
     (dolist (task task-list)
       (send task '=get-documentation :lead "   - ")   
       (send conversation '=display-text *answer*)
       )
     (send conversation '=display-text "~%")
     ))
  (:transitions
   (:always :success))
  )
  

;;;================================================================================
;;;
;;;                       SET FONT SIZE CONVERSATION
;;;
;;;================================================================================

;;; this conversation is intended to modify the size of the dialog font, usually to
;;; increase it or to return to the default value.

(eval-when (compile load eval)
  (proclaim '(special 
              _sfs-entry-state  ; required by defstate
              _sfs-dont-understand
              _sfs-try-again
              _sfs-sorry)))

;;;-------------------------------------------------- (SFS) FONT-SIZE-CONVERSATION

(defsubdialog 
    _set-font-size-conversation
    (:short-name sfs)
  (:label "Font size conversation")
  (:explanation 
   "The data is searched for the presence of words like increase, ~
    bigger, smaller, defaults, reset,...")
  ;(:action _sfs-action)
  )

;;;--------------------------------------------------- (SFS) FONT-SIZE-ENTRY-STATE

(defstate 
    _sfs-entry-state
    (:entry-state _set-font-size-conversation)
  (:label "Font size dialog entry")
  (:explanation "The data is searched for the presence of words like increase, ~
                 bigger, defaults, reset,...")
  (:transitions
   (:patterns ((* "increase" *)
               (* "bigger" *)
               (* "larger" *))
              :exec (omas::assistant-use-larger-font PA_XXXXX) :success)
   (:patterns ((* "decrease" *)
               (* "smaller" *))
              :exec (omas::assistant-reset-font PA_XXXXX) :success)
   (:patterns ((* "use" "default" *)
               (* "reset" *)
               (* "defaults" *))
              :exec (omas::assistant-reset-font PA_XXXXX) :success)
   (:otherwise :target _sfs-dont-understand)
   )
  )

;;; add entry state to dialog header
;(send 'moss::$ESTS '=delete-all _set-font-size-conversation)
;(moss::%link _set-font-size-conversation 'moss::$ESTS _sfs-entry-state)

;;;--------------------------------------------------------- (SFS) DONT-UNDERSTAND

(defstate 
    _sfs-dont-understand
    (:label "Did not understand. Ask master.")
  (:explanation "Assistant is asking master for font-size clarification.")
  (:question "~%- Do you want a bigger font size, a smaller one, or use the default?")
  (:transitions
   (:always :target _sfs-try-again)))

;;;------------------------------------------------------------------- (sfs) SORRY

(defstate 
    _sfs-sorry
    (:label "Font dialog failure.")
  (:explanation "We cannot make things out from the data.")
  (:reset)
  (:answer "~%- Sorry I do not understand what you want me to do with the font?.~%")
  (:transitions
   (:always :failure))
  )

;;;--------------------------------------------------------------- (sfs) TRY-AGAIN

;;; The simplest use is to take text in the HAS-DATA slot of the Q-CONTEXT
;;; and send a message to :SA-ADDRESS

(defstate 
    _sfs-try-again
    (:label "Font size try again")
  (:explanation "we asked master and try to pick up the answer.")
  (:transitions
   (:patterns ((* "bigger" *)
               (* "larger" *))
              :exec (omas::assistant-use-larger-font PA_XXXXX) :success)
   (:patterns ((* "smaller" *))
              :exec (omas::assistant-reset-font PA_XXXXX) :success)
   (:patterns ((* "default" *)
               (* "defaults" *))
              :exec (omas::assistant-reset-font PA_XXXXX) :success)
   (:otherwise :target _sfs-sorry)
   )
  )

;;;===============================================================================
;;;
;;;                       TRACE CONVERSATION
;;;
;;;===============================================================================

;;; this conversation is intended to modify the size of the dialog font, usually to
;;; increase it or to return to the default value.

(eval-when (compile load eval)
  (proclaim '(special 
              _tr-entry-state
              )))

;;;------------------------------------------------------- (TR) TRACE-CONVERSATION

(defsubdialog 
    _trace-conversation
    (:label "Trace conversation")
  (:explanation 
   "Command to toggle the dialog trace."))

;;;-------------------------------------------------------- (TR) TRACE-ENTRY-STATE

(defstate 
    _tr-entry-state
    (:label "Trace dialog entry")
  (:entry-state _trace-conversation)
  (:explanation "We search the input for the trace command.")
  
  (:transitions
   ;; system commands
   (:patterns ((* "show" "transitions" *)
               (* "trace" "transitions" *)
               (* "transitions" "on" *)
               (* "print" "transitions" *))
              :exec (setq moss::*transition-verbose* t
                        moss::*traced-agent* PA_XXXXX) :success)
   (:patterns ((* "no" "trace" *)
               (* "kill" "trace" *)
               (* "trace" "off" *)
               (* "stop" "trace" *)
               (* "no" "verbose" *))
              :exec (setq moss::*verbose* nil
                        moss::*transition-verbose* nil
                        moss::*traced-agent* nil) :success)
   (:patterns ((* "trace" "on" *)
               (* "set" "trace" *)
               (* "start" "trace" *)
               (* "verbose" *))
              :exec (setq moss::*verbose* t
                        moss::*traced-agent* PA_XXXXX) :success)
   (:patterns ((* "kill" "transitions" *)
               (* "hide" "transitions" *)
               (* "transitions" "off" *)
               (* "do" "not" "show" "transitions" *))
              :exec (setq moss::*transition-verbose* nil) :success)
   (:otherwise :failure)
   )
  )

#|
;;; add entry state to dialog header
(send 'moss::$ESTS '=delete-all _trace-conversation)
(moss::%link _trace-conversation 'moss::$ESTS _tr-entry-state)
|#

;;;================================================================================
;;;
;;;                          WHAT-IS CONVERSATION
;;;
;;;================================================================================

;;; this conversation is intended to obtain and print information about something
;;; corresponding to unformatted data

;;;(defParameter *empty-words* 
;;;  '("the" "a" "an" "of" "from" "to" "those" "these" "that" "s" "?"
;;;    "le" "la" "les" "de" "du" "des" #+MCL "à" #+MICROSOFT-32 "à" "aux" 
;;;    "ce" "ces"))

;;;================================================================================

(defsubdialog 
    _what-is-conversation
    (:label "Get info dialog")
  (:explanation "Master is trying to get info about some data.")
  (:states    _what-is-conversation  ; required by defstate
           _wai-entry-state
           _wai-ask-staff
           _wai-get-info
           _wai-process-staff-answer
           _wai-sorry)
  )

;;;---------------------------------------------------------- (WHAT-IS) ENTRY-STATE

(defstate 
    _wai-entry-state
    (:label "What is conversation")
  (:entry-state _what-is-conversation)
  (:explanation "Assistant is trying to get object info from its knowledge base.")
  (:transitions
   ;;keep only relevant data
   (:patterns ((* "what" "is" ?y "?")
               (?y "is" "what" "?" *)
               (?y "what" "is" "it" "?")
               (?y)  ; when entering from print-concept-documentation
               )
              :keep ?y
              :target _wai-get-info)
   (:otherwise :target _wai-sorry)
   )
  )

;;;------------------------------------------------------------ (WHAT-IS) ASK-STAFF
;;; changing strategy to :collect-answers
;;; rather than sending a broadcast right away, maybe we should
;;; 1. look into local KB
;;; 2. look into our agenda (sa-address)
;;; 3. ask BIBLIO
;;; This would avoid strange answers resulting from misinterpretation of the
;;; query.

(defstate 
    _wai-ask-staff
    (:label "Ask staff.")
  (:explanation "Ask the staff for help.")
  (:send-message :to :all :self PA_XXXXX :action :what-is :timeout 2
                 :args `(((:data . ,(read-fact moss::conversation :input))
                          (:language . :en)
                          ))
                 :strategy :collect-answers
                 )
  (:transitions 
   (:always :target _wai-process-staff-answer))
  )

;;;------------------------------------------------------------- (WHAT-IS) GET-INFO

(defstate 
    _wai-get-info
    (:label "Looking into local KB info.")
  (:explanation "Look into own KB to get info.")
  (:answer-analysis)
  )

(defownmethod =answer-analysis _wai-get-info (conversation input)
  "we try first to get info locally and if it fails ask all agents.
Arguments:
   agent: assistant."
  (declare (special *empty-words*))
  (let* (entry-list)
    (catch :return
           ;; clean input of parasitic words
           (setq input (reduce  ; JPB 140820 removing mapcan
                        #'append
                        (mapcar
                            #'(lambda (xx) 
                                (if (member xx *empty-words* :test #'string-equal)
                                    nil
                                  (list xx)))
                          input)))
           (moss::vformat "XXXXX: _wai-get-info /+++ cleaned input:~& ~S" input)
           ;; if nothing left, then go ask staff
           (unless input
             (throw :return `(:transition ,_wai-ask-staff)))
           ;; check whether the resulting words correspond to entry points
           (setq entry-list (moss::find-best-entries input))
           (moss::vformat "XXXXX: _wai-get-info /entry list:~&    ~S" entry-list)
           ;; if nothing left go ask staff
           (unless entry-list 
             (throw :return `(:transition ,_wai-ask-staff)))
           ;; if more than one result, ask for help (not yet)
           ;; get documentation or CV
           ;; print it
           (dolist (item entry-list)
             (send item '=get-documentation :lead "   - ")   
             (send conversation '=display-text *answer*)
             )
           ;; transfer to :success    
           (list :success) 
           )
    ))

;;;------------------------------------------------- (WHAT-IS) PROCESS-STAFF-ANSWER

(defstate 
    _wai-process-staff-answer
    (:label "Process staff answer.")
  (:explanation "Process anwers from the staff agents.")
  (:answer-analysis)
  )

(defownmethod =answer-analysis _wai-process-staff-answer (conversation input)
  "we got an answer from some staff or else the master got impatient.
Arguments:
   agent: assistant."
  (declare (ignore input))
  (let* ((agent (car (HAS-MOSS-AGENT conversation)))
         (answer-list (omas::answer agent))
         (*language* :en)
         )
    ;(format *debug-io* "~&+++ =resume; HAS-TO-DO: ~S" (has-to-do conversation))
    ;; clean up HAS-TO-DO (contains "answer-there")
    ;(setf (has-to-do conversation) nil)
    ;; answer contains either the word "failure" or a list of lists of strings
    ;; (("Jean-Paul Barthes is a professor at UTC ..."
    ;;  "Dominique Barthes-Biesel is a professor at UTC ...")
    ;;  ("Jean-Paul Barthes has NN publications on:  ...")
    ;;  ...)
    ;; 
    (moss::vformat "~&>>> WAIdialog /Answer to the send-request:~% ~S" answer-list)
    ;; failure if :failure is returned or :error on timeout
    (cond
     ((moss::is-answer-failure? answer-list)
      `(:transition ,_wai-sorry))
     ((null answer-list)
      (send conversation '=display-text "I don't know...")
      (list :success))
     (t 
      (moss::vformat " _wai-ask-staff /answer-list: ~& ~S" answer-list)
      (send conversation '=display-text "It is:")
      ;; each item of the answer list is like 
      ;;    ("personne" ("nom" "...")("prénom" "..."))
      (dolist (answer answer-list)
        ;; answer could be "failure" rather than a list of strings
        (unless (and (stringp answer) (string-equal answer "failure"))
          (dolist (item answer)
            (send conversation '=display-text 
                  (concatenate 'string "~%" (car item)))
            (send conversation '=display-text
                  (format nil ": ~{~%     ~{~A~^, ~}~}" (cdr item))))))
      (list :success)))
    ))

;;;---------------------------------------------------------------- (WHAT-IS) SORRY

(defstate 
    _wai-sorry
    (:label "Failure to get info.")
  (:explanation "We could not find anything.")
  (:reset)
  (:text "Sorry, I could not find information for the data.")
  (:transitions 
   (:failure))
  )


;;;================================================================================
;;;
;;;                    WHO IS (GET PERSON INFO) CONVERSATION
;;;
;;;================================================================================

;;; this conversation is intended to print information about a person.

;;;------------------------------------------------------------- CHECK-ENTRY-POINTS

(defun check-entry-points (input)
  "tries to get entry points by combining words.
Arguments:
   input: list of one or more words
Return:
   nil or a list like 
     (\"barthes\" ($T-PERSON-NAME.OF . $E-PERSON.1) ($T-PERSON-NAME.OF . $E-PERSON.2))"
  (let (result entry entry-list expr-list)
    ;; combine all possible words
    (setq expr-list (make-word-combinations input))
    ;; try all possibilities
    (dolist (expr expr-list)
      ;; entry point ?
      (when (setq entry (moss::%is-entry? expr))
        ;; extract corresponding objects
        (setq result (moss::%%get-objects-from-entry-point entry))
        ;; return to the executing environment
        (moss::vformat "...trying ~S in package ~S as entry...~&   => ~S" 
                       expr *package* result)
        (if result (setq entry-list (append entry-list result)))
        ))
    ;; clean list, eliminating tasks whose performative is different of the required
    ;; one
    (setq entry-list (delete-duplicates entry-list :test #'equal))
    (moss::vformat "check-entry-points /Resulting entry list ~&  => ~S" entry-list)
    (mapcar #'cdr entry-list)))

#|
? (XXXXX::check-entry-points '("telephone" "barthes" ))
;***** ...trying "telephone" in package #<Package "SA-ADDRESS"> as entry...
   => ((MOSS::$ENAM.OF . $E-PHONE))
;***** ...trying "barthes" in package #<Package "SA-ADDRESS"> as entry...
   => (($T-PERSON-NAME.OF . $E-PERSON.2) ($T-PERSON-NAME.OF . $E-PERSON.3)
       ($T-PERSON-NAME.OF . $E-PERSON.4))
;***** check-entry-points /Resulting entry list 
  => ((MOSS::$ENAM.OF . $E-PHONE) ($T-PERSON-NAME.OF . $E-PERSON.2)
      ($T-PERSON-NAME.OF . $E-PERSON.3) ($T-PERSON-NAME.OF . $E-PERSON.4))
($E-PHONE $E-PERSON.2 $E-PERSON.3 $E-PERSON.4)
|#
;;;--------------------------------------------------------- MAKE-WORD-COMBINATIONS

(defun make-word-combinations (word-list)
  ;; when empty list or only one word return
  (unless (cdr word-list) (return-from make-word-combinations word-list))
  ;; otherwise build word combinations
  (let ((input-length (length word-list)) ; e.g. 5
        result)
    (dotimes (nn (1+ input-length)) ; e.g. nn: 0 -> 4
      ;; build cursor to move through the list
      (dotimes (jj nn) ; e.g. jj: 0, then jj: 0->1
        (push
         (apply #'moss::%make-phrase 
                (subseq word-list jj (+ (- input-length nn) jj 1)))
         result)))
    (reverse result)))

#|
? (make-word-combinations '("a" "b" "c" "d"))
("a b c d" "a b c" "b c d" "a b" "b c" "c d" "a" "b" "c" "d")
? (moss::%make-name-for-entry "a b c")
A-B-C
|#
;;;================================================================================

(eval-when (compile load eval)
  (proclaim '(special 
              _get-person-info-conversation  ; required by defstate
              _gpi-entry-state
              _gpi-ask-staff
              _gpi-get-info
              _gpi-sorry)))

(defsubdialog 
    _get-person-info-conversation
    (:label "Get person info dialog")
  (:explanation "Master is trying to obtain a telephone number.")
  )

(defstate 
    _gpi-entry-state
    (:label "Get person info conversation")
  (:entry-state _get-person-info-conversation)
  (:explanation "Assistant is trying to get the info from its knowledge base.")
  (:transitions
   ;;keep only relevant data
   (:patterns ((* "who" "is" ?y "?")
               (?y "is" "who" *)
               )
              :keep ?y
              :target _gpi-get-info)
   (:otherwise :target _gpi-sorry)
   )
  )

;;;------------------------------------------------------------- (WHO-IS) ASK-STAFF
;;; changing strategy to :collect-answers
;;; rather than sending a broadcast right away, maybe we should
;;; 1. look into local KB
;;; 2. look into our agenda (ADDRESS)
;;; 3. ask BIBLIO
;;; This would avoid strange answers resulting from misinterpretation of the
;;; query.

(defstate 
    _gpi-ask-staff
    (:label "Ask other agents for help.")
  (:explanation "Ask the staff for help.")
  (:send-message :to :all :self PA_XXXXX :action :who-is :timeout 4
                 :args `(((:data . ,(read-fact moss::conversation :input))
                          (:language . :fr)
                          ))
                 :strategy :collect-answers
                 )
  (:transitions
   (:on-failure :target _gpi-sorry)
   (:otherwise
    :exec (dolist (answer (read-fact moss::conversation :answer))
            ;; answer could be "failure" rather than a list of strings
            (unless (or (and (stringp answer) (string-equal answer "failure"))
                        (null answer))
              (dolist (item answer)
                (send moss::conversation '=display-text item))))
    :success))
  )

;;;-------------------------------------------------------------- (WHO-IS) GET-INFO

(defstate 
    _gpi-get-info
    (:label "Looking into local KB info.")
  (:explanation "Look into own KB to get answer to who-is?.")
  (:answer-analysis)
  )

(defownmethod =answer-analysis _gpi-get-info (conversation input)
  "we got an answer from some staff or else the master got impatient.
Arguments:
   conversation: the conversation object
   input: a copy of the FACTS/input area"
  (let* (entry-list)
    (catch :return
           ;; clean input of parasitic words
           (setq input (reduce  ; JPB 140820 removing mapcan
                        #'append
                        (mapcar
                            #'(lambda (xx) 
                                (if (member xx *empty-words* :test #'string-equal)
                                    nil
                                  (list xx)))
                          input)))
           (moss::vformat "XXXXX: _gpi-get-info /+++ cleaned input:~& ~S" input)
           ;; if nothing left, then ask staff
           (unless input
             (throw :return `(:transition ,_gpi-ask-staff)))
           ; check whether the resulting words correspond to entry points
           (setq entry-list (check-entry-points input))
           (moss::vformat "XXXXX: _gpi-get-info /entry list:~& ~S" entry-list)
           ;; if nothing left go ask staff
           (unless entry-list 
             (throw :return `(:transition ,_gpi-ask-staff)))
           ;; if more than one person ask for help (not yet)
           ;; get documentation or CV
           ;; print it
           (dolist (item entry-list)
             (send item '=get-documentation)
             (send conversation '=display-text *answer*))
           ;; transfer to :success    
           (list :success) 
           )
    ))

;;;----------------------------------------------------------------- (WHO-IS) SORRY

(defstate 
    _gpi-sorry
    (:label "Sorry: no info.")
  (:explanation "We could not find anything.")
  (:reset)
  (:text ("- Sorry, I have not the slightest idea.~%"
          "- I don't know.~%"))
  (:transitions 
   (:failure))
  )

;;;===============================================================================
;;;===============================================================================
;;;
;;;                         MAIN (TOPLEVEL) CONVERSATION
;;;
;;;===============================================================================
;;;===============================================================================

;;; the following conversations are defined thereafter
;;; the entry point of the dialog is _MAIN-CONVERSATION

(eval-when (load compile eval)
  (proclaim '(special 
              _MAIN-CONVERSATION
              _EXPLAIN-CONVERSATION
              _PRINT-ALL-DOCUMENTATION-CONVERSATION
              _PRINT-HELP-CONVERSATION
              _PRINT-OBJECT-DOCUMENTATION-CONVERSATION
              _PROCESS-CONVERSATION
              _SET-FONT-SIZE-CONVERSATION
              _TRACE-CONVERSATION
              ;_PRINT-CONVERSATION
              ;_SHOW-EXAMPLE-CONVERSATION
              ;_WHAT-CONVERSATION 
              )))

;;;================================================================================
;;;
;;;                       MAIN CONVERSATION (CONTROL LOOP)
;;;
;;;================================================================================

;;; create a dialog header to be used for the main conversation

(defindividual 
    MOSS-DIALOG-HEADER
    (HAS-MOSS-LABEL "Main conversation loop")
  (HAS-MOSS-EXPLANATION "This is the main conversation loop.")
  (:var _main-conversation))


;; declare that conversation, replacing default conversation
(setf (omas::dialog-header PA_XXXXX) _main-conversation)

;;; debugging function
(defun cc () (moss::start-conversation _main-conversation))

;;;===== states are declared as global variables
;;; the set of states can be considered as a plan to be executed for conducting the
;;; conversation with the master

;;;--------------------------------------------------------------------------------
(eval-when (compile load eval)
  (proclaim '(special 
              _mc-entry-state
              _mc-get-input
              _mc-more?
              _mc-process
              _mc-sleep)))
;;;--------------------------------------------------------------------------------

;;;--------------------------------------------------------------- (MC) ENTRY-STATE

;(defun ff () (start-conversation _main-conversation :first-time t))
;(defun gg () (start-conversation _main-conversation))

;;; define an entry instance and record it in a global variable.
;;; ****** we should enter a fram on the conversation frame list, to handle
;;; success or failure returns from sub-dialogs. I.e. we should make a transition
;;; on the success or failure state of the main conversation.
;;; entry (_mc-failure _mc-success _mc-entry-state nil) ; no goal 

(defstate _mc-entry-state
    (:entry-state _main-conversation)
  (:label "Dialog start")
  (:explanation 
   "Initial state when the assistant starts a conversation. Send a welcome message ~
    and wait for data. Also entered on a restart following an abort.")
  (:reset-conversation)
  (:text "Warning. This is not a real dialog in the sense that every question ~
            usually does not use the previous context. Thus, referents like ~
            pronouns or ellipsis (referring to part of the answer) won't work.")
  (:question-no-erase
   ("Hello! What can I do for you?"
    "Hi! What would you like to know?"
    "Hi! I will try to do my best to answer your question or ~
     execute your orders, but remember that my IQ is limited."))
  (:transitions 
   (:always :target _mc-process))
  )

;;;----------------------------------------------------------------- (MC) GET-INPUT

(defstate _mc-get-input
    (:label "Get input")
  (:explanation
   "State in which the user inputs his request.")
  (:reset)
  (:question "I'm listening....")
  (:transitions
   (:always :target _mc-process)))

;;;--------------------------------------------------------------------- (MC) MORE?

(defstate _mc-more?
    (:label "More?")
  (:explanation "Asking the user it he wants to do more interaction.")
  (:reset-conversation)
  (:question-no-erase
   ("What else can I do for you?"
    "Is there anything more I can do for you?"
    "Would you like to do something else?"
    "OK.")
   ) 
  (:transitions 
   (:starts-with ("nothing") :target _mc-sleep)
   (:no :target _mc-sleep)
   (:yes :target _mc-get-input)
   (:otherwise :target _mc-process)))

;;;------------------------------------------------------------------- (MC) PROCESS

(defstate _mc-process
    (:label "Process state of main conversation")
  (:explanation
   "Here we call the main process organized as a sub-vconversation.")
  (:transitions
   (:always :sub-dialog _process-conversation
            :success _mc-more?
            :failure _mc-more?)))

;;;--------------------------------------------------------------------- (MC) SLEEP

(defstate _mc-SLEEP
    (:label "Nothing to do")
  (:explanation
   "User said she wanted nothing more.")
  (:reset)
  (:text "OK. I wait until you are ready.")
  (:question-no-erase
   "Wake me up when you want by typing something...")
  (:transitions
   (:always :target _mc-process)))


;;;================================================================================
;;;
;;;                      MAIN CONVERSATION (EXECUTION PART)
;;;
;;;================================================================================

;;; this conversation is intended to process the input from te user by first
;;; determining the performative, then the list of possible tasks

(defsubdialog 
    _process-conversation 
    (:label "Process conversation")
  (:explanation 
   "Processing steps of the main conversation.")
  (:states _mc-eliza
           _mc-find-performative
           _mc-entry-state
           _mc-failure
           _mc-find-performative
           _mc-find-task
           _mc-select-task
           _mc-task-dialog)
  )

;;;================================================================================
;;;                            Service functions
;;;================================================================================

;;;--------------------------------------------------------------------- PRINT-TIME

(defun print-time (conversation &optional (language *language*))
  (multiple-value-bind (second minute hour date month year)
      (decode-universal-time (get-universal-time))
    (send conversation '=display-text
          (case language
            (:en
             (format nil "Current time is ~S:~S:~S and today is the ~S/~S/~S~%"
               hour minute second date month year))
            (:fr
             (format nil 
                 "Il est actuellement ~S:~S:~S et nous sommes aujourd'hui le ~
                      ~S/~S/~S~%"
               hour minute second date month year)))
          )))

;;;================================================================================
;;;                            Conversation states
;;;================================================================================


;;;--------------------------------------------------------- (MC) FIND-PERFORMATIVE

(defstate _mc-find-performative
    (:entry-state _process-conversation)
  (:process-state _main-conversation)
  (:label "Find performative")
  (:explanation 
   "Process what the user said trying to determine the type of performative ~
    among :request :assert :command. Put the result if any into the performative ~
    slot of the conversation object.")
  (:transitions 
   ;; time?
   (:patterns (("what" "time" (?* ?x)))
              :exec (print-time moss::conversation :en) :success)
   ;; now really select performatives
   (:patterns (("what" "is" (?* ?x))
               ("what" "are" (?* ?x))
               ("what" (?* ?x))
               ("when" (?* ?x))
               ("where" (?* ?x))
               ("why" (?* ?x))
               ("who" (?* ?x))
               ("whom" (?* ?x))
               ("whose" (?* ?x))
               ("how" "much" (?* ?x))
               ("how" (?* ?x))
               ("is" (?* ?x))
               ("are" (?* ?x))
               ("were" (?* ?x))
               ("was" (?* ?x))
               ("have" (?* ?x))
               ("had" (?* ?x))
               ("do" (?* ?x))
               ("did" (?* ?x))
               ("can" (?* ?x))
               ("could" (?* ?x))
               ("may" (?* ?x))
               ("might" (?* ?x))
               ((?* ?x) "?")
               )
              :set-performative (list :request)
              :target _mc-find-task)
   (:patterns (((?* ?x) "note" (?* ?y))
               ((?* ?x) "remember" (?* ?y)))
              :set-performative (list :assert)
              :target _mc-find-task)
   (:otherwise 
    :set-performative (list :command)
    :target _mc-find-task))
  )

;;;--------------------------------------------------------------------- (MC) ELIZA

(moss::defstate _mc-ELIZA
    (:label "ELIZA")
  (:explanation
   "Whenever MOSS cannot interpret what the user is saying, ELIZA is called to ~
    do some meaningless conversation to keep the user happy.
    We then record what the user says and resume the analysis.")
  (:eliza)
  ;(:answer "~%What you said is certainly interesting, but I did not understand it.")
  (:transitions
   (:always :target _mc-find-performative))
  )

;;;------------------------------------------------------------------- (MC) FAILURE

(defstate _mc-failure
    (:label "Failure state of main conversation")
  (:explanation
   "Failure state is entered when we return from a sub-dialog with a :failure tag. ~
    We check for more tasks to perform (listed in the task-list slot of the ~
    conversation object). If there are more, we execute the first one. If there ~
    are no more, we go to the more? state.")
  (:answer-analysis)
  )

(defownmethod
    =answer-analysis _mc-failure (conversation input)
  (declare (ignore input))
  (let* ((task-list (HAS-MOSS-TASK-LIST conversation))
         task)
    (if task-list
        (progn
          ;; record next task
          (setq task (pop task-list))
          ;; remove it from task-list
          (send conversation '=replace 'HAS-MOSS-TASK-LIST task-list)
          ;; add it to conversation
          (send conversation '=replace 'HAS-MOSS-TASK (list task))
          ;; transfer to sub-dialog
          `(:transition ,_mc-task-dialog))
      ;; when no more tasks we return :failure and the state variable still contains
      ;; :failure. The crawler will return one more level until hitting a non
      ;; failure return containing a specific state (e.g. the one for the main
      ;; conversation)
      (list :failure)
      )))

;;;----------------------------------------------------------------- (MC) FIND-TASK

(defstate _mc-find-task
    (:label "Find task")
  (:explanation 
   "Process the input to determine the task to be undertaken. Combines the words ~
    from the sentence to see if they are entry points for the index property of ~
    any task. Collect all tasks for which there is an index in the sentence.")
  (:answer-analysis)
  )

(defownmethod
    =answer-analysis _mc-find-task (conversation input)
  "Using input to find tasks If none failure, if one, OK, if more, must ask user to ~
   select one. Checks for an entry point for an index of a task.
Arguments:
   conversation: current conversation
   input: list containing the input as a list of words"
  (let* ((performative (read-fact conversation :PERFORMATIVE))
         task-list)
    (moss::vformat "_mc-find-task /input: ~S, package: ~S" input *package*)
    ;; try to find a task from the input text
    (setq task-list (moss::find-objects 
                     '(task (has-index-pattern (task-index (has-index :is :?))))
                     input :all-objects t)) 
    ;; filter tasks that do not have the right performative
    (setq task-list
          (reduce  ; JPB 140820 removing mapcan
           #'append
           (mapcar
               #'(lambda (xx) 
                   (if (intersection performative (HAS-PERFORMATIVE xx))
                       (list xx)))
             task-list)))
    (moss::vformat "_mc-find-task /task list after performative check: ~S" task-list)
    (cond 
     ;; if empty, ask ELIZA
     ((null task-list)
      `(:transition ,_mc-ELIZA))
     ;; if one, then OK
     ((null (cdr task-list))
      ;; save results, make the task the conversation task
      (setf (HAS-MOSS-TASK conversation) task-list)
      `(:transition ,_mc-task-dialog))
     (t 
      ;; otherwise must select one from the results
      (setf (HAS-MOSS-TASK-LIST conversation) task-list)
      `(:transition ,_mc-select-task)))
    ))

;;;--------------------------------------------------------------- (MC) SELECT-TASK

(defstate 
    _mc-select-task
    (:label "mc select task")
  (:explanation "we have located more than one task. We rank the tasks by computing ~
                 the average weight of the terms in the task index slot. We then ~
                 record the list of tasks into the statte-context task-list slot ~
                 and activate the highest ranking task.")
  (:answer-analysis) 
  )

(defownmethod =answer-analysis _mc-select-task (conversation input)
  "we have located more than one task. We rank the tasks by computing ~
   the MYCIN combination weight of the terms in the task index slot. We then ~
   record the list of tasks into the conversation task-list slot ~
   and activate the highest ranking task."
  ;(declare (ignore input))
  ;; the list of tasks is in the HAS-MOSS-TASK-LIST slot. Tasks are defined in the 
  ;; application package
  (let* ((task-list (HAS-MOSS-TASK-LIST conversation))
         patterns weights result pair-list selected-task level word-weight-list)
    (moss::vformat "_select-task /input: ~S~&  task-list: ~S" input task-list)
    ;; first compute a list of patterns (combinations of words) from the input
    (setq patterns (mapcar #'car (moss::generate-access-patterns input)))
    (moss::vformat "_select-task /input: ~S~&  generated patterns:~&~S" 
                   input patterns)
    ;; then, for each task
    (dolist (task task-list)
      (setq level 0)
      ;; get the weight list
      ;(setq weights (HAS-INDEX-WEIGHTS task))
      (setq weights (moss::%get-INDEX-WEIGHTS task))
      (moss::vformat "_select-task /task: ~S~&  weights: ~S" task weights)
      ;; check the patterns according to the weight list
      (setq word-weight-list (moss::%get-relevant-weights weights patterns))  
      (moss::vformat "_select-task /word-weight-list:~&  ~S" word-weight-list)
      ;; combine the weights
      (dolist (item word-weight-list)
        (setq level (+ level (cadr item) (- (* level (cadr item))))))
      (moss::vformat "_select-task /level: ~S" level)
      ;; push the task and weight onto the result list
      (push (list task level) result)
      )
    (moss::vformat "_select-task /result:~&~S" result)
    ;; order the list
    (setq pair-list (sort result #'> :key #'cadr)) 
    (moss::vformat "_select-task /pair-list:~&  ~S" pair-list)
    ;; keep the first task whatever its score
    (setq selected-task (caar pair-list))
    ;; remove the task that have a weight less than task-threshold (default 0.4)
    (setq pair-list
          (remove nil 
                  (mapcar 
                      #'(lambda (xx) 
                          (if (>= (cadr xx) (omas::task-threshold omas::*omas*)) xx))
                    pair-list)))
    ;; if task-list is empty then return th? first saved task
    ;; this may not be a good policy if the score is too low
    (if (null pair-list)
        (progn
          ;; reset the task-list slot of the conversation object
          (setf (HAS-MOSS-TASK-LIST conversation) nil)
          ;; put the saved task into the task slot
          (setf (HAS-MOSS-TASK conversation) (list selected-task))
          ;; go to task-dialog
          `(:transition ,_mc-task-dialog)
          )
      (progn
        ;; remove the weights
        (setq task-list (mapcar #'car pair-list))
        ;; select the first task of the list
        (setq selected-task (pop task-list))
        (moss::vformat "_select-task /selected task: ~S" selected-task)
        ;; save the popped list in the task-list slot of the conversation object
        (setf (HAS-MOSS-TASK-LIST conversation) task-list)
        (setf (HAS-MOSS-TASK conversation) (list selected-task))
        ;; go to task-dialog
        `(:transition ,_mc-task-dialog)))
    ))  

;;;--------------------------------------------------------------- (MC) TASK-DIALOG

(defstate _mc-task-dialog
    (:label "Task dialog")
  (:explanation 
   "We found a task to execute (in the GOAL slot of the conversation). We activate ~
    the dialog associated with this task.")
  (:answer-analysis)
  )

;;;********** must check for possible problems
(defownmethod
    =answer-analysis _mc-task-dialog (conversation input)
  "We prepare the set up to launch the task dialog.
Arguments:
   conversation: current conversation"
  (declare (ignore input))
  (let* ((task-id (car (send conversation '=get 'HAS-MOSS-TASK))))
    (moss::vformat "_mc-task-dialog /task-id: ~S dialog: ~S" 
                   task-id (HAS-DIALOG task-id))
    ;; we launch the task dialog as a sub-conversation
    ;; the task contains the name of a sub-conversation header, e.g. _get-tel-nb
    `(:sub-dialog ,(car (HAS-DIALOG task-id)) :failure ,_mc-failure)
    ))


;;;================================================================================

;;; here the strategy is to develop a specific dialog for each task. However, we 
;;; could also use a generic dialog consisiting in:
;;;    - getting the list of required parameters
;;;    - for each parameter, trying to extract info from data using predefined
;;;      rules or extraction patterns (Note that this is not much different than
;;;      having a predefined dialog, albeit focused on a single parameter)
;;;    - if some required parameters are missing, ask master about them using 
;;;      predefined questions
;;;    - shipping result to ad hoc staff agent

;;; The generic dialog could be an option offered to the programmer, which requires
;;; a suitable modeling of the tasks.


;;;================================================================================

;;;================================================================================
;;;
;;;                             ELIZA CONVERSATION
;;;
;;;================================================================================

;;; this conversation is intended to do small talk.


(defParameter *eliza-rules*
  '((("hello")
     "Hi! I am XXXXX."
     "Hello there.")
    (("Hi")
     "Hi! I am XXXXX."
     "Hi.")
    (("hello" "XXXXX" *)
     "Hello Master."
     "Hello Master, pleased to see you up and active.")
    (("Hi" "XXXXX" *)
     "Hi Boss."
     "HI Boss. Nice day isn't it?")
    (("bonjour")
     "Je suis désolé mon français n'est pas encore utilisable. Please use English.")
    (("how" "are" "you" (?* ?x))
     "Very well thank you."
     "Really does my health concern you? I am flattered.")
    (("who"  (?* ?x) "Barthès")
     "Jean-Paul Barthès is a professor of artificial intelligence at the ~
      Université de Technologie de Compiègne in France.")
    (("who"  (?* ?x) "Barthes")
     "Jean-Paul Barthès is a professor of artificial intelligence at the ~
      Université de Technologie de Compiègne in France.
      Note however that Barthès is written with an è.")
    (("forget" "it")
     "OK. No hard feelings.")
    (("thank" "you" (?* ?y))
     "you are welcome.")
    (("thanks")
     "you are welcome."
     "don't mention it.")
    (((?* ?x) "computer" (?* ?y))
     "Do computer worry you?"
     "What do you think about machines?"
     "Why do you mention computers?"
     "What do you think machines have to do with your problem?")
    (((?* ?x) "stupid" (?* ?y))
     "Please, behave!"
     "What you say makes me really sad...")
    (((?* ?x) "does" "not" "work" (?* ?y))
     "Of course ?x works!"
     "What are  you saying that ?x does not work?")
    (("help")
     "you want help about what?")
    (((?* ?x) "sorry" (?* ?y))
     "Please don't apologize"
     "Apologies are not necessary"
     "What feelings do you have when you apologize?")
    (((?* ?x) "I" "remember" (?* ?y))
     "Do you often think of ?y"
     "Does thinking of ?y bring anything else to mind?"
     "What else do you remember?"
     "Why do you recall ?y right now?"
     "What in the present situation reminds you of ?y ?"
     "What is the connection between me and ?y ?")
    (((?* ?x) "do" "you" "remember" (?* ?y))
     "Did you think I would forget ?y ?"
     "Why do you think I should recall ?y by now ?"
     "What about ?y ?"
     "You mentined ?y ?")
    (((?* ?x) "if" (?* ?y))
     "Do you really think its likely that ?y ?"
     "Do you wish that ?y ?"
     "What do you think about ?y ?"
     "Really -- if ?y ?")
    (("what" (?* ?x) "do" "for" "me")
     "So many things..."
     "Currently I do not know."
     "What could please you?")
    
    (((?* ?x) "I" "dreamt" (?* ?y))
     "Really -- ?y"
     "Have you ever fantasized ?y while you were awake ?"
     "Have you dreamt ?y before ?")
    (((?* ?x) "dream" "about" (?* ?y))
     "How do you feel about ?y in reality ?")
    (((?* ?x) "dream" (?* ?y))
     "What does this dream suggest to you ?"
     "Do you dream often ?"
     "What persons appear in your dreams ?"
     "Don't you believe that dream has to do with your problem ?")
    (((?* ?x) "my" "mother" (?* ?y))
     "Who else in your family ?y ?"
     "Tell me more about your family")
    (((?* ?x) "my" "father" (?* ?y))
     "Your father ?"
     "Does he influence you strongly ?"
     "What else comes to mind when you think of your father ?")
    (((?* ?x) "I" "want" (?* ?y))
     "What would it mean if you got ?y ?"
     "Why do you want ?y ?"
     "Suppose you got ?y soon ?")
    (((?* ?x) "I" "am" "glad" (?* ?y))
     "How have I helped you to be ?y ?"
     "What makes you happy just now ?"
     "Can you explain why you are suddenly ?y ?")
    (((?* ?x) "I" "am" "sad" (?* ?y))
     "I am sorry to hear you are depressed"
     "I am sure it's not pleasant to be sad")
    (((?* ?x) "are" "like" (?* ?y))
     "What resemblance do you see between ?x and ?y ?")
    (((?* ?x) "is" "like" (?* ?y))
     "In what way is it that ?x is like ?y ?"
     "What resemblance do you see ?"
     "Could there really be some connection ?"
     (How ?))
    (((?* ?x) "alike" (?* ?y))
     "In what way ?"
     "What similarities are there ?")
    (((?* ?x) "same" (?* ?y))
     "What other connections do you see ?")
    
    (((?* ?x) "I" "was" (?* ?y))
     "Were you really ?"
     "Perhaps I already knew you were ?y"
     "Why do you tell me you were ?y by now ?")
    (((?* ?x) "was" "I" (?* ?y))
     "What if you were ?y ?"
     "Do you think you were ?y ?"
     "What would it meant if you were ?y ?")
    (((?* ?x) "I" "am" (?* ?y))
     "In what way are you ?y ?"
     "Do you want to be ?y ?")
    (((?* ?x) "am" "I" (?* ?y))
     "Do you believe you are ?y ?"
     "Would you want to be ?y ?"
     "You wish I would tell you you are ?y"
     "What would it mean if you were ?y ?")
    (((?* ?x) "am" (?* ?y))
     "Why do you say \"AM?\""
     "I don't understand that")
    (((?* ?x) "are" "you" (?* ?y))
     "Why are you interested in whether I am ?y or not ?"
     "Would you prefer if I weren't ?y ?"
     "Perhaps I am ?y in your fantasies")
    (((?* ?x) "you" "are" (?* ?y))
     "What makes you think I am ?y ?")
    (((?* ?x) "because" (?* ?y))
     "Is that the real reason ?"
     "What other reasons might there be ?"
     "Does that reason seem to explain anything else ?")
    (((?* ?x) "were" "you" (?* ?y))
     "Perhaps I was ?y"
     "What do you think ?"
     "What if I had been ?y ?")
    (((?* ?x) "I" "can't" (?* ?y))
     "Maybe you could ?y now"
     "What if you could ?y ?")
    (((?* ?x) "I" "feel" (?* ?y))
     "Do you often feel ?y ?")
    (((?* ?x) "I" "felt" (?* ?y))
     "What other feelings do you have ?")
    (((?* ?x) "I" (?* ?y) "you" (?* ?z))
     "Perhaps in your fantasy we ?y each other")
    (((?* ?x) "why" "don't" "you" (?* ?y))
     "Should you ?y yourself ?"
     "Do you believe I don't ?y ?"
     "Perhaps I will ?y in good time")
    (((?* ?x) "yes" (?* ?y))
     "You seem quite positive."
     "You are sure ?"
     "I understand.")
    (((?* ?x) "no" (?* ?y))
     "Why not ?"
     "You are being a bit negative"
     "Are you saying \"NO\" just to be negative?")
    
    (((?* ?x) "someone" (?* ?y))
     "Can you be more specific ?")
    (((?* ?x) "everyone" (?* ?y))
     "Surely not everyone."
     "Con you think of anyone in particular ?"
     "Who for example ?"
     "You are thinking of a special person ?")
    (((?* ?x) "always" (?* ?y))
     "Can you think of a specific example ?"
     "When ?"
     "What incident are you thinking of ?"
     "Really -- always ?")
    (((?* ?x) "perhaps" (?* ?y))
     "You do not seem quite certain")
    (((?* ?x) "are" (?* ?y))
     "Did you think there might not be ?y ?"
     "Possibly they are ?y")
    (((?* ?x))
     "I do not understand exactly what you want. You may type: help."
     "I am not sure I understand what you want. You may type: help."
     "Could you rephrase your question please?"
     "Sorry I did not get that. Could you rephrase it? Or type: help."
     "Oops I do not understand. Could you say that differently?"
     "Sorry?"
     )))


#|

;;;========================= preparing mailing lists =============================

;;; Ask MORE INFO

(m-defstate _am-get-info ADDR
            (:label "More info")
            (:explanation "Assistant inferred that master wanted to obtain some sort ~
                 of info and has to find out what kind.")
            (:exec-var data)
            (:immediate-transitions
             (:starts-with (tel fax address email mail) :target _am-ask-address)
             (:starts-with (person paper) :target _am-ask-biblio)
             (:starts-with (mailing) :trim-data (data) :target _am-mailing-kind?)
             (:test (null data) :ask "What do you want exactly?")
             ;; otherwise should branch into the ask-around? state
             (:otherwise :target _am-ask-around?))
            (:transitions
             (:starts-with (tel fax address email mail) :target _am-ask-address)
             (:starts-with (person paper) :target _am-ask-biblio)
             (:starts-with (mailing) :trim-data (input) :target _am-mailing-kind?)))

;;; DETERMINE KIND OF MAILING

;;; missing =add-tp context 'HA?-MAILING-TYPE but in order to do that we should
;;; create and use a MAILING-CONTEXT object
;;; this could be done by creating a context type corresponding to the type of
;;; conversation taking place

(m-defstate _am-mailing-kind? ADDR
            (:label "Mailing kind?")
            (:explanation "We determine that master wants to do a mailing, we need to ~
                 know whether it is through email or postal mail.")
            (:exec-var data)
            (:immediate-transitions
             (:contains (email :email) :trim-data (data) 
                        :trim-data (data) :set-context ('HAS-MAILING-TYPE '(email)) 
                        :target _am-query-info)
             (:contains (postal mail :pmail) 
                        :trim-data (data) :set-context ('HAS-MAILING-TYPE '(pmail)) 
                        :target _am-query-info)
             (:otherwise
              :ask "what kind of mailing do you want: email or postal-mail?"))
            (:transitions
             (:contains (email :email) :set-context ('HAS-MAILING-TYPE '(email))
                        :trim-data (input) :target _am-query-info)
             (:contains (postal mail :pmail) :set-context ('HAS-MAILING-TYPE '(email))
                        :trim-data (input) :target _am-query-info)
             (:otherwise (:set-context ('HAS-MAILING-TYPE '(email)) :target _am-query-info))))

;;; QUERY INFO

(m-defstate _am-query-info ADDR
            (:label "Query info")
            (:explanation "We determined that we want to select mailing on a basis of a query ~
                 So we test if we have enough information to send to the address ~
                 agent for processing the query. Currently, we only check if there ~
                 is anything left in the data area. If so, this will be sued for ~
                 the query. ~
                 A query restricts the range of addresses to the result of a query ~
                 to the A-agent. Please formulate the query (example: \"Barthes\" ~
                 for getting all the persons named Barthès, or \"AFIA\" for getting ~
                 only the members of AFIA).")
            (:exec-var data)
            (:immediate-transitions
             (:contains (:help h ? how? how) :target _am-query-help)
             (:test data :target _am-send-query))
            (:ask "You should now formulate a question to restrict the range of addresses. 
Please formulate the query (example: all persons named \"Barthes\", or 
persons working-for \"AFIA\" for getting only the members of AFIA).")
            (:transitions
             (:contains (:help h ? how? how) :target _am-query-help)
             (:otherwise :save input :target _am-send-query)))

;;; QUERY HELP

(m-defstate _am-query-help ADDR
            (:label "Query help")
            (:explanation "help function to be able to formulate request. Indicates the ~
                 understood vocabulary.")
            (:ask "~&Possible vocabulary:
    all, person, company, firm, organization, name, named, location, living,
    living-at, working, working-for, at, employer, employed-by.
Possible expressions:
    person named \"Barthes\"
    person working-at \"UTC\"
    organization named \"IRIT\"
    person living \"Toulouse\"
    company at \"Compiegne\"
")
            (:transitions
             (:otherwise :save input :target _am-send-query)))

;;; SEND QUERY

;;; the comm process must deliver answer as a list for ust to be able to use the
;;; m-defstate macro

(m-defstate _am-send-query ADDR
            (:label "Send query")
            (:explanation "We collected info for producing a request, now we send it to the ~
                 SA-ADDRESS agent.")
            (:exec-var data)
            (:send-message)
            (:transitions
             (:test (and (typep (car input) 'message) (null (contents (car input))))
                    :target _am-no-result)
             (:test (typep (car input) 'message) :save input 
                    :target _am-process-results)))

;;; NO RESULT

(m-defstate _am-no-result ADDR
            (:label "No result")
            (:explanation "We asked SA-ADDRESS for a list of objects and got none. We should ~
                 rephrase the query.")
            (:ask "~& No answer to your query. Please rephrase it.
Possible vocabulary:
    all, person, company, firm, organization, name, named, location, living,
    living-at, working, working-for, at, employer, employed-by.
Possible expressions:
    person named \"Barthes\"
    person working-at \"UTC\"
    organization named \"IRIT\"
    person living \"Toulouse\"
    company at \"Compiegne\"
")
            (:transitions
             (:otherwise :save input :target _am-send-query)))

;;; PROCESS RESULTS

(m-defstate _am-process-results ADDR
            (:label "Process results")
            (:explanation "We got an answer from the SA-ADDRESS agent from which we must chosse.")
            (:exec-var data)
            (:ask-TSW :title "Select results"
                      :choice-list (cddr (contents (car data)))
                      :why "Select the objects that you want to keep on the mailing list.") 
            ;where do we put the result? in to-do, faking a question
            (:transitions
             (:otherwise :set-context ('HAS-RESULTS (list input)) 
                         :target _am-get-mailing-index)))

;;; GET MAILING INDEX

(m-defstate _am-get-mailing-index ADDR
            (:label "Getting index")
            (:explanation "asking master to give a name for mailing list.")
            (:question "Please give me a name for this mailing list?")
            (:transitions
             (:otherwise :save input :target _am-save-mailing-list)))

|#


(mformat "~%;*** MOSS v~A - XXXXX dialogs loaded ***" moss::*moss-version-number*)

;;;=============================================================================== 

:EOF