﻿;;;-*- Mode: Lisp; Package: "BENKEI" -*-
;;;===============================================================================
;;;11/03/10
;;;             AGENT BENKEI: DIALOG (file BENKEI-dialog.lisp)
;;;             Copyright Barthes@UTC, Sugawara@CIT 2011
;;;
;;;=============================================================================== 

#|
BENKEI is the assistant agent of CIT, which supports defining a UML diagram.
It can find ask to create UML diagrams.

The current file contains its library of dialogs.

Questions asked to BENKEI:
   - クラス 定義
   - クラス　加える
   - ボックス 加える
  
2011
 0310 creation
|#

(in-package :BENKEI)

(format t "~%;*** Loading BENKEI dialogs, package is: ~S" *package*)

;;「はい」をyes、「いいえ」をno に割り当てる
(defParameter *empty-words* '())
(eval-when (:load-toplevel :compile-toplevel :execute)
  (push 'はい moss::*yes-answers*)
  (push 'OK moss::*yes-answers*)
  (push 'いいえ moss::*no-answers*)
  )

;;;================================================================================
;;;
;;;                       MAIN CONVERSATION (CONTROL LOOP)
;;;
;;;================================================================================

;;; create a dialog header to be used for the main conversation

(defindividual 
  MOSS-DIALOG-HEADER
  (HAS-MOSS-LABEL "Main conversation loop")
  (HAS-MOSS-EXPLANATION "This is the main conversation loop.")
  (:var _main-conversation))

;; declare that conversation, replacing default conversation in the agent structure
(setf (omas::dialog-header PA_BENKEI) _main-conversation)

;;; debugging function
(defUn cc () (moss::start-conversation _main-conversation))

;;;*********** Here we need to add the TASK-LIST relation to the conversation object　　？？？
;(defrelation  "task-list" (:from "q-conversation") (:to "task")) ; JPB 1003

;;;===== states are declared as global variables
;;; the set of states can be considered as a plan to be executed for conducting the
;;; conversation with the master

;;;--------------------------------------------------------------------------------
(eval-when (:compile-toplevel :load-toplevel :execute)
  (proclaim '(special 
              _mc-entry-state
              _mc-get-input
              _mc-more?
              _mc-process
              _mc-sleep)))
;;;--------------------------------------------------------------------------------

;;;--------------------------------------------------------------- (MC) ENTRY-STATE
;;; ENTRY STATE
;(defUn ff () (start-conversation _main-conversation :first-time t))
;(defUn gg () (start-conversation _main-conversation))

;;; define an entry instance and record it in a global variable.
;;; ****** we should enter a frame on the conversation frame list, to handle
;;; success or failure returns from sub-dialogs. I.e. we should make a transition
;;; on the success or failure state of the main conversation.
;;; entry (_mc-failure _mc-success _mc-entry-state nil) ; no goal 

;;; Action:
;;;   =EXECUTE
;;;   - get ACTION if any from the dialog header
;;;   - reset conversation: HAS-MOSS-FACTS, HAS-MOSS-TASK, HAS-MOSS-TASK-LIST
;;;   - if ACTION IS non NIL, set conversation HAS-GOAL with a pattern obtaines from
;;;        sending ACTION a =create-pattern message
;;;     otherwise, reset conversation HAS-GOAL
;;;   - display welcome message
;;;
;;;   <Master enters text>
;;;
;;;   =RESUME
;;;   - if INPUT contains one of the abort commands, throws to :DIALOG-ERROR
;;;   - makes a transition to _MC-PROCESS

(defstate _mc-entry-state
  (:entry-state _main-conversation)		;;_main-conversationのエントリ状態として定義する
  (:label "Dialog start")				;;
  (:explanation 						;;プログラマ用のコメントか？
   "Initial state when the assistant starts a conversation. Send a ~
    welcome message and wait for data. Also entered on a restart ~
    following an abort.")
  (:reset-conversation)
  (:text "MCが_mc-entry-stateに遷移しました．「わたしはPDAの弁慶です」")
  (:question-no-erase
   ("~%- こんにちは．なにか御用ですか?"
    "~%- なにか知りたいことがありますか?"
    "~%- もしやって欲しいことがあれば、がんばってやります．"))	;;このステートメントは乱数で適当に表示される
  (:transitions 
   (:always :target _mc-process))			;;無条件で、状態_mc-processに遷移する．
  )

;;;------------------------------------------------------------------- (MC) PROCESS

(defstate _mc-process
  (:label "Process state of main conversation")
  (:explanation
   "Here we call the main process organized as a sub-conversation.")
  (:transitions
   (:always :sub-dialog _process-conversation		;;サブダイアログのエントリ状態
            :success _mc-more?						;; :successで終了なら_mc-more?へ
            :failure _mc-more?)))					;; :failureで終了なら_mc-more?へ
;;;----------------------------------------------------------------- (MC) GET-INPUT

(defstate _mc-get-input
  (:label "Get input")
  (:explanation
   "State in which the user inputs his request.")
  (:reset)
  (:question "- 入力してください (_mc-get-input)")
  (:transitions
   (:always :target _mc-process)))

;;;-------------------------------------------------------------------- (MC) MORE?
;;; Action:
;;;   - If answer is "rien", "non" transition to SLEEP
;;;   - if answer is "oui" transition to GET-INPUT
;;;   - anything else sends to PROCESS

(defstate _mc-more?
  (:label "More?")
  (:explanation "Asking the user it he wants to do more interaction.")
  (:reset-conversation)
  (:question-no-erase
   ("~%- なにかもっとすることがありますか?"		;;_mc-processから戻ってきたときに、この対話を行う
    "~%- OK.")
   ) 
  (:transitions 
   (:starts-with ("なにも") :target _mc-sleep)			;;「なにも」から始まる分ならsleep状態へ
   (:no :target _mc-sleep)							;;:noならsleep状態へ
   (:yes :target _mc-get-input)						;; :yes なら_mc-get-inputへ
   (:otherwise :target _mc-process)))				;; それ以外は_mc-processへ戻る



;;;--------------------------------------------------------------------- (MC) SLEEP

(defstate _mc-SLEEP
  (:label "Nothing to do")
  (:explanation
   "User said she wanted nothing more.")
  (:reset)
  (:text "- OK. J'attends que vous soyez pr?t.~%")
  (:question-no-erase 
   "- お待ちください．")
  (:transitions
   (:always :target _mc-process)))

;;;================================================================================
;;;
;;;                      MAIN CONVERSATION (EXECUTION PART)
;;;
;;;================================================================================

;;; this conversation is intended to process the input from te user by first
;;; determining the performative, then the list of possible tasks

(defsubdialog 
  _process-conversation 
  (:label "Process conversation")
  (:explanation 
   "Processing steps of the main conversation.")
  (:states _mc-eliza
           _mc-find-performative
           _mc-entry-state
           _mc-failure
           _mc-find-performative
           _mc-find-task
           _mc-select-task
           _mc-task-dialog)
  )

;;;===============================================================================
;;;                            Service functions
;;;===============================================================================

;;;-------------------------------------------------------------------- PRINT-TIME

(defUn print-time (conversation &optional (language *language*))
  (multiple-value-bind (second minute hour date month year)
                       (decode-universal-time (get-universal-time))
    ;(print "+++ entering print time  +++")
    (send conversation '=display-text
          (case language
            (:en
             (format nil "Current time is ~S:~S:~S and today is the ~S/~S/~S~%"
                     hour minute second date month year))
            (:jp
             (format nil "今は ~S:~S:~S ．今日は ~S/~S/~S~%"
                     hour minute second date month year))
            (:fr
             (form nil 
                     "Il est actuellement ~S:~S:~S et nous sommes aujourd'hui le ~
                      ~S/~S/~S~%"
                     hour minute second date month year)))
          )))

;;;===============================================================================
;;;                            Conversation states
;;;===============================================================================

;;;--------------------------------------------------------- (MC) FIND-PERFORMATIVE

(defstate _mc-find-performative
  (:entry-state _process-conversation)
  (:process-state _main-conversation)
  (:label "Find performative")
  (:explanation 
   "Process what the user said trying to determine the type of ~
    performative among :request :assert :command. Put the result if any ~
    into the performative slot of the conversation object.")
  (:transitions 
   ;; time?
   (:patterns (((?* ?y) "何" "時" (?* ?x))((?* ?y) "何" "日" (?* ?x)))
              :exec (print-time moss::conversation :jp) :success)
   ;; now really select performatives
   (:patterns ( ((?* ?x) "下さ" "い"))
              :set-performative (list :request)
              :target _mc-find-task)
   (:patterns (((?* ?x) "で" "す" ))
              :set-performative (list :assert)
              :target _mc-find-task)
   (:otherwise 
    :set-performative (list :command)
    :target _mc-find-task))
  )

;;;--------------------------------------------------------------------- (MC) ELIZA

(moss::defstate _mc-ELIZA
  (:label "ELIZA")
  (:explanation
   "Whenever MOSS cannot interpret what the user is saying, ELIZA is ~
    called to do some meaningless conversation to keep the user happy. ~
    It then record the master's input and transfers it to find ~
    performative state.")
  (:eliza)
  (:transitions
   (:always :target _mc-find-performative))
  )

;;;------------------------------------------------------------------- (MC) FAILURE

(defstate _mc-failure
  (:label "Failure state of main conversation")
  (:explanation
   "Failure state is entered when we return from a sub-dialog with a ~
    :failure tag. We check for more tasks to perform (listed in the ~
    task-list slot of the conversation object). If there are more, we ~
    execute the first one. If there are no more, we return with a ~
    failure tag.")
  (:answer-analysis)
  )

(defownmethod
  =answer-analysis _mc-failure (conversation input)
  (declare (ignore input))
  (let* ((task-list (HAS-MOSS-TASK-LIST conversation))
         task)
    (if task-list
      (progn
        ;; record next task
        (setq task (pop task-list))
        ;; remove it from task-list
        (send conversation '=replace 'HAS-MOSS-TASK-LIST task-list)
        ;; add it to conversation
        (send conversation '=replace 'HAS-MOSS-TASK (list task))
        ;; transfer to sub-dialog
        `(:transition ,_mc-task-dialog))
      ;; when no more tasks we return :failure and the state variable still contains
      ;; :failure. The crawler will return one more level until hitting a non
      ;; failure return containing a specific state (e.g. the one for the main
      ;; conversation)
      (list :failure)
      )))

;;;----------------------------------------------------------------- (MC) FIND-TASK

(defstate _mc-find-task
  (:label "Find task")
  (:explanation 
   "Process the input to determine the task to be undertaken. Combines ~
    the words from the sentence to see if they are entry points for ~
    the index property of any task. Collect all tasks for which there ~
    is an index in the sentence.")
  (:answer-analysis)
  )

(defownmethod
  =answer-analysis _mc-find-task (conversation input)
  "Using input to find tasks If none failure, if one, OK, if more, ~
   must ask user to select one. Checks for an entry point for an index ~
   of a task.
Arguments:
   conversation: current conversation
   input: list containing the input as a list of words"
  (let* ((performative (read-fact conversation :PERFORMATIVE))
         task-list)
    (moss::vformat "_mc-find-task /performative: ~S, package: ~S, input:~%  ~S" 
                   performative *package* input)
    ;; try to find a task from the input text, combining input words
    (setq task-list (moss::find-objects 
                     '(TASK 
                       (HAS-INDEX-PATTERN 
                        (TASK-INDEX (HAS-INDEX :is :?))))
                     input :all-objects t)) 
    (moss::vformat "_mc-find-task /possible tasks:~%  ~S" task-list)
    ;; filter tasks that do not have the right performative
    (setq task-list
          (mapcan #'(lambda (xx) 
                      (if (intersection performative (HAS-PERFORMATIVE xx))
                        (list xx)))
                  task-list))
    (moss::vformat "_mc-find-task /task list after performative check: ~S" task-list)
    (cond 
     ;; if empty, ask ELIZA
     ((null task-list)
      `(:transition ,_mc-ELIZA))
     ;; if one, then OK
     ((null (cdr task-list))
      ;; save results, make the task the conversation task
      (setf (HAS-MOSS-TASK conversation) task-list)
      `(:transition ,_mc-task-dialog))
     (t 
      ;; otherwise must select one from the results
      (setf (HAS-MOSS-TASK-LIST conversation) task-list)
      `(:transition ,_mc-select-task)))
    ))

;;;--------------------------------------------------------------- (MC) SELECT-TASK

(defstate 
  _mc-select-task
  (:label "mc select task")
  (:explanation
   "we have located more than one task. We rank the tasks by computing ~
    the average weight of the terms in the task index slot. We then ~
    record the list of tasks into the conversation task-list slot ~
    and activate the highest ranking task.")
  (:answer-analysis) 
  )

(defownmethod =answer-analysis _mc-select-task (conversation input)
  "we have located more than one task. We rank the tasks by computing ~
   the MYCIN combination weight of the terms in the task index slot. ~
   We then record the list of tasks into the conversation task-list slot ~
   and activate the highest ranking task."
  ;; the list of tasks is in the HAS-MOSS-TASK-LIST slot. Tasks are defined in the
  ;; XXX-tasks.lisp file of the application package
  (let* ((task-list (HAS-MOSS-TASK-LIST conversation))
         patterns weights result pair-list selected-task level word-weight-list)
    (moss::vformat "_select-task /input: ~S~&  task-list: ~S" input task-list)
    ;; first compute a list of patterns (combinations of words) from the input
    ;; 1-2-3-grams, i.e. max 3 words
    (setq patterns (mapcar #'car (moss::generate-access-patterns input)))
    (moss::vformat "_select-task /input: ~S~&  generated patterns:~&~S" 
                   input patterns)
    ;; then, for each task
    (dolist (task task-list)
      (setq level 0)
      ;; get the weight list (cached on the task p-list)
      (setq weights (moss::%get-INDEX-WEIGHTS task))
      (moss::vformat "_select-task /task: ~S~&  weights: ~S" task weights)
      ;; check the patterns according to the weight list
      (setq word-weight-list (moss::%get-relevant-weights weights patterns))  
      (moss::vformat "_select-task /word-weight-list:~&  ~S" word-weight-list)
      ;; combine the weights
      (dolist (item word-weight-list)
        (setq level (+ level (cadr item) (- (* level (cadr item))))))
      (moss::vformat "_select-task /level: ~S" level)
      ;; push the task and weight onto the result list
      (push (list task level) result)
      )
    (moss::vformat "_select-task /result:~&~S" result)
    ;; order the list
    (setq pair-list (sort result #'> :key #'cadr)) 
    (moss::vformat "_select-task /pair-list:~&  ~S" pair-list)
    ;; keep the first task whatever its score
    (setq selected-task (caar pair-list))
    ;; remove the task that have a weight less than task-threshold (default 0.4)
    (setq pair-list
          (remove nil 
                  (mapcar 
                   #'(lambda (xx) 
                       (if (>= (cadr xx) (omas::task-threshold omas::*omas*)) xx))
                   pair-list)))
    ;; if task-list is empty then return the first saved task
    ;; this may not be such a good policy if the score is too low
    (if (null pair-list)
      (progn
        ;; reset the task-list slot of the conversation object
        (setf (HAS-MOSS-TASK-LIST conversation) nil)
        ;; put the saved task into the task slot
        (setf (HAS-MOSS-TASK conversation) (list selected-task))
        ;; go to task-dialog
        `(:transition ,_mc-task-dialog)
        )
      (progn
        ;; remove the weights
        (setq task-list (mapcar #'car pair-list))
        ;; select the first task of the list
        (setq selected-task (pop task-list))
        (moss::vformat "_select-task /selected task: ~S" selected-task)
        ;; save the popped list in the task-list slot of the conversation object
        (setf (HAS-MOSS-TASK-LIST conversation) task-list)
        (setf (HAS-MOSS-TASK conversation) (list selected-task))
        ;; go to task-dialog
        `(:transition ,_mc-task-dialog)))
    ))

;;;--------------------------------------------------------------- (MC) TASK-DIALOG

(defstate _mc-task-dialog
  (:label "Task dialog")
  (:explanation 
   "We found a task to execute (in the GOAL slot of the conversation). ~
    We activate the dialog associated with this task.")
  ;; we launch the task dialog as a sub-conversation
  ;; the task contains the name of a sub-conversation header, e.g. _get-tel-nb
  ;; could be done within the defstate formalism
  (:answer-analysis)
  )

;;;********** must check for possible problems
(defownmethod
  =answer-analysis _mc-task-dialog (conversation input)
  "We prepare the set up to launch the task dialog.
Arguments:
   conversation: current conversation"
  (declare (ignore input))
  (let* ((task-id (car (send conversation '=get 'HAS-MOSS-TASK))))
    (moss::vformat "_mc-task-dialog /task-id: ~S dialog: ~S" 
                   task-id (HAS-DIALOG task-id))
    ;; we launch the task dialog as a sub-conversation
    ;; the task contains the name of a sub-conversation header, e.g. _get-tel-nb
    `(:sub-dialog ,(car (HAS-DIALOG task-id)) :failure ,_mc-failure)
    ))


;;;================================================================================

;;; here the strategy is to develop a specific dialog for each task. However, we 
;;; could also use a generic dialog consisiting in:
;;;    - getting the list of required parameters
;;;    - for each parameter, trying to extract info from data using predefined
;;;      rules or extraction patterns (Note that this is not much different than
;;;      having a predefined dialog, albeit focused on a single parameter)
;;;    - if some required parameters are missing, ask master about them using 
;;;      predefined questions
;;;    - shipping result to ad hoc staff agent

;;; The generic dialog could be an option offered to the programmer, which requires
;;; a suitable modeling of the tasks.

;;;================================================================================
;;;
;;;                              Sub-Conversations
;;;
;;;================================================================================

;;; Sub-conversations are defined by a conversation header, that points to the
;;; entry state.
;;; There are two output states:
;;;   SUCCESS: the result is stored into the RESULTS slot of the state-context
;;;   FAILURE: no result could be achieved
;;; In case of bad error there is a throw to an :error label
;;; data are obtained from the HAS-DATA slot of the context object

;;;================================================================================
;;;
;;;                      GENERIC TASK CONVERSATION
;;;
;;;================================================================================

;;; this subconversation takes a task model and fills in the required parameters
;;; before shipping the data (for possible optional args) and filled pattern to the 
;;; specialist

;;; probably not very useful

;;;================================================================================
;;;
;;;                           ADD BOX CONVERSATION
;;;
;;;================================================================================

;;; this conversation is intended to ask CIT-UML to add a box to the UML diagram

(defsubdialog 
  _add-box-conversation 
  (:label "Add box conversation")
  (:explanation 
   "We want our PA to ask CIT-UML to add a box.")
  (:states    _adb-entry-state  ; required by defstate            
              _adb-ask-product-name 
              _adb-pre-product-name 
              _adb-spec-name
              _adb-location
              _adb-execute
              _result-OK?
              )
  )

;;;================================================================================
;;;                  ADD BOX CONVERSATION SERVICE FUNCTIONS
;;;================================================================================

;;;-------------------------------------------------------------- (ADB) ENTRY-STATE

(defstate 
  _adb-entry-state
  (:label "Add box dialog entry")
  (:entry-state _add-box-conversation)
  (:explanation "We try to get destination from the input data.")
  (:execute-preconditions 
   (ide::format-debug "BENKAI> ~S" (read-fact moss::conversation :raw-input)))
  (:send-message :to :ORI :self PA_BENKEI :type :inform :action :store
                 :args (list(list (read-fact moss::conversation :raw-input)
		                  (read-fact moss::conversation :input))))
  (:transitions (:always :target _adb-pre-product-name))
  )

;;;--------------------------------------------------------- (ADB) PRE-PRODUCT-NAME

(defstate 
  _adb-pre-product-name
  (:label "Print product question")
  (:explanation "we want to send the question to ORI.")
  (:send-message-no-wait :to :ORI :self PA_BENKEI :type :inform :action :store
                         :args (list"- 製品の名前はなんですか? what is the name of product?"))
  (:transitions (:always :target _adb-ask-product-name))
  )

;;;--------------------------------------------------------- (ADB) ASK-PRODUCT-NAME

(defstate 
  _adb-ask-product-name
  (:label "Ask product name")
  (:explanation "we want to know the name of the product to post into the box.")
  (:question "- 製品の名前はなんですか? what is the name of product?")
  (:answer-analysis))

(defownmethod =answer-analysis _adb-ask-product-name (conversation input)
  "We process the answer and record it unless we get an abort.
Arguments:
   agent: assistant."
  (replace-fact conversation :product-name input)
  (list :transition _adb-spec-name))

;;;---------------------------------------------------------------- (ADB) SPEC-NAME

(defstate 
  _adb-spec-name
  (:label "Ask specification name")
  (:explanation "we want to know the name of the spec to post into the box.")
  (:send-message :to :ORI :self PA_BENKEI :type :inform :action :store
                 :args (list(list (read-fact moss::conversation :raw-input)
		                  (read-fact moss::conversation :input))))
  (:question "- 仕様書の名前はなんですか? what is the name of the specification")
  (:answer-analysis))

(defownmethod =answer-analysis _adb-spec-name (conversation input)
  "We process the answer and record it unless we get an abort.
Arguments:
   agent: assistant."
  (replace-fact conversation :spec-name input)
  (list :transition _adb-location))

;;;------------------------------------------------------------------ (ADB) LOCATION

(defstate 
  _adb-location
  (:label "Ask location")
  (:explanation "we want to know the location of the box.")
  (:send-message :to :ORI :self PA_BENKEI :type :inform :action :store
                 :args (list(list (read-fact moss::conversation :raw-input)
		                  (read-fact moss::conversation :input))))
  (:question 
   ( "-箱の位置はどこにしますか? where do you want to pt in the specification?")
   )
  (:answer-analysis))

(defownmethod =answer-analysis _adb-location (conversation input)
  "We process the answer and record it unless we get an abort.
Arguments:
   agent: assistant."
  (replace-fact conversation :location input)
  (list :transition _adb-execute))

;;;------------------------------------------------------------------- (ADB) EXECUTE

(defstate 
  _adb-execute
  (:label "Executing add")
  (:explanation "we ship the command to the CIT-UML agent.")
  (:send-message :to :CIT-UML :self PA_BENKEI :action :add-box
                 :args `(((:data . ,(append
                                     (moss::read-fact moss::conversation :product-name)
                                     (moss::read-fact moss::conversation :spec-name)
                                     (moss::read-fact moss::conversation :location)
                                     ))
                          ))
                 )
  (:transitions
   (:always :target _adb-result-OK?))
  )
;;;------------------------------------------------------------- (ADB) RESULT-OK?

(defstate 
  _adb-result-OK?
  (:label "Result OK?")
  (:explanation "we ask user if action was OK.")
  (:question 
   ("- これで良いでしょうか?"
    "- Is the result OK?")
   )
  (:transitions
   (:yes :success)
   (:otherwise :target _adb-undo))
  )

;;;------------------------------------------------------------------- (ADB) UNDO

(defstate 
  _adb-undo
  (:label "Send undo")
  (:explanation "we ship the UNDO command to the CIT-UML agent.")
  (:send-message :to :CIT-UML :self PA_BENKEI :action :undo)
  (:transitions
   (:always :failure))
  )

;;;===============================================================================
;;;
;;;                          EXPLAIN CONVERSATION
;;;
;;;===============================================================================
;;; this conversation is intended to explain the meaning of a concept.

(defsubdialog 
  _explain-conversation 
  (:label "Explain conversation")
  (:states _exp-entry-state)
  (:explanation 
   "The data is searched for a concept to be explained.")
  )

;;;----------------------------------------------------- (EXP) EXPLAIN-ENTRY-STATE

(defstate 
  _exp-entry-state
  (:label "Explain dialog entry")
  (:entry-state _explain-conversation)
  (:explanation "The data is cleaned prior to look for an entry point.")
  (:transitions
   ;;remove useless words explain, define, or else...
   (:patterns (((?* ?x) "d?finir" (?* ?y))
               ((?* ?x) "d?finition" (?* ?y))
               ((?* ?x) "expliquer" (?* ?y))
               ((?* ?x) "expliquez" (?* ?y))
               ((?* ?x) "que" "veut" "dire" (?* ?y))
               ("c" "est" "quoi" (?* ?y) "?")
               ((?* ?y) "c" "est" "quoi" (?* ?x))
               )
              :keep ?y ; put a list of the binding value into context HAS-DATA
              :sub-dialog _print-concept-documentation-conversation)
   ;; we do not know how the user got here and keep everything
   (:otherwise :sub-dialog _print-concept-documentation-conversation)
   )
  )

;;;================================================================================
;;;
;;;                      GET ADDRESS CONVERSATION
;;;
;;;================================================================================

;;; this conversation is intended to print the address of a person or of a
;;; place. It fills the task pattern and sends the message to the address specialist.
;;; The simplest version is to send to the :SA-ADDRESS agent, wait for the result and
;;; print it.

(defsubdialog 
  _get-address-conversation 
  (:label "Get address conversation")
  (:explanation 
   "Master is trying to obtain an address.")
  (:states    _get-address-dialog  ; required by defstate
              _gad-entry-state
              _gad-dont-understand
              _gad-try-again
              _gad-sorry)
  )

;;;================================================================================
;;;                  GET ADDRESS CONVERSATION SERVICE FUNCTIONS
;;;================================================================================

;;;============================ printing adresses ================================
;;;********** this should be improved to handle office address and other particular
;;; types of addresses

;;;------------------------------------------------------------------- GET-ADDRESS

(defUn get-address (pattern &key (kind "adresse postale"))
  "try to extract from a structured description the sub-structure describing the ~
   address of the required type.
    (\"personne\" (\"nom\" \"Barth?s\") (\"pr?nom\" \"Jean-Paul\") 
       (\"adresse\" 
            (\"domicile\"
                  (\"rue\" \"14, All?e de la Tilloye\") (\"code postal\" \"60200\")
                  (\"ville\" \"Compi?gne\")
                  (\"pays\" \"France\")))
Arguments:
   pattern: the structured data
   kind (key): the type of address required
Return:
   the corresponding sub-structure or nil."
  ;; if the pattern does not contain the "address" property, then return nil
  (let ((address (assoc "address" (cdr pattern) :test #'string-equal)))
    (print address)
    (when address
      ;; otherwise, check for the corresponding types of addresses
      (remove nil
              (mapcar #'(lambda (xx) (print (car xx)) 
                         (if (moss::%pclass? (car xx) kind) xx))
                      (cdr address))))))
#|
(setq *language* :fr)
(setq pattern '("personne" ("nom" "Barthes") ("pr?nom" "Jean-Paul")
                ("adresse"
                 ("domicile" ("rue" "14, All?e de la Tilloye")
                  ("ville" "Compi?gne") ("pays" "France")
                  ("code postal" "60200"))
                 ("adresse professionnelle"
                  ("rue" "Universit? de Technologie de Compi?gne"
                   "Centre de Recherche de Royallieu"
                   "Departement de G?nie Informatique"
                   "UMR CNRS 6599 HEUDIASYC")
                  ("ville" "Compi?gne cedex") ("code postal" "60206")))
                ("t?l?phone"
                 ("t?l?phone du domicile"
                  ("num?ro" "+33 (0)3 44 23 31 37"))
                 ("t?l?phone professionnel"
                  ("num?ro" "+33 (0)3 44 23 44 66"))
                 ("portable" ("num?ro" "+33 (0)6 80 45 32 67")))
                ("email" ("email" ("e-adresse" "barthes@utc.fr")))
                ("page web"
                 ("adresse web" ("URL" "http://www.utc.fr/~barthes")))))

(get-address albert::pattern)
(get-address albert::pattern :kind "domicile")
(get-address albert::pattern :kind "adresse professionnelle")
|#

(defUn str-equal (aa bb)
  "make sure arguments are strings before comparing."
  (and (stringp aa)(stringp bb)(string-equal aa bb)))

;;;----------------------------------------------------------------------- PCLASS?

(defUn pclass? (input class-ref)
  "tests if input belongs to the transitive closure of class-ref. Only works in ~
   the ALBERT package. The language is important.
Arguments:
   input: some expression that must be a string
   class-ref: a string naming a class
Return:
   nil if not the case, something not nil otherwise."
  (member input (moss::%get-subclass-names class-ref) :test #'string=))

#|
? (pclass? (car '("?tudiant"
                  ("nom" "Sato") ("pr?nom" "Gilson")
                  ("adresse"
                   ("adresse postale" 
                    ("rue" "Centre de Recherche de Royallieu" "BP 20529")
                    ("ville" "Compi?gne cedex") ("code postal" "60205")))))
           "personne")
("?tudiant")
|#
;;;----------------------------------------------------------------- PRINT-ADDRESS

(defUn print-address (addr-list)
  ;(format t "~&+++ ALBERT/ print-address language: ~S address:~% ~S" 
  ;         *language* addr-list)
  (omas::assistant-display-text 
   PA_BENKEI
   (format nil "~{~A~^~%~}" 
           (mapcar #'print-single-address addr-list))))

;;;------------------------------------------------------------ PRINT-HOME-ADDRESS

(defUn print-home-address (addr-list)
  ;(format t "~&+++ BENKEI/ print-address language: ~S address:~% ~S" 
  ;         *language* addr-list)
  (omas::assistant-display-text 
   PA_BENKEI
   (format nil "~{~A~^~%~}" 
           (mapcar #'print-single-home-address addr-list))))

;;;---------------------------------------------------------- PRINT-SINGLE-ADDRESS

(defUn print-single-address (addr)
  "input is something like:
     (\"personne\" (\"nom\" \"Barthes\") (\"pr?nom\" \"Jean-Paul\") 
       (\"adresse\" 
            (\"adresse du domicile\"
                  (\"rue\" \"14, All?e de la Tilloye\") (\"code postal\" \"60200\")
                  (\"ville\" \"Compi?gne\")
                  (\"pays\" (\"pays\" (\"nom\" \"France\")))))
   to obtain
     Barthes, Jean-Paul
        adresse du domicile:
           14, all?e de la ...
           60200 Compi?gne
           France
   or something similar for companies."
  (let (result details)
    (cond
     ;; first test person or subclass
     ((pclass? (car addr) "person")
      (push (moss::%pformat "~{~A~^ ~}, ~{~A~^ ~}" (cdr addr)
                            "name" "first name") 
            result))
     ;; then test organization or subclass
     ((moss::%pclass? (car addr) "organization")
      (push (moss::%pformat  "~{~A~^, ~}" (cdr addr) "name") result)))
    
    ;; then process the address itself (if there)
    (cond
     ;; address? (check against all synonyms and subclasses)
     ((setq details (moss::%get-value-from-class-ref "address" (cdr addr)))
      ;; get standard address  (("street" "14, ...")(...)...)
      ;; for each type of address print the corresponding info
      (dolist (item details)
        (push (format nil "   ~A:" (car item)) result)
        (push (moss::%pformat  
               "~5T~{~A~^~&~5T ~}~%~5T~{~A~^ ~} ~{~A~^ ~}"
               (cdr item)
               "street" "postal code" "city")
              result)
        ;(format *debug-io* "~% item: ~S" item)
        ;; country is different since it is a structured object
        (let* ((expr (car (moss::%get-value-from-prop-ref "country" (cdr item))))
               (country-name (moss::%get-value-from-prop-ref "name" (cdr expr))))
          ;(format *debug-io* "~% expr: ~S; country-name: ~S" expr country-name)
          (if country-name (push (format nil "~5T~{~A~^ ~}" country-name)
                                 result)))
        )))
    
    ;; now print
    (format nil "~{~A~^~%~}" (reverse result))))

#|
(setq pattern '("personne" ("nom" "Barthes") ("pr?nom" "Jean-Paul")
                ("adresse"
                 ("domicile" ("rue" "14, All?e de la Tilloye")
                  ("ville" "Compi?gne") ("pays" "France")
                  ("code postal" "60200"))
                 ("adresse professionnelle"
                  ("rue" "Universit? de Technologie de Compi?gne"
                   "Centre de Recherche de Royallieu"
                   "Departement de G?nie Informatique"
                   "UMR CNRS 6599 HEUDIASYC")
                  ("ville" "Compi?gne cedex")
                  ("pays" ("pays" ("nom" "France")))
                  ("code postal" "60206")))
                ("t?l?phone"
                 ("t?l?phone du domicile"
                  ("num?ro" "+33 (0)3 44 23 31 37"))
                 ("t?l?phone professionnel"
                  ("num?ro" "+33 (0)3 44 23 44 66"))
                 ("portable" ("num?ro" "+33 (0)6 80 45 32 67")))
                ("email" ("email" ("e-adresse" "barthes@utc.fr")))
                ("page web"
                 ("adresse web" ("URL" "http://www.utc.fr/~barthes")))))

? (moss::%get-value-from-class-ref "adresse" (cdr pattern))
(("domicile" ("rue" "14, All?e de la Tilloye") ("ville" "Compi?gne") ("pays" "France")
  ("code postal" "60200"))
 ("adresse professionnelle"
  ("rue" "Universit? de Technologie de Compi?gne" "Centre de Recherche de Royallieu"
   "Departement de G?nie Informatique" "UMR CNRS 6599 HEUDIASYC")
  ("ville" "Compi?gne cedex") ("code postal" "60206")))

? (print-single-address '("personne" ("nom" "Barthes") ("pr?nom" "Jean-Paul")
                          ("adresse"
                           ("domicile" ("rue" "14, All?e de la Tilloye")
                            ("ville" "Compi?gne") ("pays" ("pays" ("nom" "France")))
                            ("code postal" "60200"))
                           ("adresse professionnelle"
                            ("rue" "Universit? de Technologie de Compi?gne"
                             "Centre de Recherche de Royallieu"
                             "Departement de G?nie Informatique"
                             "UMR CNRS 6599 HEUDIASYC")
                            ("ville" "Compi?gne cedex")
                            ("pays" ("pays" ("nom" "France")))
                            ("code postal" "60206")))))
"Barthes, Jean-Paul
   domicile:
     14, All?e de la Tilloye
     60200 Compi?gne
     France
   adresse professionnelle:
     Universit? de Technologie de Compi?gne
      Centre de Recherche de Royallieu
      Departement de G?nie Informatique
      UMR CNRS 6599 HEUDIASYC
     60206 Compi?gne cedex
     France"
|#
;;;----------------------------------------------------- PRINT-SINGLE-HOME-ADDRESS

(defUn print-single-home-address (addr)
  (let (result details)
    (cond
     ;; first test person or subclass
     ((pclass? (car addr) "person")
      (push (moss::%pformat "~{~A~^ ~}, ~{~A~^ ~}" (cdr addr)
                            "name" "first name") 
            result)))
    
    ;; then process the address itself (if there)
    ;; address? (check against all synonyms and subclasses)
    (setq details (get-address addr :kind "home address"))
    ;; get standard address  (("street" "14, ...")(...)...)
    ;; for each type of address print the corresponding info
    (cond 
     (details
      (dolist (item details)
        (push (format nil "   ~A:" (car item)) result)
        (push (moss::%pformat  
               "~5T~{~A~^~&~5T ~}~%~5T~{~A~^ ~} ~{~A~^ ~}~%~5T~{~A~^ ~}"
               (cdr item)
               "street and number" "zip" "town" "country")
              result)))
     (t (push "   I do not know the home address...
" result)))    
    ;; now print
    (format nil "~{~A~^~%~}" (reverse result))))

#|
(setq pattern '("personne" ("nom" "Barthes") ("pr?nom" "Jean-Paul")
                ("adresse"
                 ("domicile" ("rue" "14, All?e de la Tilloye")
                  ("ville" "Compi?gne") ("pays" "France")
                  ("code postal" "60200"))
                 ("adresse professionnelle"
                  ("rue" "Universit? de Technologie de Compi?gne"
                   "Centre de Recherche de Royallieu"
                   "Departement de G?nie Informatique"
                   "UMR CNRS 6599 HEUDIASYC")
                  ("ville" "Compi?gne cedex") ("code postal" "60206")))
                ("t?l?phone"
                 ("t?l?phone du domicile"
                  ("num?ro" "+33 (0)3 44 23 31 37"))
                 ("t?l?phone professionnel"
                  ("num?ro" "+33 (0)3 44 23 44 66"))
                 ("portable" ("num?ro" "+33 (0)6 80 45 32 67")))
                ("email" ("email" ("e-adresse" "barthes@utc.fr")))
                ("page web"
                 ("adresse web" ("URL" "http://www.utc.fr/~barthes")))))
(print-single-home-address pattern)
(setq pattern '("personne" ("nom" "Barthes") ("pr?nom" "Jean-Paul")
                ("adresse"
                 
                 ("adresse professionnelle"
                  ("rue" "Universit? de Technologie de Compi?gne"
                   "Centre de Recherche de Royallieu"
                   "Departement de G?nie Informatique"
                   "UMR CNRS 6599 HEUDIASYC")
                  ("ville" "Compi?gne cedex") ("code postal" "60206")))
                ("t?l?phone"
                 ("t?l?phone du domicile"
                  ("num?ro" "+33 (0)3 44 23 31 37"))
                 ("t?l?phone professionnel"
                  ("num?ro" "+33 (0)3 44 23 44 66"))
                 ("portable" ("num?ro" "+33 (0)6 80 45 32 67")))
                ("email" ("email" ("e-adresse" "barthes@utc.fr")))
                ("page web"
                 ("adresse web" ("URL" "http://www.utc.fr/~barthes")))))
|#
;;;===============================================================================
;;;                        GET ADDRESS CONVERSATION STATES
;;;===============================================================================

;;;----------------------------------------------------- (GAD) ADDRESS-ENTRY-STATE

(defstate 
  _get-address-entry-state
  (:entry-state _get-address-conversation)
  (:label "Get address dialog entry")
  (:explanation "Assistant is sending a free-style message to the ADDRESS agent.")
  (:send-message :to :TYL-ADDRESS :self PA_BENKEI :action :get-address
                 :args `(((:data . ,(moss::read-fact moss::conversation :input))
                          (:language . :en)))
                 )
  (:transitions
   (:on-failure :target _gad-dont-understand)
   (:test (> (length (read-fact moss::conversation :answer)) 3)
          :exec
          (progn
            ;; record the pattern for printing selection summaries into the working 
            ;; list
            (replace-fact moss::conversation :control-string "~{~A~^ ~}, ~{~A~^ ~}")
            (replace-fact moss::conversation :properties (list "name" "first name"))
            )
          :sub-dialog _make-choice-conversation :failure _gad-dont-understand)
   (:otherwise
    :print-answer #'print-address :success))
  )

;;;----------------------------------------------------- (GAD) GAD-DONT-UNDERSTAND

(defstate 
  _gad-dont-understand
  (:label "Did not understand. Ask master.")
  (:explanation "could not find the address with the given information. Thus, ~
                 asking master for whose address...")
  (:question-no-erase "~%- Whose address?")
  (:answer-type :answer)
  (:transitions
   (:always :target _gad-try-again)))

;;;--------------------------------------------------------------- (GAD) GAD-SORRY

(defstate 
  _gad-sorry
  (:label "Address failure.")
  (:explanation "We don't have the requested address.")
  (:text "Sorry, if you were looking for an address, I can't find it.~%")
  (:reset)
  (:transitions (:failure))
  )

;;;----------------------------------------------------------- (GAD) GAD-TRY-AGAIN

(defstate 
  _gad-try-again
  (:label "Ask again for address.")
  (:explanation "Assistant is asking :TYL-ADDRESS.")
  (:send-message :to :TYL-ADDRESS :self PA_BENKEI :action :get-address
                 :args `(((:data . ,(read-fact moss::conversation :input))
                          (:language . :en))))
  (:transitions
   (:on-failure :target _gad-sorry)
   (:otherwise
    :print-answer #'print-address
    :success))
  )

;;;===============================================================================
;;;
;;;                      GET HOME ADDRESS CONVERSATION
;;;
;;;===============================================================================

;;; this conversation is intended to print the home address of a person.
;;; It builds a task pattern and sends the message to the address specialist.
;;; The simplest version is to send to the :TYL-ADDRESS agent, wait for the result and
;;; print it.

(defsubdialog 
  _get-home-address-conversation 
  (:label "Get address conversation")
  (:explanation 
   "Master is trying to obtain a private address.")
  (states     _get-home-address-conversation  ; required by defstate
              _ghad-entry-state
              _ghad-dont-understand
              _ghad-process-answer
              _ghad-try-again
              _ghad-try-again-process-answer
              _ghad-sorry)
  )

;;;------------------------------------------- (GHAD) GET-HOME-ADDRESS-ENTRY-STATE

(defstate 
  _get-home-address-entry-state
  (:entry-state _get-home-address-conversation)
  (:label "Get home address dialog entry")
  (:explanation "Assistant is sending a free-style message to the ADDRESS agent.")
  (:send-message :to :TYL-ADDRESS :self PA_BENKEI :action :get-address
                 :args `(((:data . ,(read-fact moss::conversation :input))
                          (:language . :en)
                          #|
(:pattern . ("person" 
                                       ("name")
                                       ("first name")
                                       ("address" 
                                        ("home address" 
                                         ("street") ("postal code") ("city")))))
|#)))
  (:answer-analysis)
  )

(defownmethod =answer-analysis _get-home-address-entry-state (conversation input)
  "Home address: we got an answer from the ADDRESS message or else the master got ~
   impatient.
Arguments:
   agent: assistant."
  (declare (ignore input))
  (let* ((agent (car (has-agent conversation)))
         (answer (omas::answer agent))
         (*language* :en)
         )
    ;; answer contains either the word "failure" or a list of objects expressed as
    ;; strings
    ;; (("personne" ("nom" "Barthes") ("pr?nom" "Jean-Paul") :home-phone "...")*)
    ;; 
    (moss::vformat "~&Answer to the send-request: ~S" answer)
    ;; on failure do something
    (cond
     ((equal "failure" answer)
      `(:transition ,_ghad-dont-understand))
     ;; if more than 3 persons, then ask the user to make a choice
     ((> (length answer) 3)
      ;; record answer into results
      (replace-fact conversation :results answer)
      ;; record the pattern for printing selection summaries into the working list
      (replace-fact conversation :control-string "~{~A~^ ~}, ~{~A~^ ~}")
      (replace-fact conversation :properties (list "name" "first name"))
      ;; then call sub-conversation
      `(:sub-dialog _make-choice-conversation
                    ;:success :success  ; normally the default
                    :failure ,_ghad-dont-understand))
     (t 
      ;(format t "~&+++ address: ~S" input)
      (print-home-address answer)
      `(:success)))))

;;;-------------------------------------------------------- (GHAD) DONT-UNDERSTAND

(defstate 
  _ghad-dont-understand
  (:label "Did not understand. Ask master.")
  (:explanation "could not find the address with the given information. Thus, ~
                 asking master for whose address...")
  (:question "~%- Whose home address?")
  (:answer-type :answer)
  (:transitions
   (:always :save :target _ghad-try-again)))

;;;------------------------------------------------------------------ (GHAD) SORRY

(defstate 
  _ghad-sorry
  (:label "Address failure.")
  (:explanation "We don't have the requested address.")
  (:text 
   "Sorry, if you were looking for a home address, I can't find it.~%")
  (:reset)
  (:immediate-transitions (:failure))
  (:no-resume))

;;;-------------------------------------------------------------- (GHAD) TRY-AGAIN

(defstate 
  _ghad-try-again
  (:label "Ask again for address.")
  (:explanation "Assistant is asking :TYL-ADDRESS.")
  (:send-message :to :TYL-ADDRESS :self PA_BENKEI :action :get-address
                 :args `(((:data . ,(read-fact moss::conversation :input))
                          (:language . :en)
                          (:pattern . ("person" 
                                       ("name")
                                       ("first name")
                                       ("address" 
                                        ("home address" 
                                         ("street") ("postal code") ("city"))))))))
  (:transitions
   (:on-failure :target _ghad-sorry)
   (:otherwise
    :print-answer #'print-home-address)
   )
  )



;;;===============================================================================
;;;
;;;                      GET NEWS BY SECTION CONVERSATION
;;;
;;;===============================================================================

;;; this conversation is intended to print news about a specific topic. It fills 
;;; the task pattern and sends the message to the news specialist.

(defsubdialog 
  _get-news-by-section-conversation 
  (:label "Get news by section conversation")
  (:explanation 
   "Master is trying to obtain news of a specific section.")
  (:states    _get-news-by-section-dialog  ; required by defstate
              _gnbsd-entry-state
              _gnbsd-dont-understand
              _gnbsd-try-again
              _gnbsd-sorry)
  )

;;;----------------------------------------------------------- (GNBSD) ENTRY-STATE

(defstate 
  _get-news-by-section-entry-state
  (:entry-state _get-news-by-section-conversation)
  (:label "Get news by section dialog entry")
  (:explanation "Assistant is sending a free-style message to the NEWS agent.")
  (:send-message :to :NEWS :self PA_BENKEI :action :get-news-by-section
                 :args `(((:data . ,(moss::read-fact moss::conversation :input))
                          (:language . :en)
                          ))
                 )
  (:transitions
   (:on-failure :target _gnbsd-dont-understand)
   (:otherwise :exec (print-news (read-fact PA_BENKEI :answer)) :success))
  )

;;;------------------------------------------------------- (GNBSD) DONT-UNDERSTAND

(defstate 
  _gnbsd-dont-understand
  (:label "Did not understand. Ask master.")
  (:explanation "could not find news in the requested section. Thus, ~
                 asking master for what section...")
  (:question-no-erase "~%- What topic (section) do you mean exactly?")
  (:answer-type :answer)
  (:transitions
   (:always  :target _gnbsd-try-again)))

;;;----------------------------------------------------------------- (GNBSD) SORRY

(defstate 
  _gnbsd-sorry
  (:label "News obtention failure.")
  (:explanation "We don't have the requested news.")
  (:text "Sorry, I could not find anything.~%")  
  (:reset)
  (:immediate-transitions (:failure))
  (:no-resume))

;;;------------------------------------------------------------- (GNBSD) TRY-AGAIN

(defstate 
  _gnbsd-try-again
  (:label "Ask again for news.")
  (:explanation "Assistant is asking :NEWS.")
  (:send-message :to :NEWS :self PA_BENKEI :action :get-news-by-section
                 :args `(((:data . ,(moss::read-fact moss::conversation :input))
                          (:language . :en))))
  (:transitions
   (:on-failure :target _gnbsd-dont-understand)
   (:otherwise :exec (print-news (read-fact PA_BENKEI :answer)) :success))
  )

;;;===============================================================================
;;;
;;;                       GET LAST NEWS CONVERSATION
;;;
;;;===============================================================================

;;; this conversation is intended to show the 5 last published news

(defsubdialog
  _GET-LAST-NEWS-CONVERSATION
  (:label "Get the more recent news")
  (:explanation 
   "This dialog is meant to show the last news.")
  (:states    _gln-entry-state
              _gln-dont-understand
              _gln-try-again
              _gln-sorry)
  )

;;;-------------------------------------------------------------- PRINT-SINGLE-NEW

(defUn print-news-item (new)
  "prints the title (a list of words) and the content (a list containing a string)."
  (format nil "~A ~A (~A) : ~{~A ~}~%    ~A" 
          (cadr (assoc :current-date new))
	  (cadr (assoc :current-hour new))
	  (or (cadr (assoc :author new)) "anonyme")
	  (or (cdr (assoc :title new)) (list "no title"))
          (cadr (assoc :content new))))

;;;-------------------------------------------------------------------- PRINT-NEWS

(defUn print-news (news-list)
  ;(format t "PARAMETRO A PRINT-NEWS ~S~%" news-list)
  (omas::assistant-display-text 
   PA_BENKEI
   (format nil "~{~A~^~%~}" (mapcar #'print-news-item news-list))))

;;;---------------------------------------------------- (gln) GET-LAST-NEWS-DIALOG
;;;********** lust check if :answer contains the answer and the "failure" mechanism

(defstate 
  _gln-entry-state
  (:label "Get last news dialog")
  (:entry-state _get-last-news-conversation)
  (:explanation 
   "Sends message to the news agent, no arguments are necessary.")
  (:send-message :to :NEWS :self PA_BENKEI :action :get-last-news
                 :args `(((:language . :en))))
  (:transitions
   (:on-failure :target _gln-sorry)
   (:otherwise :exec (print-news (read-fact moss::conversation :answer))
               :success))
  )

;;;------------------------------------------------------------------- (GLN) SORRY

(defstate 
  _gln-sorry
  (:label "Obtaining news failure.")
  (:explanation "NEWS failed to obtain news.")
  (:answer "~%- ~%- Je n'ai rien trouv?. ~
            Peut-?tre n'y a-t-il pas de nouvelles sous cette rubrique ?")
  (:reset)			
  (:transitions 
   (:always :failure))
  )

;;;===============================================================================
;;;
;;;                      GET TELEPHONE NUMBER CONVERSATION
;;;
;;;===============================================================================

;;; this conversation is intended to print the telephone number of a person or 
;;; of a place. 

(defsubdialog 
  _get-phone-conversation
  (:label "Get phone dialog")
  (:explanation "Master is trying to obtain a telephone number.")
  (:states    _gpd-entry-state
              _gpd-dont-understand
              _gpd-try-again
              _gpd-sorry)
  )

;;;-------------------------------------------------------------------- MAKE-ALIST

(defUn make-alist (ll)
  (cond ((null ll) nil)
        (t (cons (list (car ll)(cadr ll)) 
                 (make-alist (cddr ll))))))
#|
? (make-alist '(:OWNER ($E-PERSON.1) :CELL-PHONE "(0)6 80 45 32 67" :OFFICE-PHONE
  "(0)3 44 23 44 66" :HOME-PHONE "(0)3 44 23 31 37"))
((:OWNER ($E-PERSON.1)) (:CELL-PHONE "(0)6 80 45 32 67")
 (:OFFICE-PHONE "(0)3 44 23 44 66") (:HOME-PHONE "(0)3 44 23 31 37"))
|#
;;;------------------------------------------------------------- (GPD) ENTRY-STATE

(defstate 
  _gpd-entry-state
  (:label "Get phone dialog")
  (:entry-state _get-phone-conversation)
  (:explanation "Assistant is sending a free-style message to the ADDRESS agent.")
  (:send-message :to :TYL-ADDRESS :self PA_BENKEI :action :get-phone-number
                 :args `(((:data . ,(read-fact moss::conversation :input))
                          (:language . :en)
                          #|
(:pattern . ("person" ("name")("first name")
             ("phone" ("telephone"))))
|#)))
  (:transitions
   (:on-failure :target _gpd-dont-understand)
   (:otherwise
    :exec (omas::assistant-display-text 
           PA_BENKEI (moss::make-print-list (read-fact moss::conversation :answer)))
    :success))
  )

;;;----------------------------------------------------- (GPD) GPD-DONT-UNDERSTAND

(defstate 
  _gpd-dont-understand
  (:label "Did not understand. Ask master.")
  (:explanation "Assistant is asking master for whose phone.")
  (:question "- Whose telephone number?")
  (:answer-type :answer)
  (:transitions
   (:always :target _gpd-try-again)
   ))

;;;--------------------------------------------------------------- (GPD) GPD-SORRY

(defstate 
  _gpd-sorry
  (:label "Phone number failure.")
  (:explanation "We don' have the requested number.")
  (:reset)
  (:text "- Sorry, I can't find your number.")
  (:transitions (:failure))
  )

;;;----------------------------------------------------------- (GPD) GPD-TRY-AGAIN

(defstate 
  _gpd-try-again
  (:label "Ask again for phone number.")
  (:explanation "Assistant is asking :TYL-ADDRESS.")
  (:send-message :to :TYL-ADDRESS :self PA_BENKEI :action :get-phone-number
                 :args `(((:data . ,(read-fact moss::conversation :input))
                          (:language . :en)
                          ;; we do not give pattern to allow organizations
                          #|(:pattern . ("personne" ("nom")("pr?nom")
                                       ("t?l?phone" ("t?l?phone"))))
|#)))
  (:transitions
   (:on-failure :target _gpd-sorry)
   (:otherwise
    :exec (omas::assistant-display-text 
           PA_BENKEI (moss::make-print-list (read-fact moss::conversation :answer)))
    :success))
  )

;;;===============================================================================
;;;
;;;                  PRINT CONCEPT DOCUMENTATION CONVERSATION
;;;
;;;===============================================================================

;;; this conversation is meant to print the definition of a concept or properties
;;; to be extracted from a list of words

(defsubdialog
  _PRINT-CONCEPT-DOCUMENTATION-CONVERSATION
  (:label "Print concept documentation")
  (:explanation 
   "This dialog is meant to print the documentation attached to concepts ~
    or properties referenced by a list of words.")
  (:states _pcd-entry-state
           _pcd-sorry
           )
  )

;;;------------------------------------------------------------- (PCD) ENTRY-STATE

(defstate 
  _pcd-entry-state
  (:label "start of the PRINT CONCEPT DOCUMENTATION dialog")
  (:entry-state _print-concept-documentation-conversation)
  (:explanation 
   "Initial state may have data left in the input area. The data either represent ~
    an entry point or something else that should be used to select objects.")
  (:answer-analysis)
  )

(defownmethod
  =answer-analysis _pcd-entry-state (conversation input)
  "Using list of words to access objects. If one, OK, if more, must ask user to ~
   select one, if none, failure."
  ;(declare (ignore more-args))
  (let* ((*language* :fr)
         entry-point-list object-list)
    (catch :return
      (moss::vformat "_pcd-entry-state /input: ~%  ~S" input)
      ;; first get entry-points
      (setq entry-point-list (moss::find-best-entries input))
      
      ;; if we cannot find anything locally try for a what-is conversation
      (unless entry-point-list
        (throw :return 
               `(:sub-dialog _what-is-conversation :failure ,_pcd-sorry)))
      
      ;; get objects corresponding to entry points
      ;(moss::vformat "_pcd-entry-state /entry-point-list: ~%  ~S" entry-point-list)
      (setq object-list (delete-duplicates (mapcan #'access entry-point-list)))
      (moss::vformat "_pcd-entry-state /object-list: ~%  ~S" object-list)
      
      ;; first filter concepts and properties
      (setq object-list
            (mapcan #'(lambda (xx)
                        (or 
                         (moss::%is-concept? xx)
                         (moss::%is-attribute? xx)
                         (moss::%is-relation? xx)
                         )
                        (list xx))
                    object-list))
      ;(print object-list)
      (if object-list
        ;; print documentation
        (dolist (obj-id object-list)
          (cond
           ((moss::%is-concept? obj-id)
            (send obj-id '=get-documentation :lead "CONCEPT: " 
                  :final-new-line t)
            (send conversation '=display-text *answer*))
           ((moss::%is-attribute? obj-id)
            (send obj-id '=get-documentation :lead "ATTRIBUT: " 
                  :final-new-line t)
            (send conversation '=display-text *answer*))
           ((moss::%is-relation? obj-id)
            (send obj-id '=get-documentation :lead "RELATION: " 
                  :final-new-line t)
            (send conversation '=display-text *answer*))
           )
          )
        ;; otherwise print failure
        (throw :return 
               `(:transitions ,_pcd-sorry))
        ;(send conversation '=display-text 
        ;      "~%D?sol?, je n'ai pas de concept ni de propri?t? correspondant ~
        ;       ? votre demande.~%")
        )
      ;; success
      (list :success))))

;;;------------------------------------------------------------------- (PCD) SORRY

(defstate 
  _pcd-sorry
  (:label "Reference search failure.")
  (:explanation "We did not find any explanation for the reference.")
  (:reset)
  (:text "~%- D?sol?, je n'ai pas de concept ni de propri?t?  ~
          correspondant ? votre demande.~%." :no-erase)
  (:transitions 
   (:failure))
  )


;;;===============================================================================
;;;
;;;                       PRINT HELP CONVERSATION
;;;
;;;===============================================================================

;;; this conversation is intended to help the master by giving information:
;;;   - in general (what tasks are available
;;;      "what can you do for me?" "help." "what can I do?"
;;;   - on a particular topic
;;;      "how do I send a mail?" "help me with the mail?"
;;; specific help corresponding to special help tasks are caught by the task 
;;; task selection mechanism?

(defsubdialog 
  _print-help-conversation 
  (:label "Help conversation")
  (:explanation 
   "Help was asked.")
  (:states _ph-print-global-help)
  )


;;;-------------------------------------------------------- (PH) PRINT-GLOBAL-HELP

(defstate
  _ph-print-global-help
  (:label "Print general help")
  (:entry-state _print-help-conversation )
  (:explanation "No specific subject was included.")
  (:execute
   (let
     ((obj-id (car (send '>-global-help '=get 'is-title-of)))
      (*language* :fr)
      (task-list (access '(task)))
      (conversation (omas::conversation BENKEI::pa_BENKEI))
      )
     (when obj-id (send obj-id '=get-documentation)
           (send conversation '=display-text *answer*))
     (send conversation '=display-text
           "~2%Je peux faire les choses suivantes :")
     (dolist (task task-list)
       (send task '=get-documentation :lead "   - ")   
       (send conversation '=display-text *answer*)
       )
     (send conversation '=display-text "~2%")
     ))
  (:transitions
   (:always :success))
  )

;;;===============================================================================
;;;
;;;                         PROCESS ASK CONVERSATION
;;;
;;;===============================================================================

;;; this conversation is meant to post a message for inspection and send a  
;;; possible answer, or abort the corresponding task

;;; The problem is that the PA has received a request corresponding to the message
;;; being processed and the static part of the skill has posted the message and
;;; is waiting for the answer.
;;; One problem is that there are as many threads as posted messages waiting for  
;;; an answer.

;;; If we went to answer the request message, then the situation is quite complex:
;;; we have several processes:
;;;   - the thread executing the ask skill
;;;   - the converse process
;;;   - the assistant window thread handling the callbacks
;;;   - the timeout thread on the ask thread
;;; One can solve all the problems but the result is not simple.
;;;
;;; A better approach is to consider that all messages ASk and TELL are considered
;;; as INFORM messages, and do not create any specific process (i.e. waiting for a
;;; master's answer), ? la e-mail. This allows to send answers directly as INFORM
;;; messages, bypassing the complex mechanism of getting the control back to the
;;; ASK skill.
;;; Thus, all request messages triggering the ASK skill must send a response like
;;; "message well received, transmitted to the master." 

;;; In the simplified approach the role of the PROCESS ASK conversation is to 
;;; answer the message when the PROCESS (TRAITER) button is clicked.
;;; The sequence of events is:
;;;   - the ":process-ask" tag is inserted into the agent TO-DO slot
;;;   - the converse process wakes up and triggers the PROCESS ASK dialog
;;;   - a message is posted to the master
;;; If the master wants to answer then
;;;   - the receiver is first determined
;;;   - the *pass-every-char* flag (that should be a specific slot of the 
;;;     interface window) is set
;;;   - the master inputs text
;;;   - the DONE (Termin?) button is clicked resetting *pass-every-char* 
;;;   - the text is transferred into the :input area of FACTS
;;;   - the message is sent to the receiver directly as an inform message

(defsubdialog 
  _process-ask-conversation
  (:label "Process ask dialog")
  (:explanation "Display the content of a question message (ask).")
  (:states    _pa-entry-state  ; required by defstate
              _pa-brush ; clean up lightly
              _pa-destroy ; destroy message return :abort to ask-static
              _pa-discard? ; ask whather we dump the question or not
              _pa-get-answer ; get the answer from the master
              )
  )

;;;========================== service functions ==================================

;;;----------------------------------------------------- (PA) DISPLAY-ASK-MESSAGES

(defUn display-ask-message (message conversation)
  "takes a message and displays its content into the assistant pane"
  (let (sender content)
    ;; get sender identity
    (setq sender (omas::from! message))
    ;; get content of the message
    (setq content (car (omas::args message)))
    ;; ask to print it
    (send conversation '=DISPLAY-TEXT 
          (format nil "From : ~A ~%Priority : ~A ~%Object : ~A ~2% ~A"
                  sender
                  (or (cadr (member :priority content)) "Normal")
                  (or (cadr (member :object content)) "")
                  (or (getf content :text) "<empty message>")))))

;;;===============================================================================

;;;-------------------------------------------------------------- (PA) ENTRY-STATE
;;; We enter the dialog when the PROCESS (Traiter) button has been clicked. 
;;; FACTS/INPUT contains ":proces-ask" and message is in FACTS/TODO-MESSAGE

(defstate 
  _pa-entry-state
  (:label "process ask dialog")
  (:entry-state _process-ask-conversation)
  (:explanation "Shows the message and ask if an answer is needed.")
  (:execute-preconditions
   ;; the :input area contains a pair (<message> . <process-id>)
   (display-ask-message (read-fact moss::conversation :todo-message)
                        moss::conversation))
  (:question-no-erase "~2%Do you want to answer this message?")
  (:transitions 
   (:yes :target _pa-get-answer) 
   (:otherwise :target _pa-discard?))
  )

;;;-------------------------------------------------------------------- (PA) BRUSH
;;; reset conversation slot
(defstate 
  _pa-brush
  (:label "bookkeeping")
  (:explanation "do some light bookkeeping.")
  (:execute (replace-fact moss::conversation :todo-message nil))
  (:transitions
   (:always :success))
  )

;;;------------------------------------------------------------------ (PA) DESTROY

(defstate 
  _pa-destroy
  (:label "Destroy ASK message.")
  (:explanation "Master does not want to answer the message nor keep it. We ~
                 remove it from the list and return to the task with an :abort~
                 mark, so that it will exit without answering.")
  (:transitions
   (:always
    :exec 
    (let ((agent (car (has-agent moss::conversation))))
      ;; discard the message from the list, assuming it is still selected
      (omas::ASSISTANT-DISCARD-SELECTED-TO-DO-TASK agent)
      ;; clean up conversation
      (replace-fact moss::conversation :todo-message nil))
    :success))
  )

;;;----------------------------------------------------------------- (PA) DISCARD?

(defstate 
  _pa-discard?
  (:label "ask whether we throw the message away.")
  (:explanation "Master did not answer message, ask master if we keep it to it ~
                 later.")
  (:question-no-erase "~2%Do you want to keep the message pour lfor later processing?")
  (:transitions
   (:no :target _pa-destroy)
   (:otherwise :target _pa-brush))
  )

;;;--------------------------------------------------------------- (PA) GET-ANSWER

(defstate
  _pa-get-answer
  (:label "obtain master's answer to an ask message.")
  (:explanation "Record text typed into the master pane until the DONE button is ~
                 pushed. We must disable . and ? terminations.")
  ;; set global so that we can have . and ? in the text
  (:execute-preconditions 
   (let ((win (car (has-input-window moss::conversation))))
     (when win
       (setf (omas::pass-every-char win) t)))
   )
  (:question-no-erase "~% ...Type or tell your answzer then click the end button.") 
  (:answer-analysis)
  )

(defownmethod =answer-analysis _pa-get-answer (conversation input)
  "We process the answer and send it back unless we get an abort.
Arguments:
   conversation: conversation object is
   input: copy of the :input area of FACTS"
  (let (agent message content object)
    (setq agent (car (has-agent conversation)))
    ;(print  `(=====> input ,input))
    ;; if input is different from :abort we build a fuller answer
    (unless (eql input :abort)
      (setq message (read-fact conversation :todo-message)
            content (car (omas::args message))
            object (getf content :object))
      ;; format an answer content adding object to text
      (setq input (list :object (format nil "RE: ~A" (or object " "))
                        :text (read-fact conversation :raw-input)))
      ;; make an answer and send it back as an INFORM message
      (send-message 
       (make-instance 'omas::message
         :from :BENKEI
         :to (omas::from! message)
         :date (get-universal-time)
         :type :inform
         :action :tell
         :args (list input)
         )
       )
      )
    ;; clean FACTS
    (replace-fact conversation :todo-message nil)
    ;; remove message from the tasks to do pane
    (omas::ASSISTANT-DISCARD-SELECTED-TO-DO-TASK agent)
    )
  ;; we then get out of dialog
  '(:success))

;;;===============================================================================
;;;
;;;                         PROCESS TELL CONVERSATION
;;;
;;;===============================================================================

;;; this conversation is meant to post a message for inspection and quit

(defsubdialog 
  _process-tell-conversation
  (:label "Process tell dialog")
  (:explanation "Display the content of an info message (tell).")
  (:states _pt-entry-state  ; required by defstate
           _pt-erase? ; go clean the message
           _pt-clean ; remove item from list
           _pt-brush ; clean up lightly
           )
  )

;;;========================== service functions ==================================

;;;---------------------------------------------------- (PT) DISPLAY-TELL-MESSAGES

(defUn display-tell-message (message conversation)
  "takes a message and displays its content into the assistant pane"
  (let (sender content)
    ;; get sender identity
    (setq sender (omas::from! message))
    ;; get content of the message
    (setq content (car (omas::args message)))
    ;; ask to print it
    (send conversation '=DISPLAY-TEXT 
          (format nil "From : ~A ~%Priority : ~A ~%Object : ~A ~2% ~A"
                  sender
                  (or (cadr (member :priority content)) "Normal")
                  (or (cadr (member :object content)) "")
                  (or (getf content :text) "<empty message>")))))

;;;=========================================================?=====================
;;;                        PROCESS TELL CONVERSATION STATES
;;;===============================================================================

;;;-------------------------------------------------------------- (PT) ENTRY-STATE

(defstate 
  _pt-entry-state
  (:label "Process tell dialog")
  (:entry-state _process-tell-conversation)
  (:explanation "Affiche un message et demande si on le garde.")
  (:execute (display-tell-message 
             (car (read-fact moss::conversation :message)) MOSS::Conversation))
  (:question-no-erase "~2%OK? Do I erase the message?")
  (:transitions 
   (:yes :target _pt-clean) 
   (:otherwise :target _pt-brush))
  )

;;;-------------------------------------------------------------------- (PT) BRUSH

(defstate 
  _pt-brush
  (:label "bookkeeping")
  (:explanation "remove item from facts.")
  (:execute (replace-fact moss::conversation :message nil))
  (:transitions
   (:always :success))
  )

;;;-------------------------------------------------------------------- (PT) CLEAN

(defstate 
  _pt-clean
  (:label "Cleaning state")
  (:explanation "Ask whether we keep the message in the to-do pane.")
  (:execute (omas::ASSISTANT-DISCARD-SELECTED-TO-DO-TASK 
             (car (has-agent moss::conversation)))
            (replace-fact moss::conversation :message nil))
  (:transitions
   (:always :success))
  )

;;;===============================================================================
;;;
;;;                      PUBLISH NEWS CONVERSATION
;;;
;;;===============================================================================

(defsubdialog 
  _publish-new-conversation
  (:label "Publish new conversation")
  (:explanation 
   "Master is trying to publish a new.")
  (:states    _publish-new-entry-state
              _pnd-get-category
              _pnd-get-content
              _pnd-get-sections
              _pnd-publish
              _pnd-sorry)
  )

;;;------------------------------------------------------------- (PND) ENTRY STATE

(defstate 
  _publish-new-entry-state
  (:entry-state _publish-new-conversation)
  (:label "Publish new dialog entry")
  (:explanation "Ask for the title of the new.")  
  (:question-no-erase "~%- Object?")
  (:answer-type :answer)
  (:execute 
   (omas::remember PA_BENKEI (moss::read-fact moss::conversation :input)
     	           :news-title))
  (:transitions
   (:always  :target _pnd-get-category)))

;;;------------------------------------------------------------ (PND) GET-CATEGORY

(defstate 
  _pnd-get-category
  (:label "Ask master the content.")
  (:explanation "Obtain title, ask for category")
  ;(format t "~S~%" (moss::read-fact moss::conversation :input))
  ;(format t "~% TITULO A GUARDAR: ~S" (moss::read-fact moss::conversation :input))
  ;; Store title in agent´s short term memory  OMARAGP PORQUE ESTO NO FUNCIONA  
  (:question-no-erase "~%- Under which topic do you want to publish the news?")
  (:execute 
   (omas::remember PA_BENKEI (moss::read-fact moss::conversation :input)
     	           :news-section))
  (:transitions
   (:patterns ((?* ?x) "topics" (?* ?y))
              :target _pnd-get-sections)
   (:otherwise  :save :target _pnd-get-content)))

;;;------------------------------------------------------------- (PND) GET-CONTENT

(defstate 
  _pnd-get-content
  (:label "Ask master the content.")
  (:explanation "Obtain category, ask for content")
  
  (:execute-preconditions 
   (setf (omas::pass-every-char (car (has-input-window moss::conversation))) t)
   ;; include :verbatim flag into the FACT base (?)
   (replace-fact moss::conversation :verbatim t)
   )
  (:question-no-erase "~%- Ecrivez le contenu de la nouvelle et puis cliquez sur terminer")
  (:answer-type :answer)
  (:execute 
   ;; Store category in agent´s short term memory						   
   (remember PA_BENKEI (moss::read-fact moss::conversation :input)
     	     :news-content))
  (:transitions
   (:always  :target _pnd-publish)))

;;;------------------------------------------------------------ (PND) GET-SECTIONS

(defstate 
  _pnd-get-sections
  (:label "Ask NEWS about sections.")
  (:explanation "Master does not know sections. Asking NEWS for the list.")
  (:send-message :to :NEWS :self PA_BENKEI :action :get-all-sections
                 :args '((:language . :en)))
  (:execute 
   (omas::assistant-display-text PA_BENKEI
	                         (format nil "~S" (moss::read-fact moss::conversation :answer))))
  (:transitions
   (:on-failure :target _pnd-sorry)
   (:otherwise
    :target _pnd-get-category)))

;;;----------------------------------------------------------------- (PND) PUBLISH

(defstate 
  _pnd-publish
  (:label "Ask master the content.")
  (:explanation "Ask NEWS to create the news")
  (:send-message :to :NEWS :self PA_BENKEI :action :publish-new
                 ;; Recall gets data stored un ALBERT´s short term memory
                 :args `(((:title . ,(recall PA_BENKEI :news-title))
                          (:section . ,(recall PA_BENKEI :news-section))
                          (:content . ,(recall PA_BENKEI :news-content))
                          (:author  "BENKEI")
                          (:language . :en))))
  (:execute (print (car (read-fact moss::conversation :input))))
  (:transitions
   (:on-failure :target _pnd-sorry)
   (:otherwise :success))
  )

;;;--------------------------------------------------------------- (PND) PND-SORRY

(defstate 
  _pnd-sorry
  (:label "Publish failure.")
  (:explanation "It was an error while publishing the new.")
  (:text "Sorry, the NEWS service agent could not publish your text.~%")
  (:reset)
  (:transitions (:failure))
  )

;;;===============================================================================
;;;
;;;                       SET FONT SIZE CONVERSATION
;;;
;;;===============================================================================

;;; this conversation is intended to modify the size of the dialog font, usually 
;;; to increase it or to return to the default value.

(defsubdialog 
  _set-font-size-conversation
  (:short-name sfs)
  (:label "Font size conversation")
  (:explanation 
   "The data is searched for the presence of words like increase, ~
    bigger, smaller, defaults, reset,...")
  (:states _sfs-entry-state  ; required by defstate
           _sfs-dont-understand
           _sfs-try-again
           _sfs-sorry)
  )

;;;--------------------------------------------------- (SFS) FONT-SIZE-ENTRY-STATE

(defstate 
  _sfs-entry-state
  (:entry-state _set-font-size-conversation)
  (:label "Font size dialog entry")
  (:explanation "The data is searched for the presence of words like increase, ~
                 bigger, defaults, reset,...")
  (:transitions
   (:patterns (((?* ?x) "increase" (?* ?y))
               ((?* ?x) "bigger" (?* ?y))
               ((?* ?x) "larger" (?* ?y)))
              :exec (omas::assistant-use-larger-font PA_BENKEI) :success)
   (:patterns (((?* ?x) "decrease" (?* ?y))
               ((?* ?x) "smaller" (?* ?y)))
              :exec (omas::assistant-reset-font PA_BENKEI) :success)
   (:patterns (((?* ?x) "use" "default" (?* ?y))
               ((?* ?x) "reset" (?* ?y))
               ((?* ?x) "defaults" (?* ?y)))
              :exec (omas::assistant-reset-font PA_BENKEI) :success)
   (:otherwise :target _sfs-dont-understand)
   )
  )

;;;--------------------------------------------------------- (SFS) DONT-UNDERSTAND

(defstate 
  _sfs-dont-understand
  (:label "Did not understand. Ask master.")
  (:explanation "Assistant is asking master for font-size clarification.")
  (:question "~%- Do you want a bigger font size, a smaller one, or use the default?")
  (:transitions
   (:always :target _sfs-try-again)))

;;;------------------------------------------------------------------- (sfs) SORRY

(defstate 
  _sfs-sorry
  (:label "Font dialog failure.")
  (:explanation "We cannot make things out from the data.")
  (:reset)
  (:answer "~%- Sorry I do not understand what you want me to do with the font?.~%")
  (:transitions
   (:always :failure))
  )

;;;--------------------------------------------------------------- (sfs) TRY-AGAIN

;;; The simplest use is to take text in the HAS-DATA slot of the Q-CONTEXT
;;; and send a message to :SA-ADDRESS

(defstate 
  _sfs-try-again
  (:label "Font size try again")
  (:explanation "we asked master and try to pick up the answer.")
  (:transitions
   (:patterns (((?* ?x) "bigger" (?* ?y))
               ((?* ?x) "larger" (?* ?y)))
              :exec (omas::assistant-use-larger-font PA_BENKEI) :success)
   (:patterns (((?* ?x) "smaller" (?* ?y)))
              :exec (omas::assistant-reset-font PA_BENKEI) :success)
   (:patterns (((?* ?x) "default" (?* ?y))
               ((?* ?x) "defaults" (?* ?y)))
              :exec (omas::assistant-reset-font PA_BENKEI) :success)
   (:otherwise :target _sfs-sorry)
   )
  )

;;;===============================================================================
;;;
;;;                           TELL CONVERSATION
;;;
;;;===============================================================================

;;; this conversation is intended to tell something to a remote PA.
;;; We must first locate our receiver (presumably the PA of somebody in the 
;;; coterie. We then want to prepare topic and text, the send the message

(defsubdialog 
  _tell-conversation 
  (:label "Tell conversation")
  (:explanation 
   "We want our PA to tell something to somebody.")
  (:states    _tell-entry-state  ; required by defstate 
              _tell-ask-destination
              _tell-choose-destination 
              _tell-get-data
              _tell-get-destination
              _tell-process-destination
              _tell-send-message
              _tell-sorry
              )
  )

;;;===============================================================================
;;;                       TELL CONVERSATION SERVICE FUNCTIONS
;;;===============================================================================

;;;----------------------------------------------------------- EXTRACT-DESTINATION
;;; See ASK dialog
;;;===============================================================================
;;;                            TELL CONVERSATION STATES
;;;===============================================================================

;;;------------------------------------------------------- (TELL) TELL-ENTRY-STATE

(defstate 
  _tell-entry-state
  (:label "Tell dialog entry")
  (:entry-state _tell-conversation)
  (:explanation "We try to get destination from the input data.")
  (:transitions
   ;; normally the following test never happens
   (:test (null (read-fact moss::conversation :input)) 
          :target _tell-ask-destination)
   (:patterns
    (((?* ?X) "tell" (?* ?Y)) 
     ((?* ?X) "inform" (?* ?Y))
     ((?* ?X) "let" (?* ?Y) "know" (?* ?Z))
     ((?* ?x) "send" (?* ?Z) "to" (?* ?Y))
     ((?* ?X) "message" "for" (?* ?Y))
     ((?* ?X) "message" "to" (?* ?Y))
     )
    :keep ?Y
    :target _tell-process-destination)
   (:otherwise :target _tell-ask-destination))
  )

;;;-------------------------------------------------------- (TELL) ASK-DESTINATION

(defstate
  _tell-ask-destination
  (:label "Tell ask destination")
  (:explanation "ask to whom we send message")
  (:question-no-erase 
   "~% To whom do you want to send the message?")
  (:transitions (:always :target _tell-process-destination))
  )

;;;----------------------------------------------------- (TELL) CHOOSE-DESTINATION

(defstate 
  _tell-choose-destination
  (:label "Tell choose destination")
  (:explanation "ambiguous destination (list) select one.")
  (:execute
   (let* ((conversation moss::conversation)
          (choices (read-fact conversation :to-choose))
          )
     ;; remove the temporary data
     (replace-fact conversation :to-choose nil)
     (replace-fact conversation :to (car choices))))
  (:transitions (:always :target _tell-get-data)))

;;;--------------------------------------------------------------- (TELL) GET-DATA

(defstate 
  _tell-get-data
  (:label "set up parameters for continuous input.")
  (:explanation "Let the master type text into the pane until the DONE button is ~
                 pushed. We must disable . and ? terminations.")
  ;; set global flag in the interface window so that we can have . and ? in the text
  (:execute-preconditions 
   (setf (omas::pass-every-char (car (has-input-window moss::conversation))) t)
   ;; include :verbatim flag into the FACT base (?)
   (replace-fact moss::conversation :verbatim t)
   )
  (:question-no-erase 
   "~% ...Type or tell your text, then click the DONE button.")
  (:transitions
   (:always :target _tell-send-message))
  )


;;;---------------------------------------------------- (TELL) PROCESS-DESTINATION

(defstate 
  _tell-process-destination
  (:label "Tell: Process destination")
  (:explanation "process input to find to whom we send message")
  (:execute (replace-fact moss::conversation :to 
                          (extract-destination (read-fact moss::conversation :input))))
  (:transitions
   (:test (null (read-fact moss::conversation :to)) :target _tell-sorry)
   (:test (listp (read-fact moss::conversation :to)) 
          :exec (replace-fact moss::conversation 
                              :to-choose (read-fact moss::conversation :to))
          :target _tell-choose-destination)
   (:otherwise
    :target _tell-get-data))
  )

;;;----------------------------------------------------------- (TELL) SEND-MESSAGE

(defstate 
  _tell-send-message
  (:label "Tell send final message")
  (:explanation "build and send the message.")
  (:send-message-no-wait 
   :to (pop-fact moss::conversation :to)
   :type :inform
   :action :tell
   :args (list (list :text (pop-fact moss::conversation :raw-input))))
  (:text "~&...Message was sent")
  (:transitions (:always :success)) ; return
  )

;;;------------------------------------------------------------------ (TELL) SORRY

(defstate 
  _tell-sorry
  (:label "Tell failure.")
  (:explanation "We don't have the needed parameters")
  (:text 
   "Sorry, I don't see to whom you want to send a message.~%")
  (:transitions (:always :failure))
  )

;;;===============================================================================
;;;
;;;                       TRACE CONVERSATION
;;;
;;;===============================================================================

;;; this conversation is intended to modify the size of the dialog font, usually 
;;; to increase it or to return to the default value.

(defsubdialog 
  _trace-conversation
  (:label "Trace conversation")
  (:explanation 
   "Command to toggle the dialog trace.")
  (:states _tr-entry-state)
  )

;;;-------------------------------------------------------- (TR) TRACE-ENTRY-STATE

(defstate 
  _tr-entry-state
  (:label "Trace dialog entry")
  (:entry-state _trace-conversation)
  (:explanation "We search the input for the trace command.")
  
  (:transitions
   ;; system commands
   (:patterns (((?* ?x) "show" "transitions" (?* ?y))
               ((?* ?x) "trace" "transitions" (?* ?y))
               ((?* ?x) "transitions" "on" (?* ?y))
               ((?* ?x) "print" "transitions" (?* ?y)))
              :exec (setq moss::*transition-verbose* t
                          moss::*traced-agent* PA_BENKEI) :success)
   (:patterns (((?* ?x) "no" "trace" (?* ?y))
               ((?* ?x) "kill" "trace" (?* ?y))
               ((?* ?x) "trace" "off" (?* ?y))
               ((?* ?x) "stop" "trace" (?* ?y))
               ((?* ?x) "no" "verbose" (?* ?y)))
              :exec (setq moss::*verbose* nil
                          moss::*transition-verbose* nil
                          moss::*traced-agent* nil) :success)
   (:patterns (((?* ?x) "trace" "on" (?* ?y))
               ((?* ?x) "set" "trace" (?* ?y))
               ((?* ?x) "start" "trace" (?* ?y))
               ((?* ?x) "verbose" (?* ?y)))
              :exec (setq moss::*verbose* t
                          moss::*traced-agent* PA_BENKEI) :success)
   (:patterns (((?* ?x) "kill" "transitions" (?* ?y))
               ((?* ?x) "hide" "transitions" (?* ?y))
               ((?* ?x) "transitions" "off" (?* ?y))
               ((?* ?x) "do" "not" "show" "transitions" (?* ?y)))
              :exec (setq moss::*transition-verbose* nil) :success)
   (:otherwise :failure)
   )
  )

;;;===============================================================================
;;;
;;;                      WHAT-IS CONVERSATION
;;;
;;;===============================================================================

;;; this conversation is intended to obtain and print information about something
;;; corresponding to unformatted data

;;;(defParameter *empty-words* 
;;;  '("the" "a" "an" "of" "from" "to" "those" "these" "that" "s" "?"
;;;    "le" "la" "les" "de" "du" "des" #+MCL "?" #+MICROSOFT-32 "?" "aux" 
;;;    "ce" "ces"))

;;;===============================================================================

(defsubdialog 
  _what-is-conversation
  (:label "Get info dialog")
  (:explanation "Master is trying to get info about some data.")
  (:states    _wai-entry-state
              _wai-ask-staff
              _wai-get-info
              _wai-process-staff-answer
              _wai-sorry)
  )

;;;------------------------------------------------------------- (WAI) ENTRY-STATE

(defstate 
  _wai-entry-state
  (:label "What is conversation")
  (:entry-state _what-is-conversation)
  (:explanation "Assistant is trying to get object info from its knowledge base.")
  (:transitions
   ;;keep only relevant data
   (:patterns (((?* ?x) "what" "is" (?* ?y) "?")
               ((?* ?y) "is" "what" "?" (?* ?x))
               ((?* ?y))  ; when entering from print-concept-documentation
               )
              :keep ?y
              :target _wai-get-info)
   (:otherwise :target _wai-sorry)
   )
  )

;;;--------------------------------------------------------------- (WAI) ASK-STAFF
;;; changing strategy to :collect-answers
;;; rather than sending a broadcast right away, maybe we should
;;; 1. look into local KB
;;; 2. look into our agenda (sa-address)
;;; 3. ask BIBLIO
;;; This would avoid strange answers resulting from misinterpretation of the
;;; query.

(defstate 
  _wai-ask-staff
  (:label "Ask staff.")
  (:explanation "Ask the staff for help.")
  (:send-message :to :all :self PA_BENKEI :action :what-is :timeout 2
                 :args `(((:data . ,(read-fact moss::conversation :input))
                          (:language . :en)
                          ))
                 :strategy :collect-answers
                 )
  (:transitions 
   (:always :target _wai-process-staff-answer))
  )

;;;---------------------------------------------------------------- (WAI) GET-INFO

(defstate 
  _wai-get-info
  (:label "Looking into local KB info.")
  (:explanation "Look into own KB to get info.")
  (:answer-analysis)
  )

(defownmethod =answer-analysis _wai-get-info (conversation input)
  "we try first to get info locally and if it fails ask all agents.
Arguments:
   agent: assistant."
  (declare (special *empty-words*))
  (let* (entry-list)
    (catch :return
      ;; clean input of parasitic words
      (setq input (mapcan #'(lambda (xx) 
                              (if (member xx *empty-words* :test #'string-equal)
                                nil
                                (list xx)))
                          input))
      (moss::vformat "BENKEI: _wai-get-info /+++ cleaned input:~& ~S" input)
      ;; if nothing left, then go ask staff
      (unless input
        (throw :return `(:transition ,_wai-ask-staff)))
      ;; check whether the resulting words correspond to entry points
      (setq entry-list (moss::find-best-entries input))
      (moss::vformat "BENKEI: _wai-get-info /entry list:~&    ~S" entry-list)
      ;; if nothing left go ask staff
      (unless entry-list 
        (throw :return `(:transition ,_wai-ask-staff)))
      ;; if more than one result, ask for help (not yet)
      ;; get documentation or CV
      ;; print it
      (dolist (item entry-list)
        (send item '=get-documentation :lead "   - ")   
        (send conversation '=display-text *answer*)
        )
      ;; transfer to :success    
      (list :success) 
      )
    ))

;;;---------------------------------------------------- (WAI) PROCESS-STAFF-ANSWER

(defstate 
  _wai-process-staff-answer
  (:label "Process staff answer.")
  (:explanation "Process anwers from the staff agents.")
  (:answer-analysis)
  )

(defownmethod =answer-analysis _wai-process-staff-answer (conversation input)
  "we got an answer from some staff or else the master got impatient.
Arguments:
   agent: assistant."
  (declare (ignore input))
  (let* ((agent (car (has-agent conversation)))
         (answer-list (omas::answer agent))
         (*language* :en)
         )
    ;(format *debug-io* "~&+++ =resume; HAS-TO-DO: ~S" (has-to-do conversation))
    ;; clean up HAS-TO-DO (contains "answer-there")
    ;(setf (has-to-do conversation) nil)
    ;; answe? contains either the word "failure" or a list of lists of strings
    ;; (("Jean-Paul Barthes is a professor at UTC ..."
    ;;  "Dominique Barthes-Biesel is a professor at UTC ...")
    ;;  ("Jean-Paul Barthes has NN publications on:  ...")
    ;;  ...)
    ;; 
    (moss::vformat "~&>>> WAIdialog /Answer to the send-request:~% ~S" answer-list)
    ;; failure if :failure is returned or :error on timeout
    (cond
     ((or (equal "failure" answer-list) (eql :error answer-list))
      `(:transition ,_wai-sorry))
     ((null answer-list)
      (send conversation '=display-text "- I don't know...")
      (list :success))
     (t 
      (moss::vformat " _wai-ask-staff /answer-list: ~& ~S" answer-list)
      (send conversation '=display-text "~%It is:")
      ;; each item of the answer list is like 
      ;;    ("personne" ("nom" "...")("pr?nom" "..."))
      (dolist (answer answer-list)
        ;; answer could be "failure" rather than a list of strings
        (unless (and (stringp answer) (string-equal answer "failure"))
          (dolist (item answer)
            (send conversation '=display-text 
                  (concatenate 'string "~%" (car item)))
            (send conversation '=display-text
                  (format nil ": ~{~%     ~{~A~^, ~}~}" (cdr item))))))
      (list :success)))
    ))

;;;------------------------------------------------------------------- (WAI) SORRY

(defstate 
  _wai-sorry
  (:label "Failure to get info.")
  (:explanation "We could not find anything.")
  (:reset)
  (:text "~%- Sorry, I could not find information for the data.~%")
  (:transitions 
   (:failure))
  )

;;;===============================================================================
;;;
;;;                    WHO IS (GET PERSON INFO) CONVERSATION
;;;
;;;===============================================================================

;;; this conversation is intended to print information about a person.

(defsubdialog 
  _get-person-info-conversation
  (:label "Get person info dialog")
  (:explanation "Master is trying to obtain a telephone number.")
  (:states    _gpi-entry-state
              _gpi-ask-staff
              _gpi-get-info
              _gpi-sorry)
  )
;;;------------------------------------------------------------ CHECK-ENTRY-POINTS

(defUn check-entry-points (input)
  "tries to get entry points by combining words.
Arguments:
   input: list of one or more words
Return:
   nil or a list like 
     (\"barthes\" ($T-PERSON-NAME.OF . $E-PERSON.1) ($T-PERSON-NAME.OF . $E-PERSON.2))"
  (let (result entry entry-list expr-list)
    ;; combine all possible words
    (setq expr-list (make-word-combinations input))
    ;; try all possibilities
    (dolist (expr expr-list)
      ;; entry point ?
      (when (setq entry (moss::%entry? expr))
        ;; extract corresponding objects
        (setq result (moss::%%get-objects-from-entry-point entry))
        ;; return to the executing environment
        (moss::vformat "...trying ~S in package ~S as entry...~&   => ~S" 
                       expr *package* result)
        (if result (setq entry-list (append entry-list result)))
        ))
    ;; clean list, eliminating tasks whose performative is different of the required
    ;; one
    (setq entry-list (delete-duplicates entry-list :test #'equal))
    (moss::vformat "check-entry-points /Resulting entry list ~&  => ~S" entry-list)
    (mapcar #'cdr entry-list)))

#|
? (BENKEI::check-entry-points '("telephone" "barthes" ))
;***** ...trying "telephone" in package #<Package "SA-ADDRESS"> as entry...
   => ((MOSS::$ENAM.OF . $E-PHONE))
;***** ...trying "barthes" in package #<Package "SA-ADDRESS"> as entry...
   => (($T-PERSON-NAME.OF . $E-PERSON.2) ($T-PERSON-NAME.OF . $E-PERSON.3)
       ($T-PERSON-NAME.OF . $E-PERSON.4))
;***** check-entry-points /Resulting entry list 
  => ((MOSS::$ENAM.OF . $E-PHONE) ($T-PERSON-NAME.OF . $E-PERSON.2)
      ($T-PERSON-NAME.OF . $E-PERSON.3) ($T-PERSON-NAME.OF . $E-PERSON.4))
($E-PHONE $E-PERSON.2 $E-PERSON.3 $E-PERSON.4)
|#
;;;-------------------------------------------------------- MAKE-WORD-COMBINATIONS

(defUn make-word-combinations (word-list)
  ;; when empty list or only one word return
  (unless (cdr word-list) (return-from make-word-combinations word-list))
  ;; otherwise build word combinations
  (let ((input-length (length word-list)) ; e.g. 5
        result)
    (dotimes (nn (1+ input-length)) ; e.g. nn: 0 -> 4
      ;; build cursor to move through the list
      (dotimes (jj nn) ; e.g. jj: 0, then jj: 0->1
        (push
         (apply #'moss::%make-phrase 
                (subseq word-list jj (+ (- input-length nn) jj 1)))
         result)))
    (reverse result)))

#|
? (make-word-combinations '("a" "b" "c" "d"))
("a b c d" "a b c" "b c d" "a b" "b c" "c d" "a" "b" "c" "d")
? (moss::%make-name-for-entry "a b c")
A-B-C
|#
;;;===============================================================================

(defstate 
  _gpi-entry-state
  (:label "Get person info conversation")
  (:entry-state _get-person-info-conversation)
  (:explanation "Assistant is trying to get the info from its knowledge base.")
  (:transitions
   ;;keep only relevant data
   (:patterns (((?* ?x) "who" "is" (?* ?y) "?")
               ((?* ?x) "is" "who" (?* ?y))
               )
              :keep ?y
              :target _gpi-get-info)
   (:otherwise :target _gpi-sorry)
   )
  )

;;;--------------------------------------------------------------- (WOI) ASK-STAFF
;;; changing strategy to :collect-answers
;;; rather than sending a broadcast right away, maybe we should
;;; 1. look into local KB
;;; 2. look into our agenda (ADDRESS)
;;; 3. ask BIBLIO
;;; This would avoid strange answers resulting from misinterpretation of the
;;; query.

(defstate 
  _gpi-ask-staff
  (:label "Ask other agents for help.")
  (:explanation "Ask the staff for help.")
  (:send-message :to :all :self PA_BENKEI :action :who-is :timeout 4
                 :args `(((:data . ,(read-fact moss::conversation :input))
                          (:language . :en)
                          ))
                 :strategy :collect-answers
                 )
  (:transitions
   ;; can't use :on-failure that test the FACTS/input area
   ;; answers to messages are put into the FACTS/answer area
   (:test (null (read-fact moss::conversation :answer)) :target _gpi-sorry)
   (:otherwise
    :exec (dolist (answer (read-fact moss::conversation :answer))
            (dolist (item answer)
              (send moss::conversation '=display-text item)))
    :success))
  )

;;;---------------------------------------------------------------- (WOI) GET-INFO

(defstate 
  _gpi-get-info
  (:label "Looking into local KB info.")
  (:explanation "Look into own KB to get answer to who-is?.")
  (:answer-analysis)
  )

(defownmethod =answer-analysis _gpi-get-info (conversation input)
  "we got an answer from some staff or else the master got impatient.
Arguments:
   conversation: the conversation object
   input: a copy of the FACTS/input area"
  (let* (entry-list)
    (catch :return
      ;; clean input of parasitic words
      (setq input (mapcan #'(lambda (xx) 
                              (if (member xx *empty-words* :test #'string-equal)
                                nil
                                (list xx)))
                          input))
      (moss::vformat "BENKEI: _gpi-get-info /+++ cleaned input:~& ~S" input)
      ;; if nothing left, then ask staff
      (unless input
        (throw :return `(:transition ,_gpi-ask-staff)))
      ; check whether the resulting words correspond to entry points
      (setq entry-list (check-entry-points input))
      (moss::vformat "BENKEI: _gpi-get-info /entry list:~& ~S" entry-list)
      ;; if nothing left go ask staff
      (unless entry-list 
        (throw :return `(:transition ,_gpi-ask-staff)))
      ;; if more than one person ask for help (not yet)
      ;; get documentation or CV
      ;; print it
      (dolist (item entry-list)
        (send item '=get-documentation)
        (send conversation '=display-text *answer*))
      ;; transfer to :success    
      (list :success) 
      )
    ))

;;;------------------------------------------------------------------- (WOI) SORRY

(defstate 
  _gpi-sorry
  (:label "Sorry: no info.")
  (:explanation "We could not find anything.")
  (:reset)
  (:text ("~%- Sorry, I have not the slightest idea. Please check the spelling.~%"
          "- I don't know. Please check the spelling.~%"))
  (:transitions 
   (:failure))
  )

;;;===============================================================================

;;;===============================================================================
;;;
;;;                             ELIZA CONVERSATION
;;;
;;;===============================================================================

;;; this conversation is intended to do small talk.

(defParameter *eliza-rules*
  '((("hello")
     "こんにちは．私は弁慶です?")
    (("こんにちは")
     "こんにちは．私は弁慶です?")
    (("もしもし")
     "なにか御用ですか?")
    (("はじめまして")
     "Enchanté, je suis BENKEI."
     "How do you do, I am BENKEI.")
    (((?* ?x) "の" "体調" "は" "どうですか")
     "?xはとても元気です")
    (((?* ?x) "の" (?* ?y) "は" "どうですか")
     "?xの?yは晴れです")
    (("Hi" (?* ?x))
     "なにか御用ですか")
    (("bonjour")
     "Je suis désolé mon français n'est pas encore utilisable. Please use English.")
    (("how" "are" "you" (?* ?x))
     "Very well thank you."
     "Really does my health concern you? I am flattered.")
    (("who"  (?* ?x) "Barthès")
     "Jean-Paul Barth?s is a professor of artificial intelligence at the ~
      Universit? de Technologie de Compi?gne in France.")
    (("who"  (?* ?x) "Barthes")
     "Jean-Paul Barthès is a professor of artificial intelligence at the ~
      Université de Technologie de Compiègne in France.
      Note however that Barthès is written with an è.")
    (("forget" "it")
     "OK. No hard feelings.")
    (("thank" "you" (?* ?y))
     "you are welcome."
     "don't mention it.")
    (("thanks")
     "you are welcome."
     "don't mention it.")
    (((?* ?x) "computer" (?* ?y))
     "Do computer worry you?"
     "What do you think about machines?"
     "Why do you mention computers?"
     "What do you think machines have to do with your problem?")
    (((?* ?x) "stupid" (?* ?y))
     "Please, behave!"
     "I am pained by your remark.")
    (((?* ?x) "does" "not" "work" (?* ?y))
     "Of course ?x works!"
     "What are  you saying that ?x does not work?")
    (("help")
     "you want help about what?")
    (((?* ?x) "sorry" (?* ?y))
     "Please don't apologize"
     "Apologies are not necessary"
     "What feelings do you have when you apologize?")
    (((?* ?x) "I" "remember" (?* ?y))
     "Do you often think of ?y"
     "Does thinking of ?y bring anything else to mind?"
     "What else do you remember?"
     "Why do you recall ?y right now?"
     "What in the present situation reminds you of ?y ?"
     "What is the connection between me and ?y ?")
    (((?* ?x) "do" "you" "remember" (?* ?y))
     "Did you think I would forget ?y ?"
     "Why do you think I should recall ?y by now ?"
     "What about ?y ?"
     "You mentined ?y ?")
    (((?* ?x) "if" (?* ?y))
     "Do you really think its likely that ?y ?"
     "Do you wish that ?y ?"
     "What do you think about ?y ?"
     "Really -- if ?y ?")
    (("what" (?* ?x) "do" "for" "me")
     "So many things..."
     "Currently I do not know."
     "What could please you?")
    
    (((?* ?x) "I" "dreamt" (?* ?y))
     "Really -- ?y"
     "Have you ever fantasized ?y while you were awake ?"
     "Have you dreamt ?y before ?")
    (((?* ?x) "dream" "about" (?* ?y))
     "How do you feel about ?y in reality ?")
    (((?* ?x) "dream" (?* ?y))
     "What does this dream suggest to you ?"
     "Do you dream often ?"
     "What persons appear in your dreams ?"
     "Don't you believe that dream has to do with your problem ?")
    (((?* ?x) "my" "mother" (?* ?y))
     "Who else in your family ?y ?"
     "Tell me more about your family")
    (((?* ?x) "my" "father" (?* ?y))
     "Your father ?"
     "Does he influence you strongly ?"
     "What else comes to mind when you think of your father ?")
    (((?* ?x) "I" "want" (?* ?y))
     "What would it mean if you got ?y ?"
     "Why do you want ?y ?"
     "Suppose you got ?y soon ?")
    (((?* ?x) "I" "am" "glad" (?* ?y))
     "How have I helped you to be ?y ?"
     "What makes you happy just now ?"
     "Can you explain why you are suddenly ?y ?")
    (((?* ?x) "I" "am" "sad" (?* ?y))
     "I am sorry to hear you are depressed"
     "I am sure it's not pleasant to be sad")
    (((?* ?x) "are" "like" (?* ?y))
     "What resemblance do you see between ?x and ?y ?")
    (((?* ?x) "is" "like" (?* ?y))
     "In what way is it that ?x is like ?y ?"
     "What resemblance do you see ?"
     "Could there really be some connection ?"
     (How ?))
    (((?* ?x) "alike" (?* ?y))
     "In what way ?"
     "What similarities are there ?")
    (((?* ?x) "BENKEIe" (?* ?y))
     "What other connections do you see ?")
    
    (((?* ?x) "I" "was" (?* ?y))
     "Were you really ?"
     "Perhaps I already knew you were ?y"
     "Why do you tell me you were ?y by now ?")
    (((?* ?x) "was" "I" (?* ?y))
     "What if you were ?y ?"
     "Do you think you were ?y ?"
     "What would it meant if you were ?y ?")
    (((?* ?x) "I" "am" (?* ?y))
     "In what way are you ?y ?"
     "Do you want to be ?y ?")
    (((?* ?x) "am" "I" (?* ?y))
     "Do you believe you are ?y ?"
     "Would you want to be ?y ?"
     "You wish I would tell you you are ?y"
     "What would it mean if you were ?y ?")
    (((?* ?x) "am" (?* ?y))
     "Why do you say \"AM?\""
     "I don't understand that")
    (((?* ?x) "are" "you" (?* ?y))
     "Why are you interested in whether I am ?y or not ?"
     "Would you prefer if I weren't ?y ?"
     "Perhaps I am ?y in your fantasies")
    (((?* ?x) "you" "are" (?* ?y))
     "What makes you think I am ?y ?")
    (((?* ?x) "because" (?* ?y))
     "Is that the real reason ?"
     "What other reasons might there be ?"
     "Does that reason seem to explain anything else ?")
    (((?* ?x) "were" "you" (?* ?y))
     "Perhaps I was ?y"
     "What do you think ?"
     "What if I had been ?y ?")
    (((?* ?x) "I" "can't" (?* ?y))
     "Maybe you could ?y now"
     "What if you could ?y ?")
    (((?* ?x) "I" "feel" (?* ?y))
     "Do you often feel ?y ?")
    (((?* ?x) "I" "felt" (?* ?y))
     "What other feelings do you have ?")
    (((?* ?x) "I" (?* ?y) "you" (?* ?z))
     "Perhaps in your fantasy we ?y each other")
    (((?* ?x) "why" "don't" "you" (?* ?y))
     "Should you ?y yourself ?"
     "Do you believe I don't ?y ?"
     "Perhaps I will ?y in good time")
    (((?* ?x) "yes" (?* ?y))
     "You seem quite positive."
     "You are sure ?"
     "I understand.")
    (((?* ?x) "no" (?* ?y))
     "Why not ?"
     "You are being a bit negative"
     "Are you saying \"NO\" just to be negative?")
    
    (((?* ?x) "someone" (?* ?y))
     "Can you be more specific ?")
    (((?* ?x) "everyone" (?* ?y))
     "Surely not everyone."
     "Con you think of anyone in particular ?"
     "Who for example ?"
     "You are thinking of a special person ?")
    (((?* ?x) "always" (?* ?y))
     "Can you think of a specific example ?"
     "When ?"
     "What incident are you thinking of ?"
     "Really -- always ?")
    (((?* ?x) "perhaps" (?* ?y))
     "You do not seem quite certain")
    (((?* ?x) "are" (?* ?y))
     "Did you think there might not be ?y ?"
     "Possibly they are ?y")
    (((?* ?x))
     "I do not understand exactly what you want. You may type: help."
     "I am not sure I understand what you want. You may type: help."
     "Could you rephrase your question please?"
     "Sorry I did not get that. Could you rephrase it? Or type: help."
     "Oops I do not understand. Could you say that differently?"
     "Sorry?"
     )))


(mformat "~%;*** MOSS v~A - BENKEI dialogs loaded ***" moss::*moss-version-number*)

;;;===============================================================================

:EOF