﻿;;;-*- Mode: Lisp; Package: "ORIBE" -*-
;;;===============================================================================
;;;09/08/02
;;;                               AGENT ORIBE
;;;
;;;===============================================================================

#|
ORIBE has the following skills:
	- ASK INFO 
	    internal task that sends a subtask, puts the result into
	    FACT/answer and "answer there" into FACTS/input.
	- CN TEST
	    Sends an :hello request to everybody to see who is around using
		contract net (check with graphics trace)
	- GET ADDRESS 
	    Skill to answer an external request of another PA that wants to 
		obtain an address.
	- GET PHONE NUMBER
	    Skill to answer an external request of another PA that wants to 
		obtain a telephone number.
	- HELLO 
	    Atomic test function, returns a greeting string.
	- SHOW ONTOLOGY
	    Displays a window with an ontology during 20s.
	- TEST
	    Sends an :hello request to everybody to see who is around (check with
		graphics trace)
2007
 0802 created by cloning STEVENS
|#

(defpackage :ORIBE (:use :cl #+MCL :ccl :moss :omas))
(in-package :ORIBE)

;; Can display characters only if locale is Japanese and input is Japanese

(omas::defassistant :ORIBE :redefine t :show-dialog t 
         ;:font "Arial Unicode MS / ShiftJIS" :size 11)
         :font "Arial Unicode MS" :size 11)

(format t "~%;*** Loading ORIBE file ***")

;;;===============================================================================
;;; 
;;;                         service macros and functions 
;;;
;;;===============================================================================


;;; ==================================== globals =================================
;;; Globals  can be used within the agent space to simplify programming, although
;;; it is better to use the agent memory.


;;;===============================================================================
;;; 
;;;                                    skills 
;;;
;;;===============================================================================

;;;========================================================================= skill
;;;                                   ASK INFO
;;;===============================================================================

(defskill :ask-info :ORIBE
  :static-fcn static-ask-info
  :dynamic-fcn dynamic-ask-info
  :time-limit-fcn time-limit-hello
  :timeout-handler timeout-handler-ask-info
  )

(defun static-ask-info (agent message to my-action args)
  "internal task that sends a subtask
Arguments:
   agent:
   message
   to: whom we should ask (default :all)
   my-action: predicate
   args: arguments of the predicate."
  (declare (ignore message))
  (send-subtask agent :to (or to :all) :action my-action :args args)
  (static-exit agent :done))

(defun dynamic-ask-info (agent message result)
  "receiving the result of the request. Put it into the results slot of the agent ~
   conversation."
  (declare (ignore message))
  (let* ((conversation (omas::conversation agent)))
    (if conversation
     (progn
        (replace-fact conversation :answer result)
        (replace-fact conversation :raw-input "answer-there")
        )
      ;; otherwise indicate failure
      (error "no conversation active for ORIBE"))
    (dynamic-exit agent result :internal t)))

(defun timeout-handler-ask-info (agent message)
  "documentation"
  (declare (ignore agent message))
  (error "timeout"))

;;;========================================================================= skill
;;;                                   GET ADDRESS
;;;===============================================================================

;;; Skill to answer an external request of another PA that wants to obtain an
;;; address. 

(defskill :get-address :ORIBE
  :static-fcn static-get-address
  :dynamic-fcn dynamic-get-address
  )

(defun static-get-address (agent message data)
  "the skill is triggered by a message from an agent wanting to obtain address ~
   information.
Arguments:
   data: an a-list, e.g. ((:data \"dominique\" \"barthès\")(:pattern ...))
Return:
   \"failure\" or a list of addresses."
  (declare (ignore message))
  (when data
    (send-subtask  agent :to :CIT-address :action :get-address 
                   :args (list data)))
  (static-exit agent :done))

(defun dynamic-get-address (agent message result)
  "return result to calling agent"
  (declare (ignore message))
  ;(print result)
  (dynamic-exit agent result))

;;;========================================================================= skill
;;;                              GET PHONE NUMBER
;;;===============================================================================

;;; Skill to answer an external request of another PA that wants to obtain a
;;; telephone number 

(defskill :get-phone-number :ORIBE
  :static-fcn static-get-phone-number
  :dynamic-fcn dynamic-get-phone-number
  )

(defun static-get-phone-number (agent message data)
  "the skill is triggered by a message from an agent wanting to obtain phone-number ~
   information.
Arguments:
   data: an a-list, e.g. ((:data \"dominique\" \"barthès\")(:pattern ...))
Return:
   \"failure\" or a list of phone-numbers."
  (declare (ignore message))
  (when data
    (send-subtask  agent :to :CIT-address :action :get-phone-number 
                   :args (list data)))
  (static-exit agent :done))

(defun dynamic-get-phone-number (agent message result)
  "return result to calling agent"
  (declare (ignore message))
  ;(print result)
  (dynamic-exit agent result))

;;;========================================================================= skill
;;;                               SHOW ONTOLOGY
;;;===============================================================================
;;; internal skill to post an ontology browser

(defskill :show-ontology :ORIBE
  :static-fcn static-show-ontology
  )

(defun static-show-ontology (agent message)
  "task that displays an ontology browser"
  (declare (ignore message))
  (moss::make-overview-window)
  (omas::acl/mcl-process-wait-with-timeout "showing ontology" 20 #'null t)
  (static-exit agent :done))

;;;========================================================================= skill
;;;                                   TEST
;;;===============================================================================
;;; sends an :hello request to everybody to see who is around (graphics trace)

(defskill :test :ORIBE
  :static-fcn static-test
  :dynamic-fcn dynamic-test
  )

(defun static-test (self message)
  "greeting function"
  (declare (ignore message))
  (send-subtask  self :to :all :action :hello :strategy :collect-answers)
  (static-exit self "Hello there..."))

(defun dynamic-test (self message result)
  (declare (ignore message))
  (dynamic-exit self result))

;;;========================================================================= skill
;;;                                 CN TEST
;;;===============================================================================

(defskill :cn-test :ORIBE
  :static-fcn static-cn-test
  :dynamic-fcn dynamic-cn-test
  )

(defun static-cn-test (self message)
  "greeting function"
  (declare (ignore message))
  (send-subtask  self :to :all :action :hello :protocol :contract-net 
                 :strategy :collect-answers)
  (static-exit self "Hello there..."))

(defun dynamic-cn-test (self message result)
  (declare (ignore message))
  (dynamic-exit self result))

;;;===============================================================================
;;; 
;;;                               initial conditions 
;;;
;;;===============================================================================


:ENV

;;;==============================================================================
;;; 
;;;                                    goals
;;;
;;;==============================================================================


:EOF