﻿;;;===============================================================================
;;;11/03/03
;;;                AGENT TRAVEL-SERVICE
;;;                Barthès, Copyright@UTC, 2004-2011
;;;===============================================================================

(defpackage :travel-service (:use :cl #+MCL :ccl :moss :omas))
(in-package :travel-service)

(omas::defagent :travel-service)

;;; =============================== skill section ==============================

(defskill :hotel-info-request :travel-service
  :static-fcn static-hotel-info-request
  :dynamic-fcn dynamic-hotel-info-request
  :select-best-answer-fcn best-answer-hotel-info-request
  )

(defun static-hotel-info-request (agent message request)
  "when receiving a request the agent broadcasts it to objects that can answer ~
      an :info-request"
  (declare (ignore message))
  ;; request contains the needed parameters. e.g. (:price :lowest :breakfast :yes)
  ;; Keep them in the task environment
  (env-set agent (getf request :price) :price)
  (env-set agent (getf request :breakfast) :breakfast) 
  (env-set agent (getf request :distance) :distance) 
  ;; they will be used for selecting the best answer
  (format t "~%; static-hotel-info-request /request: ~S" request)
  ;; broadcast demand
  (send-subtask agent 
                :to :all
                :action :info-request
                :args '(:dummy)
                :strategy :collect-answers
                )
  (static-exit agent nil))

(defun dynamic-hotel-info-request (agent message result)
  "called when an answer returns"
  (declare (ignore message))
  ;; return the first answer that comes back
  (format t "~&Possible offers: ~S" result)
  (dynamic-exit agent result))

;(defvar *ml*)
(defun  best-answer-hotel-info-request (agent message-list)
  "selects the best answer for the travel agent, with default values."
  (declare (ignore agent))
  ;(format t "~&;=== bahir; environment:~&~S" environment)
  ;(setq *ml* message-list)
  ;(format t "~&;=== best-answer-hotel-info-request; message-list:~{~& - ~S~}" 
  ;        message-list)
  (let ((breakfast (env-get agent :breakfast))
        (distance (env-get agent :distance))
        (price (env-get agent :price))
        info ref)
    ;; extract a list with answering agent and content
    (setq info (mapcar #'(lambda (xx) (cons (omas::from! xx) (omas::contents xx)))
                       message-list))
    ;; when breakfast is specified remove entries are not suitable
    (when breakfast
      (setq info (remove-if #'(lambda (xx) 
                                (not (eql breakfast (cadr (member :breakfast xx)))))
                            info)))
    ;; when price condition, get the best-price hotel
    (when price
      (case price
        (:lowest
         (setq info (sort info #'< :key #'(lambda (xx) (cadr (member :price xx)))))
         ;; get reference price AFTER the sort!
         (setq ref (cadr (member :price (car info))))
         ;; remove entries more expensive than the first one
         ;; remove those that are too expensive
         (setq info (remove-if #'(lambda (xx) (> (cadr (member :price xx)) ref)) 
                               info)))
        (:highest
         (setq info (sort info #'> :key #'(lambda (xx) (cadr (member :price xx)))))
         ;; get reference price
         (setq ref (cadr (member :price (car info))))
         ;; remove those that are too expensive
         (setq info (remove-if #'(lambda (xx) (< (cadr (member :price xx)) ref)) 
                               info)))
        ))
    ;; when distance is important, then sort on the distance
    (when distance
      (case distance
        (:shortest
         (setq info (sort info #'< :key #'(lambda (xx) (cadr (member :distance xx)))))
         ;; set ref AFTER the sort!
         (setq ref (cadr (member :distance (car info))))
         ;; remove those that are too far
         (setq info (remove-if #'(lambda (xx) (> (cadr (member :distance xx)) ref)) 
                               info)))
        (:longest
         (setq info (sort info #'> :key #'(lambda (xx) (cadr (member :distance xx)))))
         ;; set ref
         (setq ref (cadr (member :distance (car info))))
         ;; remove those that are not far enough
         (setq info (remove-if #'(lambda (xx) (< (cadr (member :distance xx)) ref)) 
                               info)))
        ))
    ;; return a list containing a message (required) and a set of best offers
    (list (car message-list) info)))


;;; ================================= environment ==============================


:EOF