﻿;;;-*- Mode: Lisp; Package: "XXXXX" -*-
;;;===============================================================================
;;;14/12/03
;;;              ENGLISH ASSISTANT: XXXXX (file: XXXXX-EN.lisp)
;;;                       Copyright, Barthès@UTC, 2014
;;;
;;;===============================================================================

;;; this file is used to create a service agent, by replacing the 5 Xs
;;; by the target name and saving the file in the same folder and the 5 Ms by a
;;; description of the master e.g.
;;;  ("personne" ("nom" "ramos")("prénom" "milton")("assistant personnel" "ze"))
;;; used by omas::create-assistant

;;; skills are:
;;;  - GET-LANGUAGE
;;;  - PROCESS-WEB-TEXT

#|
2014
 0918 creation
|#

(defpackage :XXXXX (:use :cl :moss :omas))
(in-package :XXXXX)

(format t "~%;*** Loading XXXXX file ***")

(omas::defassistant :XXXXX :language :en :no-window t
   :ontology-file "SHARED-ONTOLOGY-EN" :tasks-file "SHARED-TASKS-EN"
   :dialog-file "SHARED-DIALOG-EN" :master YYYYY)

;;;(omas::defagent :XXXXX)

;;;===============================================================================
;;; 
;;;                     globals, service macros and functions 
;;;
;;;===============================================================================

;;; ==================================== globals =================================
;;; Globals  can be used within the agent space to simplify programming, although
;;; it is better to use the agent memory.

;;;===============================================================================
;;; 
;;;                                    SKILLS 
;;;
;;;===============================================================================

;;;========================================================================= SKILL
;;;                             GET-LANGUAGE
;;;===============================================================================
;;; ask for the PA language

(defskill :get-language XXXXX
  :static-fcn static-get-language)

(defun static-get-language (agent message &optional args)
  "read the laguage slot and returns it. If nil, sets it to :EN"
  (declare (ignore message args))
  (unless (omas::language agent)
    (setf (omas::language agent) :en))
  (static-exit agent (omas::language agent)))

;;;======================================================================== skill
;;;                          :PROCESS-WEB-TEXT
;;;==============================================================================
;;; this skill extracts the text from the data, sets up a new gate, tells the
;;; agent that the request comes from the web, puts the text into the to-do-slot
;;; of the agent, waits with a timeout for the text to be processed, recovers the
;;; answer from the conversation FACTS area and returns it to the caller (:WEB)
;;; ending a dialog turn.

(defskill :PROCESS-WEB-TEXT XXXXX
   :static-fcn static-process-web-text)
   
(defun static-process-web-text (agent message args)
  "function processes the data received by the web server to send the text to the
   dialog process.
Arguments:
   agent: this agent
   message: incoming message
   args: ((:data ...)) contains the text to be processed
Return:
   returns the answer to the caller as an a-list 
      ((:text ...)(:question ...)(:answer ...))"
  (declare (ignore message))
  (format t "~%;--- args: ~S" args)
  (let* ((text (cdr (assoc :data args)))
         (timeout (cdr (assoc :timeout args)))
         conversation gate answer)
    
    ;; check if converse process still exists (not crashed during conversation)
    (unless (and (omas::assistant-process agent)
                 (mp:process-active-p (omas::assistant-process agent)))
      (format t "~%;--- ~S: We restart a conversation process" (omas::name agent))
      ;; restart converse process
      (omas::start-converse-process agent))
    
    ;; just in case it started a new conversation loop
    (setq conversation (omas::conversation agent))
    
    ;;=== here we start a new conversation turn
    (web-clear-text conversation)
    
    ;; save the gate index into the conversation object, and put a :web mark
    (web-set-mark conversation)
    ;; set up a new gate to synchronize with the result, returns an index
    ;; new gate information is save onto the omas.web::*web-gate-list*
    (setq gate (omas::web-create-gate))
    ;; save it in the conversation object
    (web-set-gate conversation gate)
    (format t "~%; ~S/ static-process-web-text/ gate to synchronize conversation: ~S"
      (omas::name agent) gate)
    ;; put text into the to-do slot of the agent (reviving the conversation process)
    (setf (omas::to-do agent) text)
    ;; and wait with timeout
    (unwind-protect
        (when
            ;; the function waits until the gate is opened or until timeout (default 30s)
            ;; the function returns nil on timeout
            (mp:process-wait-with-timeout "waiting web processing" timeout 
                                          #'mp:gate-open-p 
                                          (omas::web-get-gate-data gate))
          ;; answer is recovered from the FACTS
          ;;;
          (format t "~%~S; / static-process-web-text /no timeout, ~
                     conversation:~%  ~%~{~&   ~S~}"
            (omas::name agent) (has-moss-facts conversation))
          (setq answer (web-get-text conversation))
          (format t "~%; ~S/ static-process-web-text /no timeout, answer:  ~s"
            (omas::name agent) answer)
          ;; if FACTS contains :verbatim with a T value, then add a :verbatim
          ;; property to answer to get all chars on input
          (if (read-fact conversation :verbatim)
              (push (cons :verbatim t) answer))
          
          (web-clear-text conversation)
          ;; the answer may be a string or a list (for calling an edit page)
          (cond
           ((stringp answer)
            ;; if it is a string we send it with a text property
            (setq answer `((:text . ,answer))))
           (t
            ;; otherwise send it as data
            (setq answer `((:data . ,answer)))))
          
          (format t "~%; ~S/ static-process-web-text /no timeout, answer:  ~S"
            (omas::name agent) answer)
          ;(break "UNTEL: conversation: ~S" conversation)
          ;; reset conversation parameters
          (web-clear-tag conversation)
          (web-clear-text conversation)
          )
      
      ;; unwind forms always executed
      ;; close gate again for next time around (actually kills it)
      (omas::web-close-gate gate))
    
    ;; quit, return answer to caller (e.g. WEBNEWS)
    (static-exit agent (or answer :failure))))

;;;===============================================================================
;;; 
;;;                               initial conditions 
;;;
;;;===============================================================================
;;; remove comment when voice interface is present. Not sure this is useful...

;#+MICROSOFT-32 (omas::net-initialize-voice XXXXX::PA_XXXXX)

:EOF
