﻿;;;-*- Mode: Lisp; Package: "PATIENT" -*-
;;;===============================================================================
;;;11/04/25
;;;             AGENT PATIENT: DIALOG (file PATIENT-dialog.lisp)
;;;             Copyright Barthes@UTC, 2011
;;;
;;;=============================================================================== 

;;; Note that this file must be in UTF-8 format (like the other agent files)
;;; The top-level dialog works in English, French, Portuguese (Brasilian), or Japanese
;;; For any other language the questions and answers must be translated and added
;;; to the already existing ones.

#|
PATIENT is a therapist assistant agent. Its has a single main dialog.
 
2011
 0423 creation
|#

(in-package :PATIENT)

;; this is used by the Japanese language, can be removed if not needed
(eval-when (:load-toplevel :compile-toplevel :execute)
  (push 'はい moss::*yes-answers*)
  (push 'OK moss::*yes-answers*)
  (push 'いいえ moss::*no-answers*)
  )
  
;;;================================================================================
;;;
;;;                         PATIENT macros.
;;;
;;;================================================================================

(defmacro clean-topics ()
  `(post-topics pa_patient (omas::com-window pa_patient) 
       '("" "" "" "" "" "" "" "")))
	   
(defmacro show-topics (topic-list)
  `(post-topics pa_patient (omas::com-window pa_patient) ',topic-list))
  
;;;================================================================================
;;;================================================================================
;;;
;;;                       PATIENT CONVERSATION (CONTROL LOOP)
;;;
;;;================================================================================
;;;================================================================================

;;; create a dialog header to be used for the main conversation

(defindividual 
  MOSS-DIALOG-HEADER
  (HAS-MOSS-LABEL "Main conversation loop")
  (HAS-MOSS-EXPLANATION "This is the main conversation loop.")
  (:var _main-conversation))

;; declare that conversation, replacing default conversation in the agent structure
(setf (omas::dialog-header PA_PATIENT) _main-conversation)

;;;------------------------------------------------------------- (DOC) ENTRY-STATE

(defstate
  _doc-entry-state
  (:label "PATIENT dialog entry")
  (:entry-state _main-conversation )
  (:explanation "PATIENT is starting session.")
  ;(:execute-preconditions
  ;    (setf (omas::answer-timeout pa_patient) 10))
  (:question "Hello!")
  (:transitions
    (:always :target _doc-get-name))
  ) 
  
;;;--------------------------------------------------------------------- (DOC) END

(defstate
  _doc-end
  (:label "Closing state")
  (:explanation "PATIENT is ending session.")
  (:question "Nice talking with you. Will do that later another time. Bye...")
  (:transitions
    (:always :success))
  )

;;;---------------------------------------------------------------- (DOC) GET-NAME 

(defstate
  _doc-get-name
  (:label "Get name")
  (:explanation "PATIENT is asking patient's name.")
  (:question "What is your name?")
  (:transitions
    ;; failure case must come before the rules for some reason...
    (:on-failure :exec 
	   (progn (send moss::conversation '=display-text "Nice meeting you."
	                :erase t)
	          (sleep 1.5))
	     :target _doc-post-topics)
    (:rules
	   ((((?* ?x) "my" "name" "is" (?* ?y))
	     "Hi, ?y. Nice meeting you.")
	    (((?* ?x) "my" "friends" "call" "me" (?* ?y))
	     "Hi, ?y. Nice meeting you.")
	    (((?* ?x) "call" "me" (?* ?y))
	     "Hi, ?y. Nice meeting you.")
	    (((?* ?y))
		 "Hi, ?y. Nice meeting you."))
		:display-result :erase :target _doc-post-topics)
	)
  )

;;;----------------------------------------------------------------- (DOC) NO-VISIT

(defstate
  _doc-no-visit
  (:label "No visit")
  (:explanation "Patient did not answer or answered negatively.")
  (:question "Ah, have you been,...?")
  (:transitions
     (:always :target _doc-end))
  )
  
;;;------------------------------------------------------------- (DOC) POST-TOPICS

(defstate 
  _doc-post-topics
  (:label "Post topics")
  (:explanation "PA is posting some topics.")
  (:execute-preconditions
      (post-topics pa_patient (omas::com-window pa_patient)
	      '("Palace" "Nijo" "Kiyomizu")))
  (:question-no-erase "~%Now we are going to talk using topics displayed on the screen. Have you ever been to any place listed here? If so please select it.")
  (:transitions
    (:patterns
	  (((?* ?x) "palace" (?* ?y))) 
	    :sub-dialog _palace-conversation :success _doc-end)
	(:patterns
	  (((?* ?x) "nijo" (?* ?y))) :sub-dialog _nijo-conversation)
	(:patterns
	  (((?* ?x) "kiyomizu" (?* ?y))) :sub-dialog _kiyomizu-conversation)
    (:otherwise :target _doc-no-visit)
   ))

;;;-------------------------------------------------------------------- (DOC) TEST

(defstate
  _doc-test
  (:label "Test")
  (:explanation "PATIENT is asking patient's name.")
  (:question "Test for rules...")
  (:transitions
    (:rules
	  ((((?* ?x) "does" "not" "work" (?* ?y))
       "Of course ?x works!"
       "Why are you saying that ?x does nor work?"))
	  :display-result :erase :target _doc-get-name)
    (:otherwise :success))
  )
  
;;;================================================================================
;;;
;;;                            PALACE CONVERSATION
;;;
;;;================================================================================

(defsubdialog 
  _palace-conversation 
  (:label "Palace conversation")
  (:explanation "We talk about the palace.")
  (:states _pal-entry-state  ; required by defstate
           _pal-do-not-know
		   _pal-show-topics
		   )
  )
  
;;;------------------------------------------------------------- (PAL) ENTRY-STATE

(defstate
  _pal-entry-state
  (:label "Entering palace topic")
  (:entry-state _palace-conversation )
  (:explanation "Topic is Palace.")
  (:question "Palace is a nice place to visit don't you think so?")
  (:transitions
    (:always :target _pal-show-topics))
  ) 
  
;;;-------------------------------------------------------------- (PAL) DO-NOT-KNOW

(defstate
  _pal-do-not-know
  (:label "Patient does not know topics")
  (:explanation "Patient did not answer or answered negatively.")
  (:question "Ah, maybe we can tackle these topics later on...")
  (:transitions
     (:always :exec (clean-topics) :success))
  )
;;;------------------------------------------------------------- (PAL) SHOW-TOPICS

(defstate
  _pal-show-topics
  (:label "Show palace topics")
  (:explanation "Doctor is selecting a list of topics to show patient.")
  (:execute-preconditions
      (post-topics pa_patient (omas::com-window pa_patient)
	      '("Sakura" "Emperor" "Tea ceremony")))
  (:question "Sakura, Emperor, Tea ceremony: Do you know something about these topics?")
  (:transitions
    ; (:patterns
	  ; (((?* ?x) "sakura" (?* ?y)) :sub-dialog _sakura-conversation)
	  ; (((?* ?x) "emperor" (?* ?y)) :sub-dialog _emperor-conversation)
	  ; (((?* ?x) "tea" "ceremony" (?* ?y)) 
	           ; :sub-dialog _tea-ceremony-conversation)
	  ; )
    (:otherwise :target _pal-do-not-know))
  )
  
;;;================================================================================
;;;
;;;                            NIJO CONVERSATION
;;;
;;;================================================================================

(defsubdialog 
  _nijo-conversation 
  (:label "Nijo conversation")
  (:explanation "We talk about the Nijo castle.")
  (:states _nij-entry-state  ; required by defstate
           _nij-do-not-know
		   )
  )
  
;;;------------------------------------------------------------- (NIJ) ENTRY-STATE

(defstate
  _nij-entry-state
  (:label "Entering palace topic")
  (:entry-state _nijo-conversation )
  (:explanation "Topic is Nijo castle.")
  (:question "Nijo castle is a nice place to visit don't you think so?")
  (:transitions
    (:always :target _pal-do-not-know))
  ) 
  
;;;-------------------------------------------------------------- (NIJ) DO-NOT-KNOW

(defstate
  _nij-do-not-know
  (:label "Patient does not know topics")
  (:explanation "Patient did not answer or answered negatively.")
  (:question "Ah, maybe we can tackle these topics later on...")
  (:transitions
     (:always :exec (clean-topics):success))
  )
 ;;;================================================================================
;;;
;;;                            KIYOMIZU CONVERSATION
;;;
;;;================================================================================

(defsubdialog 
  _kiyomizu-conversation 
  (:label "Kiyomizu conversation")
  (:explanation "We talk about the Kiyomizu temple.")
  (:states _kiy-entry-state  ; required by defstate
           _kiy-do-not-know
		   )
  )
  
;;;------------------------------------------------------------- (KIY) ENTRY-STATE

(defstate
  _kiy-entry-state
  (:label "Entering kiyomizu topic")
  (:entry-state _kiyomizu-conversation )
  (:explanation "Topic is Kiyomizu temple.")
  (:question "Kiyomizu temple is a nice place to visit don't you think so?")
  (:transitions
    (:always :target _kiy-do-not-know))
  ) 
  
;;;-------------------------------------------------------------- (KIY) DO-NOT-KNOW

(defstate
  _kiy-do-not-know
  (:label "Patient does not know topics")
  (:explanation "Patient did not answer or answered negatively.")
  (:question "Ah, maybe we can tackle these topics later on...")
  (:transitions
     (:always :exec (clean-topics) :success))
  )
  

;;;=============================================================================== 


(format t "~%;*** MOSS v~A - PATIENT dialogs loaded ***" moss::*moss-version-number*):EOF