﻿;;;-*- Mode: Lisp; Package: "ARIBERTO" -*-
;;;==============================================================================
;;;13/11/21
;;;                              ARIBERTO
;;;                       Copyright, Fuckner@UTC, 2013
;;;
;;;==============================================================================

#|
2013
 1121 creation. used albert from hdsri as a source.
|#

(defpackage :ARIBERTO (:use :moss :omas :cl))
(in-package :ARIBERTO) 

(omas::defassistant :ARIBERTO :language :fr :no-window t)

;;;======================================================================== skill
;;;                          :PROCESS-WEB-TEXT
;;;==============================================================================
;;; this skill extracts the text from the data, sets up a new gate, tells the
;;; agent that the request comes from the web, puts the text into the to-do-slot
;;; of the agent, waits with a timeout for the text to be processed, recovers the
;;; answer from the conversation FACTS area and returns it to the caller (:WEB)
;;; ending a dialog turn.

(defskill :PROCESS-WEB-TEXT :ARIBERTO
  :static-fcn static-process-web-text)

(defun static-process-web-text (agent message args)
  "function processes the data received by the web server to send the text to the
   dialog process.
Arguments:
   agent: this agent
   message: incoming message
   args: ((:data ...)) contains the text to be processed
Return:
   returns the answer to the caller as an a-list 
      ((:text ...)(:question ...)(:answer ...))"
  (declare (ignore message))
  (let* ((text (cdr (assoc :data args)))
         (timeout (cdr (assoc :timeout args)))
         conversation gate answer)
    
    ;; check if converse process still exists (not crashed during conversation)
    (unless (and (omas::assistant-process agent)
                 (mp:process-active-p (omas::assistant-process agent)))
      ;; restart converse process
      (omas::start-converse-process agent))
    
    ;; just in case it started a new conversation loop
    (setq conversation (omas::conversation agent))
    
    ;;=== here we start a new conversation turn
    (web-clear-text conversation)
    
    ;; save the gate index into the conversation object, and put a :web mark
    (web-set-mark conversation)
    ;; set up a new gate to synchronize with the result
    (setq gate (mp:make-gate nil))
    ;; save it in the conversation object
    (web-set-gate conversation gate)
    (format t "~%; UNTEL/ static-process-web-text/ gate: ~S" gate)
    ;; put text into the to-do slot of the agent
    (setf (omas::to-do agent) text)
    ;; and wait with timeout
    (unwind-protect
        (when
            ;; the function waits until the gate is opened or until timeout (default 30s)
            ;; the function returns nil on timeout
            (mp:process-wait-with-timeout "waiting web processing" timeout 
                                          #'mp:gate-open-p 
                                          gate)
          ;; answer is recovered from the FACTS
          ;;;
          (format t "~%; UNTEL/ static-process-web-text /no timeout, ~
                     conversation:~%  ~%~{~&   ~S~}" (has-moss-facts conversation))
          (setq answer
                `((:text . ,(web-get-text conversation ))))
          (format t "~%; UNTEL/ static-process-web-text /no timeout, answer:  ~S"
            answer)
          ;(break "UNTEL: conversation: ~S" conversation)
          ;; reset conversation parameters
          (web-clear-tag conversation)
          (web-clear-text conversation)
          )
      
      ;; unwind forms always executed
      ;; close gate again for next time around (actually kills it)
      (mp:close-gate gate))
    
    ;; quit 
    (static-exit agent (or answer :failure))))

;;;===============================================================================

:EOF